@@ -1,4 +1,5 @@
-/* Copyright (c) 2007-2012. The SimGrid Team. All rights reserved.                             */
+/* Copyright (c) 2007-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -63,7 +64,11 @@ int master(int argc, char *argv[]) {
     argv[0] = xbt_strdup(slavename);
     argv[1] = bprintf("%d",i);
     argv[2] = NULL;
-    msg_vm_t vm = MSG_vm_start(slaves[i],2);
+
+    char vmName[64];
+    snprintf(vmName, 64, "vm_%d", i);
+
+    msg_vm_t vm = MSG_vm_start(slaves[i],vmName,2);
     MSG_vm_bind(vm, MSG_process_create_with_arguments(slavename,slave_fun,NULL,slaves[i],2,argv));
   }
 
@@ -130,6 +135,9 @@ int master(int argc, char *argv[]) {
     MSG_task_send(finalize, mailbox_buffer);
   }
 
+  XBT_INFO("Wait a while before effective shutdown.");
+  MSG_process_sleep(2);
+
   xbt_dynar_foreach(vms,i,vm) {
     MSG_vm_shutdown(vm);
     MSG_vm_destroy(vm);
