@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -22,18 +22,16 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
-  m_gpu_task_t mytask = NULL;
+  msg_gpu_task_t mytask = NULL;
 
   mytask = MSG_gpu_task_create("testTask", 2000.0, 20.0, 20.0);
 
   XBT_INFO("GPU task %p was created", mytask);
 
-  MSG_clean();
-
   if (res == MSG_OK)
     return 0;
   else
