@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -21,7 +21,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int master(int argc, char *argv[]);
 int slave(int argc, char *argv[]);
 int timer(int argc, char *argv[]);
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 int timer_start = 1;
@@ -31,7 +31,7 @@ int timer_start = 1;
 int bool_printed = 0;
 double start_time, end_time, elapsed_time;
 double gl_data_size[NTASKS];
-m_task_t gl_task_array[NTASKS];
+msg_task_t gl_task_array[NTASKS];
 const char *slavenames[NTASKS];
 const char *masternames[NTASKS];
 int gl_task_array_id = 0;
@@ -44,7 +44,7 @@ int master(int argc, char *argv[])
 {
   char *slavename = NULL;
   double task_comm_size = 0;
-  m_task_t todo;
+  msg_task_t todo;
   char id_alias[10];
   //unique id to control statistics
   int id = -1;
@@ -117,7 +117,7 @@ int timer(int argc, char *argv[])
 int slave(int argc, char *argv[])
 {
 
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   int a;
   int id = 0;
 #ifdef HAVE_LATENCY_BOUND_TRACKING
@@ -201,10 +201,10 @@ int slave(int argc, char *argv[])
 }                               /* end_of_slave */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model", "GTNETS"); */
   /* MSG_config("workstation/model","KCCFLN05"); */
@@ -228,10 +228,10 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
   bool_printed = 0;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     exit(1);
@@ -239,8 +239,6 @@ int main(int argc, char *argv[])
 
   res = test_all(argv[1], argv[2]);
 
-  MSG_clean();
-
   if (res == MSG_OK)
     return 0;
   else
