@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -28,7 +28,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int sender(int argc, char *argv[]);
 int receiver(int argc, char *argv[]);
 
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 /** Sender function  */
@@ -46,7 +46,7 @@ int sender(int argc, char *argv[])
 
   msg_comm_t comm = NULL;
   int i;
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   MSG_process_sleep(sleep_start_time);
   for (i = 0; i < number_of_tasks; i++) {
     char mailbox[256];
@@ -67,8 +67,8 @@ int sender(int argc, char *argv[])
       while (MSG_comm_test(comm) == 0) {
         MSG_process_sleep(sleep_test_time);
       };
-      MSG_comm_destroy(comm);
     }
+    MSG_comm_destroy(comm);
 
   }
 
@@ -84,8 +84,8 @@ int sender(int argc, char *argv[])
       while (MSG_comm_test(comm) == 0) {
         MSG_process_sleep(sleep_test_time);
       };
-      MSG_comm_destroy(comm);
     }
+    MSG_comm_destroy(comm);
 
   }
 
@@ -96,8 +96,8 @@ int sender(int argc, char *argv[])
 /** Receiver function  */
 int receiver(int argc, char *argv[])
 {
-  m_task_t task = NULL;
-  _XBT_GNUC_UNUSED MSG_error_t res;
+  msg_task_t task = NULL;
+  _XBT_GNUC_UNUSED msg_error_t res;
   int id = -1;
   char mailbox[80];
   msg_comm_t res_irecv;
@@ -125,8 +125,8 @@ int receiver(int argc, char *argv[])
       while (MSG_comm_test(res_irecv) == 0) {
         MSG_process_sleep(sleep_test_time);
       };
-      MSG_comm_destroy(res_irecv);
     }
+    MSG_comm_destroy(res_irecv);
 
     XBT_INFO("Received \"%s\"", MSG_task_get_name(task));
     if (!strcmp(MSG_task_get_name(task), "finalize")) {
@@ -145,10 +145,10 @@ int receiver(int argc, char *argv[])
 }                               /* end_of_receiver */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model","KCCFLN05"); */
   {                             /*  Simulation setting */
@@ -169,16 +169,15 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
