@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -22,7 +22,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int sender(int argc, char *argv[]);
 int receiver(int argc, char *argv[]);
 
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 /** Sender function  */
@@ -35,7 +35,7 @@ int sender(int argc, char *argv[])
 
   msg_comm_t *comm = xbt_new(msg_comm_t, number_of_tasks + receivers_count);
   int i;
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   for (i = 0; i < number_of_tasks; i++) {
     char mailbox[256];
     char sprintf_buffer[256];
@@ -57,6 +57,8 @@ int sender(int argc, char *argv[])
   }
   /* Here we are waiting for the completion of all communications */
   MSG_comm_waitall(comm, (number_of_tasks + receivers_count), -1);
+  for (i = 0; i < number_of_tasks + receivers_count; i++)
+    MSG_comm_destroy(comm[i]);
 
   XBT_INFO("Goodbye now!");
   xbt_free(comm);
@@ -66,8 +68,8 @@ int sender(int argc, char *argv[])
 /** Receiver function  */
 int receiver(int argc, char *argv[])
 {
-  m_task_t task = NULL;
-  _XBT_GNUC_UNUSED MSG_error_t res;
+  msg_task_t task = NULL;
+  _XBT_GNUC_UNUSED msg_error_t res;
   int id = -1;
   char mailbox[80];
   msg_comm_t res_irecv;
@@ -80,6 +82,7 @@ int receiver(int argc, char *argv[])
     res_irecv = MSG_task_irecv(&(task), mailbox);
     XBT_INFO("Wait to receive a task");
     res = MSG_comm_wait(res_irecv, -1);
+    MSG_comm_destroy(res_irecv);
     xbt_assert(res == MSG_OK, "MSG_task_get failed");
     XBT_INFO("Received \"%s\"", MSG_task_get_name(task));
     if (!strcmp(MSG_task_get_name(task), "finalize")) {
@@ -98,10 +101,10 @@ int receiver(int argc, char *argv[])
 }                               /* end_of_receiver */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model","KCCFLN05"); */
   {                             /*  Simulation setting */
@@ -122,16 +125,15 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
