@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -22,7 +22,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int sender(int argc, char *argv[]);
 int receiver(int argc, char *argv[]);
 
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 /** Sender function  */
@@ -36,7 +36,7 @@ int sender(int argc, char *argv[])
   double coef = 0;
   xbt_dynar_t d = xbt_dynar_new(sizeof(msg_comm_t), NULL);
   int i;
-  m_task_t task;
+  msg_task_t task;
   char mailbox[256];
   char sprintf_buffer[256];
   msg_comm_t comm;
@@ -69,7 +69,7 @@ int sender(int argc, char *argv[])
   sprintf(mailbox, "finalize");
 
   msg_comm_t res_irecv;
-  _XBT_GNUC_UNUSED MSG_error_t res_wait;
+  _XBT_GNUC_UNUSED msg_error_t res_wait;
   for (i = 0; i < receivers_count; i++) {
     task = NULL;
     res_irecv = MSG_task_irecv(&(task), mailbox);
@@ -91,7 +91,7 @@ int receiver(int argc, char *argv[])
   char mailbox[80];
   xbt_dynar_t comms = xbt_dynar_new(sizeof(msg_comm_t), NULL);
   int tasks = atof(argv[2]);
-  m_task_t *task = xbt_new(m_task_t, tasks);
+  msg_task_t *task = xbt_new(msg_task_t, tasks);
 
   _XBT_GNUC_UNUSED int read;
   read = sscanf(argv[1], "%d", &id);
@@ -107,9 +107,9 @@ int receiver(int argc, char *argv[])
   }
 
   /* Here we are waiting for the receiving of all communications */
-  m_task_t task_com;
+  msg_task_t task_com;
   while (!xbt_dynar_is_empty(comms)) {
-    _XBT_GNUC_UNUSED MSG_error_t err;
+    _XBT_GNUC_UNUSED msg_error_t err;
     xbt_dynar_remove_at(comms, MSG_comm_waitany(comms), &res_irecv);
     task_com = MSG_comm_get_task(res_irecv);
     MSG_comm_destroy(res_irecv);
@@ -132,10 +132,10 @@ int receiver(int argc, char *argv[])
 }                               /* end_of_receiver */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model","KCCFLN05"); */
   {                             /*  Simulation setting */
@@ -156,16 +156,15 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
