@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -16,8 +16,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 
 int master(int argc, char *argv[]);
 int slave(int argc, char *argv[]);
-int forwarder(int argc, char *argv[]);
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 #define FINALIZE ((void*)221297)        /* a magic number to tell people to stop working */
@@ -26,7 +25,7 @@ MSG_error_t test_all(const char *platform_file,
 int master(int argc, char *argv[])
 {
   int slaves_count = 0;
-  m_host_t *slaves = NULL;
+  msg_host_t *slaves = NULL;
   int number_of_tasks = 0;
   double task_comp_size = 0;
   double task_comm_size = 0;
@@ -42,7 +41,7 @@ int master(int argc, char *argv[])
 
   {                             /* Process organisation */
     slaves_count = argc - 4;
-    slaves = xbt_new0(m_host_t, slaves_count);
+    slaves = xbt_new0(msg_host_t, slaves_count);
 
     for (i = 4; i < argc; i++) {
       slaves[i - 4] = MSG_get_host_by_name(argv[i]);
@@ -60,9 +59,9 @@ int master(int argc, char *argv[])
   XBT_INFO("Got %d task to process :", number_of_tasks);
 
   for (i = 0; i < number_of_tasks; i++) {
-    m_task_t task = MSG_task_create("Task", task_comp_size, task_comm_size,
+    msg_task_t task = MSG_task_create("Task", task_comp_size, task_comm_size,
                                     xbt_new0(double, 1));
-    int a;
+    msg_error_t a;
     *((double *) task->data) = MSG_get_clock();
 
     a = MSG_task_send_with_timeout(task,MSG_host_get_name(slaves[i % slaves_count]),10.0);
@@ -97,7 +96,7 @@ int master(int argc, char *argv[])
   XBT_INFO
       ("All tasks have been dispatched. Let's tell everybody the computation is over.");
   for (i = 0; i < slaves_count; i++) {
-    m_task_t task = MSG_task_create("finalize", 0, 0, FINALIZE);
+    msg_task_t task = MSG_task_create("finalize", 0, 0, FINALIZE);
     int a = MSG_task_send_with_timeout(task,MSG_host_get_name(slaves[i]),1.0);
     if (a == MSG_OK)
       continue;
@@ -131,7 +130,7 @@ int master(int argc, char *argv[])
 int slave(int argc, char *argv[])
 {
   while (1) {
-    m_task_t task = NULL;
+    msg_task_t task = NULL;
     int a;
     double time1, time2;
 
@@ -156,6 +155,8 @@ int slave(int argc, char *argv[])
       } else if (a == MSG_HOST_FAILURE) {
         XBT_INFO
             ("Gloups. The cpu on which I'm running just turned off!. See you!");
+        free(task->data);
+        MSG_task_destroy(task);
         return 0;
       } else {
         XBT_INFO("Hey ?! What's up ? ");
@@ -177,10 +178,10 @@ int slave(int argc, char *argv[])
 }                               /* end_of_slave */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model","KCCFLN05"); */
   {                             /*  Simulation setting */
@@ -201,16 +202,15 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
