@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int master(int argc, char *argv[]);
 int slave(int argc, char *argv[]);
 int forwarder(int argc, char *argv[]);
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 #define FINALIZE ((void*)221297)        /* a magic number to tell people to stop working */
@@ -35,8 +35,8 @@ MSG_error_t test_all(const char *platform_file,
 int master(int argc, char *argv[])
 {
   int slaves_count = 0;
-  m_host_t *slaves = NULL;
-  m_task_t *todo = NULL;
+  msg_host_t *slaves = NULL;
+  msg_task_t *todo = NULL;
   int number_of_tasks = 0;
   double task_comp_size = 0;
   double task_comm_size = 0;
@@ -53,7 +53,7 @@ int master(int argc, char *argv[])
   {                             /*  Task creation */
     char sprintf_buffer[64];
 
-    todo = xbt_new0(m_task_t, number_of_tasks);
+    todo = xbt_new0(msg_task_t, number_of_tasks);
 
     for (i = 0; i < number_of_tasks; i++) {
       sprintf(sprintf_buffer, "Task_%d", i);
@@ -65,7 +65,7 @@ int master(int argc, char *argv[])
 
   {                             /* Process organisation */
     slaves_count = argc - 4;
-    slaves = xbt_new0(m_host_t, slaves_count);
+    slaves = xbt_new0(msg_host_t, slaves_count);
 
     for (i = 4; i < argc; i++) {
       slaves[i - 4] = MSG_get_host_by_name(argv[i]);
@@ -93,7 +93,7 @@ int master(int argc, char *argv[])
   XBT_INFO
       ("All tasks have been dispatched. Let's tell everybody the computation is over.");
   for (i = 0; i < slaves_count; i++) {
-    m_task_t finalize = MSG_task_create("finalize", 0, 0, FINALIZE);
+    msg_task_t finalize = MSG_task_create("finalize", 0, 0, FINALIZE);
     MSG_task_send(finalize, MSG_host_get_name(slaves[i]));
   }
 
@@ -106,7 +106,7 @@ int master(int argc, char *argv[])
 /** Receiver function  */
 int slave(int argc, char *argv[])
 {
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   _XBT_GNUC_UNUSED int res;
   while (1) {
     res = MSG_task_receive(&(task),MSG_host_get_name(MSG_host_self()));
@@ -133,11 +133,11 @@ int forwarder(int argc, char *argv[])
 {
   int i;
   int slaves_count;
-  m_host_t *slaves;
+  msg_host_t *slaves;
 
   {                             /* Process organisation */
     slaves_count = argc - 1;
-    slaves = xbt_new0(m_host_t, slaves_count);
+    slaves = xbt_new0(msg_host_t, slaves_count);
 
     for (i = 1; i < argc; i++) {
       slaves[i - 1] = MSG_get_host_by_name(argv[i]);
@@ -150,7 +150,7 @@ int forwarder(int argc, char *argv[])
 
   i = 0;
   while (1) {
-    m_task_t task = NULL;
+    msg_task_t task = NULL;
     int a;
     a = MSG_task_receive(&(task),MSG_host_get_name(MSG_host_self()));
     if (a == MSG_OK) {
@@ -160,7 +160,7 @@ int forwarder(int argc, char *argv[])
             ("All tasks have been dispatched. Let's tell everybody the computation is over.");
         for (i = 0; i < slaves_count; i++)
           MSG_task_send(MSG_task_create("finalize", 0, 0, FINALIZE),
-        		  MSG_host_get_name(slaves[i]));
+              MSG_host_get_name(slaves[i]));
         MSG_task_destroy(task);
         break;
       }
@@ -180,10 +180,10 @@ int forwarder(int argc, char *argv[])
 }                               /* end_of_forwarder */
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* MSG_config("workstation/model","KCCFLN05"); */
   {                             /*  Simulation setting */
@@ -205,16 +205,15 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
