@@ -1,3 +1,10 @@
+/* Copyright (c) 2010-2012. The SimGrid Team.
+ * All rights reserved.                                                     */
+
+/* This program is free software; you can redistribute it and/or modify it
+ * under the terms of the license (GNU LGPL) which comes with this package. */
+
+
 /******************** Non-deterministic message ordering  *********************/
 /* Server assumes a fixed order in the reception of messages from its clients */
 /* which is incorrect because the message ordering is non-deterministic       */
@@ -5,6 +12,7 @@
 
 #include <msg/msg.h>
 #include <simgrid/modelchecker.h>
+
 #define N 3
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(example, "this example");
@@ -14,7 +22,7 @@ int client(int argc, char *argv[]);
 
 int server(int argc, char *argv[])
 {
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   int count = 0;
   while (count < N) {
     if (task) {
@@ -33,7 +41,7 @@ int server(int argc, char *argv[])
 int client(int argc, char *argv[])
 {
 
-  m_task_t task =
+  msg_task_t task =
       MSG_task_create(argv[1], 0 /*comp cost */ , 10000 /*comm size */ ,
                       NULL /*arbitrary data */ );
 
@@ -46,7 +54,7 @@ int client(int argc, char *argv[])
 int main(int argc, char *argv[])
 {
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
   MSG_create_environment("platform.xml");
 
@@ -58,8 +66,6 @@ int main(int argc, char *argv[])
 
   MSG_main();
 
-  MSG_clean();
-
   return 0;
 
 }
