@@ -1,10 +1,6 @@
 #ifndef _BUGGED1_LIVENESS_H
 #define _BUGGED1_LIVENESS_H
 
-int yyparse(void);
-int yywrap(void);
-int yylex(void);
-
 int predR(void);
 int predCS(void);
 
