@@ -1,3 +1,10 @@
+/* Copyright (c) 2010-2012. The SimGrid Team.
+ * All rights reserved.                                                     */
+
+/* This program is free software; you can redistribute it and/or modify it
+ * under the terms of the license (GNU LGPL) which comes with this package. */
+
+
 /******************** Non-deterministic message ordering  *********************/
 /* Server assumes a fixed order in the reception of messages from its clients */
 /* which is incorrect because the message ordering is non-deterministic       */
@@ -14,8 +21,8 @@ int client(int argc, char *argv[]);
 
 int server(int argc, char *argv[])
 {
-  m_task_t task1 = NULL;
-  m_task_t task2 = NULL;
+  msg_task_t task1 = NULL;
+  msg_task_t task2 = NULL;
   long val1, val2;
 
   MSG_task_receive(&task1, "mymailbox");
@@ -48,9 +55,9 @@ int server(int argc, char *argv[])
 
 int client(int argc, char *argv[])
 {
-  m_task_t task1 =
+  msg_task_t task1 =
       MSG_task_create("task", 0, 10000, (void *) atol(argv[1]));
-  m_task_t task2 =
+  msg_task_t task2 =
       MSG_task_create("task", 0, 10000, (void *) atol(argv[1]));
 
   XBT_INFO("Send %d!", atoi(argv[1]));
@@ -64,7 +71,7 @@ int client(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
   MSG_create_environment("platform.xml");
 
