@@ -1,3 +1,10 @@
+/* Copyright (c) 2010-2012. The SimGrid Team.
+ * All rights reserved.                                                     */
+
+/* This program is free software; you can redistribute it and/or modify it
+ * under the terms of the license (GNU LGPL) which comes with this package. */
+
+
 /**************** Shared buffer between asynchronous receives *****************/
 /* Server process assumes that the data from the second communication comm2   */
 /* will overwrite the one from the first communication, because of the order  */
@@ -17,7 +24,7 @@ int client(int argc, char *argv[]);
 
 int server(int argc, char *argv[])
 {
-  m_task_t task1;
+  msg_task_t task1;
   long val1;
   msg_comm_t comm1, comm2;
 
@@ -39,7 +46,7 @@ int client(int argc, char *argv[])
 {
   msg_comm_t comm;
   char *mbox;
-  m_task_t task1 =
+  msg_task_t task1 =
       MSG_task_create("task", 0, 10000, (void *) atol(argv[1]));
 
   mbox = bprintf("mymailbox%s", argv[1]);
@@ -55,7 +62,7 @@ int client(int argc, char *argv[])
 
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
   MSG_create_environment("platform.xml");
 
