@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -6,6 +6,7 @@
 
 #include "msg/msg.h"            /* core library */
 #include "xbt/sysdep.h"         /* calloc */
+#include "xbt/synchro_core.h"
 
 /* Create a log channel to have nice outputs. */
 #include "xbt/log.h"
@@ -21,12 +22,12 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 
 xbt_mutex_t mutex = NULL;
 xbt_cond_t cond = NULL;
-static m_process_t process_to_migrate = NULL;
+static msg_process_t process_to_migrate = NULL;
 
 /** The guy we will move from host to host. It move alone and then is moved by policeman back  */
 static int emigrant(int argc, char *argv[])
 {
-  m_task_t task;
+  msg_task_t task;
   XBT_INFO
       ("I'll look for a new job on another machine where the grass is greener.");
   MSG_process_migrate(MSG_process_self(), MSG_get_host_by_name("Boivin"));
@@ -45,7 +46,7 @@ static int emigrant(int argc, char *argv[])
   xbt_cond_broadcast(cond);
   xbt_mutex_release(mutex);
   MSG_process_suspend(MSG_process_self());
-  m_host_t h = MSG_process_get_host(MSG_process_self());
+  msg_host_t h = MSG_process_get_host(MSG_process_self());
   XBT_INFO("I've been moved on this new host: %s", MSG_host_get_name(h));
   XBT_INFO("Uh, nothing to do here. Stopping now");
   return 0;
@@ -71,10 +72,10 @@ static int policeman(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* Argument checking */
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     XBT_CRITICAL("Usage: %s platform_file deployment_file\n", argv[0]);
     XBT_CRITICAL("example: %s msg_platform.xml msg_deployment_suspend.xml\n",
@@ -98,9 +99,6 @@ int main(int argc, char *argv[])
   xbt_cond_destroy(cond);
   xbt_mutex_destroy(mutex);
 
-  if (res == MSG_OK)
-    res = MSG_clean();
-
   if (res == MSG_OK)
     return 0;
   else
