@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -25,19 +25,19 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
  */
 
 int test(int argc, char *argv[]);
-MSG_error_t test_all(const char *platform_file);
+msg_error_t test_all(const char *platform_file);
 
 /** Emitter function  */
 int test(int argc, char *argv[])
 {
   xbt_dynar_t slaves_dynar;
   int slaves_count = 0;
-  m_host_t *slaves = NULL;
+  msg_host_t *slaves = NULL;
   double task_comp_size = 100000;
   double task_comm_size = 10000;
   double *computation_amount = NULL;
   double *communication_amount = NULL;
-  m_task_t ptask = NULL;
+  msg_task_t ptask = NULL;
   int i, j;
 
   slaves_dynar = MSG_hosts_as_dynar();
@@ -71,17 +71,17 @@ int test(int argc, char *argv[])
 }
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file)
+msg_error_t test_all(const char *platform_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
   xbt_dynar_t all_hosts;
-  m_host_t first_host;
+  msg_host_t first_host;
 
   MSG_config("workstation/model", "ptask_L07");
   MSG_create_environment(platform_file);
 
   all_hosts = MSG_hosts_as_dynar();
-  first_host = xbt_dynar_getfirst_as(all_hosts,m_host_t);
+  first_host = xbt_dynar_getfirst_as(all_hosts,msg_host_t);
   MSG_process_create("test", test, NULL, first_host);
   res = MSG_main();
   xbt_dynar_free(&all_hosts);
@@ -92,16 +92,15 @@ MSG_error_t test_all(const char *platform_file)
 
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 2) {
     printf("Usage: %s platform_file\n", argv[0]);
     printf("example: %s msg_platform.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1]);
-  MSG_clean();
 
   if (res == MSG_OK)
     return 0;
