@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@ static int test(int argc, char *argv[])
 {
   double computation_amount = 0.0;
   double priority = 1.0;
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
 
   _XBT_GNUC_UNUSED int res = sscanf(argv[1], "%lg", &computation_amount);
   xbt_assert(res, "Invalid argument %s\n", argv[1]);
@@ -45,10 +45,10 @@ static int test(int argc, char *argv[])
   return 0;
 }
 
-static MSG_error_t test_all(const char *platform_file,
+static msg_error_t test_all(const char *platform_file,
                             const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   {                             /*  Simulation setting */
     MSG_create_environment(platform_file);
@@ -65,7 +65,7 @@ static MSG_error_t test_all(const char *platform_file,
 
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
 #ifdef _MSC_VER
   unsigned int prev_exponent_format =
@@ -73,14 +73,13 @@ int main(int argc, char *argv[])
 #endif
 
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     printf("example: %s msg_platform.xml msg_deployment.xml\n", argv[0]);
     exit(1);
   }
   res = test_all(argv[1], argv[2]);
-  MSG_clean();
 
 #ifdef _MSC_VER
   _set_output_format(prev_exponent_format);
