@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -31,7 +31,6 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(test, "Property test");
 int alice(int argc, char *argv[]);
 int bob(int argc, char *argv[]);
 int carole(int argc, char *argv[]);
-int forwarder(int argc, char *argv[]);
 msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
@@ -43,7 +42,7 @@ static void test_host(const char*hostname)
   char *key, *data;
   const char *noexist = "Unknown";
   const char *value;
-  char exist[] = "SG_TEST_Hdd";
+  char exist[] = "Hdd";
 
   XBT_INFO("== Print the properties of the host");
   xbt_dict_foreach(props, cursor, key, data)
@@ -63,7 +62,7 @@ static void test_host(const char*hostname)
   XBT_INFO("   Property: %s old value: %s", exist, value);
 
   XBT_INFO("== Trying to modify a host property");
-  xbt_dict_set(props, exist, xbt_strdup("250"), NULL);
+  MSG_host_set_property_value(thehost, exist, xbt_strdup("250"), NULL);
 
   /* Test if we have changed the value */
   value = MSG_host_get_property_value(thehost, exist);
@@ -72,9 +71,9 @@ static void test_host(const char*hostname)
               "Value of property %s is defined to %s (where it should be 250)",
               exist, value);
   XBT_INFO("   Property: %s old value: %s", exist, value);
-   
+
   /* Restore the value for the next test */
-  xbt_dict_set(props, exist, xbt_strdup("180"), NULL);
+  MSG_host_set_property_value(thehost, exist, xbt_strdup("180"), NULL);
 }
 
 int alice(int argc, char *argv[]) { /* Dump what we have on the current host */
@@ -112,14 +111,35 @@ int bob(int argc, char *argv[])
 msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
+  int host_number;
+  unsigned int i;
+  xbt_dynar_t hosts;
+  msg_host_t host;
+  msg_error_t ret;
+
   MSG_function_register("alice", alice);
   MSG_function_register("bob", bob);
   MSG_function_register("carole", carole);
 
   MSG_create_environment(platform_file);
+
+  host_number = MSG_get_host_number();
+  XBT_INFO("There are %d hosts in the environment", host_number);
+
+  hosts = MSG_hosts_as_dynar();
+
+  xbt_dynar_foreach(hosts, i, host){
+    XBT_INFO("Host '%s' runs at %.0f flops/s",MSG_host_get_name(host),
+       MSG_get_host_speed(host));
+  }
+
   MSG_launch_application(application_file);
 
-  return MSG_main();
+  ret = MSG_main();
+
+  xbt_dynar_free(&hosts);
+
+  return ret;
 }                               /* end_of_test_all */
 
 
