@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,7 @@
 #include "xbt/asserts.h"
 
 /** @addtogroup MSG_examples
- * 
+ *
  *  - <b>sendrecv/sendrecv.c: Ping-pong example</b>. It's hard to
  *    think of a simpler example. The tesh files laying in the
  *    directory are instructive concerning the way to pass options to the simulators (as described in \ref options).
@@ -26,7 +26,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 int sender(int argc, char *argv[]);
 int receiver(int argc, char *argv[]);
 
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file);
 
 double task_comm_size_lat = 10e0;
@@ -35,16 +35,16 @@ double task_comm_size_bw = 10e8;
 /** Emitter function  */
 int sender(int argc, char *argv[])
 {
-  m_host_t host = NULL;
+  msg_host_t host = NULL;
   double time;
-  m_task_t task_la = NULL;
-  m_task_t task_bw = NULL;
+  msg_task_t task_la = NULL;
+  msg_task_t task_bw = NULL;
   char sprintf_buffer_la[64];
   char sprintf_buffer_bw[64];
 
   XBT_INFO("sender");
 
-  /*host = xbt_new0(m_host_t,1); */
+  /*host = xbt_new0(msg_host_t,1); */
 
   XBT_INFO("host = %s", argv[1]);
 
@@ -62,7 +62,7 @@ int sender(int argc, char *argv[])
       MSG_task_create(sprintf_buffer_la, 0.0, task_comm_size_lat, NULL);
   task_la->data = xbt_new(double, 1);
   *(double *) task_la->data = time;
-  XBT_INFO("task_la->data = %le", *((double *) task_la->data));
+  XBT_INFO("task_la->data = %e", *((double *) task_la->data));
   MSG_task_send(task_la, argv[1]);
 
   /* Bandwidth */
@@ -72,7 +72,7 @@ int sender(int argc, char *argv[])
       MSG_task_create(sprintf_buffer_bw, 0.0, task_comm_size_bw, NULL);
   task_bw->data = xbt_new(double, 1);
   *(double *) task_bw->data = time;
-  XBT_INFO("task_bw->data = %le", *((double *) task_bw->data));
+  XBT_INFO("task_bw->data = %e", *((double *) task_bw->data));
   MSG_task_send(task_bw, argv[1]);
 
   return 0;
@@ -82,15 +82,13 @@ int sender(int argc, char *argv[])
 int receiver(int argc, char *argv[])
 {
   double time, time1, sender_time;
-  m_task_t task_la = NULL;
-  m_task_t task_bw = NULL;
+  msg_task_t task_la = NULL;
+  msg_task_t task_bw = NULL;
   int a;
   double communication_time = 0;
 
   XBT_INFO("receiver");
 
-  time = MSG_get_clock();
-
   /* Get Latency */
   a = MSG_task_receive(&task_la,MSG_host_get_name(MSG_host_self()));
   if (a == MSG_OK) {
@@ -101,13 +99,12 @@ int receiver(int argc, char *argv[])
     XBT_INFO("Task received : %s", task_la->name);
     xbt_free(task_la->data);
     MSG_task_destroy(task_la);
-    XBT_INFO("Communic. time %le", communication_time);
+    XBT_INFO("Communic. time %e", communication_time);
     XBT_INFO("--- la %f ----", communication_time);
   } else {
     xbt_die("Unexpected behavior");
   }
 
-
   /* Get Bandwidth */
   a = MSG_task_receive(&task_bw,MSG_host_get_name(MSG_host_self()));
   if (a == MSG_OK) {
@@ -118,7 +115,7 @@ int receiver(int argc, char *argv[])
     XBT_INFO("Task received : %s", task_bw->name);
     xbt_free(task_bw->data);
     MSG_task_destroy(task_bw);
-    XBT_INFO("Communic. time %le", communication_time);
+    XBT_INFO("Communic. time %e", communication_time);
     XBT_INFO("--- bw %f ----", task_comm_size_bw / communication_time);
   } else {
     xbt_die("Unexpected behavior");
@@ -130,13 +127,10 @@ int receiver(int argc, char *argv[])
 
 
 /** Test function */
-MSG_error_t test_all(const char *platform_file,
+msg_error_t test_all(const char *platform_file,
                      const char *application_file)
 {
-
-  MSG_error_t res = MSG_OK;
-
-
+  msg_error_t res = MSG_OK;
 
   XBT_INFO("test_all");
 
@@ -158,15 +152,14 @@ MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
 #ifdef _MSC_VER
   unsigned int prev_exponent_format =
       _set_output_format(_TWO_DIGIT_EXPONENT);
 #endif
 
-  MSG_global_init(&argc, argv);
-
+  MSG_init(&argc, argv);
 
   if (argc != 3) {
     XBT_CRITICAL("Usage: %s platform_file deployment_file <model>\n",
@@ -188,9 +181,7 @@ int main(int argc, char *argv[])
 
   res = test_all(argv[1], argv[2]);
 
-  XBT_INFO("Total simulation time: %le", MSG_get_clock());
-
-  MSG_clean();
+  XBT_INFO("Total simulation time: %e", MSG_get_clock());
 
 #ifdef _MSC_VER
   _set_output_format(prev_exponent_format);
