@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007, 2009-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -31,7 +31,7 @@ static int lazy_guy(int argc, char *argv[])
     resumes it 10 seconds later. */
 static int dream_master(int argc, char *argv[])
 {
-  m_process_t lazy = NULL;
+  msg_process_t lazy = NULL;
 
   XBT_INFO("Let's create a lazy guy.");
   lazy = MSG_process_create("Lazy", lazy_guy, NULL, MSG_host_self());
@@ -44,10 +44,10 @@ static int dream_master(int argc, char *argv[])
 }                               /* end_of_dram_master */
 
 /** Test function */
-static MSG_error_t test_all(const char *platform_file,
+static msg_error_t test_all(const char *platform_file,
                             const char *application_file)
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   {                             /*  Simulation setting */
     MSG_create_environment(platform_file);
@@ -66,9 +66,9 @@ static MSG_error_t test_all(const char *platform_file,
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     XBT_CRITICAL("Usage: %s platform_file deployment_file\n", argv[0]);
     XBT_CRITICAL("example: %s msg_platform.xml msg_deployment_suspend.xml\n",
@@ -76,7 +76,6 @@ int main(int argc, char *argv[])
     exit(1);
   }
   test_all(argv[1], argv[2]);
-  res = MSG_clean();
 
   if (res == MSG_OK)
     return 0;
