@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2008-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -32,7 +32,7 @@ int host(int argc, char *argv[])
 {
   int host_number = atoi(MSG_process_get_name(MSG_process_self()));
   char mailbox[256];
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   _XBT_GNUC_UNUSED int res;
   if (host_number == 0){ //master  send then receive
     sprintf(mailbox, "%d", host_number+1);
@@ -63,7 +63,7 @@ int host(int argc, char *argv[])
 int main(int argc, char **argv)
 {
   int i,res;
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   MSG_create_environment(argv[1]);
   xbt_dynar_t hosts = MSG_hosts_as_dynar();
   nb_hosts =  xbt_dynar_length(hosts);
@@ -73,14 +73,13 @@ int main(int argc, char **argv)
   for(i = 0 ; i<nb_hosts; i++)
   {
     char* name_host = bprintf("%d",i);
-    MSG_process_create( name_host, host, NULL, xbt_dynar_get_as(hosts,i,m_host_t) );
+    MSG_process_create( name_host, host, NULL, xbt_dynar_get_as(hosts,i,msg_host_t) );
     free(name_host);
   }
   xbt_dynar_free(&hosts);
 
   res = MSG_main();
   XBT_INFO("Simulation time %g", MSG_get_clock());
-  MSG_clean();
   if (res == MSG_OK)
     return 0;
   else
