@@ -16,8 +16,8 @@
  * these two hosts, if there is one defined in the platform file, and update the variable
  * of all the links of that route to the value provided.
  * You might want to run this program with the following parameters:
- * --cfg=tracing:1
- * --cfg=tracing/platform:1
+ * --cfg=tracing:yes
+ * --cfg=tracing/platform:yes
  * (See \ref tracing_tracing_options for details)
  */
 
@@ -26,7 +26,7 @@ int master(int argc, char *argv[]);
 //dump function to create and execute a task
 static void create_and_execute_task (void)
 {
-  m_task_t task = MSG_task_create("task", 1000000, 0, NULL);
+  msg_task_t task = MSG_task_create("task", 1000000, 0, NULL);
   MSG_task_execute (task);
   MSG_task_destroy (task);
 }
@@ -75,7 +75,7 @@ int master(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     exit(1);
@@ -95,6 +95,5 @@ int main(int argc, char *argv[])
   MSG_launch_application(deployment_file);
 
   MSG_main();
-  MSG_clean();
   return 0;
 }
