@@ -13,8 +13,8 @@
  * to provide the name of the link to update the value of the variable associated
  * to that link.
  * You might want to run this program with the following parameters:
- * --cfg=tracing:1
- * --cfg=tracing/platform:1
+ * --cfg=tracing:yes
+ * --cfg=tracing/platform:yes
  * (See \ref tracing_tracing_options for details)
  */
 
@@ -23,7 +23,7 @@ int master(int argc, char *argv[]);
 //dump function to create and execute a task
 static void create_and_execute_task (void)
 {
-  m_task_t task = MSG_task_create("task", 1000000, 0, NULL);
+  msg_task_t task = MSG_task_create("task", 1000000, 0, NULL);
   MSG_task_execute (task);
   MSG_task_destroy (task);
 }
@@ -67,7 +67,7 @@ int master(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     exit(1);
@@ -87,6 +87,5 @@ int main(int argc, char *argv[])
   MSG_launch_application(deployment_file);
 
   MSG_main();
-  MSG_clean();
   return 0;
 }
