@@ -1,4 +1,5 @@
-/* Copyright (c) 2010 The SimGrid team. All rights reserved.                */
+/* Copyright (c) 2010-2012. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -10,8 +11,8 @@
  * of where the process has been during its execution. Migrations are represented by
  * arrows from the origin to the destination host.
  * You might want to run this program with the following parameters:
- * --cfg=tracing:1
- * --cfg=tracing/msg/process:1
+ * --cfg=tracing:yes
+ * --cfg=tracing/msg/process:yes
  * (See \ref tracing_tracing_options for details)
  */
 
@@ -26,7 +27,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test,
 /** The guy we will move from host to host. It move alone and then is moved by policeman back  */
 static int emigrant(int argc, char *argv[])
 {
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
   char *destination = NULL;
 
   MSG_process_sleep(2);
@@ -46,7 +47,7 @@ static int emigrant(int argc, char *argv[])
 
 static int master(int argc, char *argv[])
 {
-  m_task_t task = NULL;
+  msg_task_t task = NULL;
 
   // I am the master of emigrant process,
   // I tell it where it must emigrate to.
@@ -80,7 +81,7 @@ static int master(int argc, char *argv[])
 int main(int argc, char *argv[])
 {
   /* Argument checking */
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     XBT_CRITICAL("Usage: %s platform_file deployment_file\n", argv[0]);
     exit(1);
@@ -98,6 +99,5 @@ int main(int argc, char *argv[])
   MSG_launch_application(deployment_file);
 
   MSG_main();
-  MSG_clean();
   return 0;
 }                               /* end_of_main */
