@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -14,7 +14,7 @@
  * 
  * - <b>tracing/simple.c</b> very simple program where each process creates, executes and
  *   destroy a task. You might want to run this program with the following parameters:
- *   --cfg=tracing/uncategorized:1
+ *   --cfg=tracing/uncategorized:yes
  *   (See \ref tracing_tracing_options for details)
  */
 
@@ -29,7 +29,7 @@ int simple_func(int argc, char *argv[]);
 /** Emitter function  */
 int simple_func(int argc, char *argv[])
 {
-  m_task_t task = MSG_task_create("task", 100, 0, NULL);
+  msg_task_t task = MSG_task_create("task", 100, 0, NULL);
   MSG_task_execute (task);
   MSG_task_destroy (task);
   return 0;
@@ -38,7 +38,7 @@ int simple_func(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 3) {
     printf("Usage: %s platform_file deployment_file\n", argv[0]);
     exit(1);
@@ -53,6 +53,5 @@ int main(int argc, char *argv[])
   MSG_launch_application(deployment_file);
 
   MSG_main();
-  MSG_clean();
   return 0;
 }
