@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -10,8 +10,8 @@
  * platform file is traced to a Paje trace file format using the tracing
  * mechanism of Simgrid.
  * You might want to run this program with the following parameters:
- * --cfg=tracing:1
- * --cfg=tracing/categorized:1
+ * --cfg=tracing:yes
+ * --cfg=tracing/categorized:yes
  * (See \ref tracing_tracing_options for details)
  */
 
@@ -26,7 +26,7 @@ XBT_LOG_NEW_DEFAULT_CATEGORY(msg_test, "Messages specific for this msg example")
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 2) {
     printf("Usage: %s platform_file\n", argv[0]);
     exit(1);
@@ -35,6 +35,5 @@ int main(int argc, char *argv[])
   char *platform_file = argv[1];
   MSG_create_environment(platform_file);
   MSG_main();
-  MSG_clean();
   return 0;
 }
