@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -9,8 +9,8 @@
  * - <b>tracing/user_variables.c</b>: This program demonstrates how to
  * trace user variables associated to the hosts of the platform file.
  * You might want to run this program with the following parameters:
- * --cfg=tracing:1
- * --cfg=tracing/platform:1
+ * --cfg=tracing:yes
+ * --cfg=tracing/platform:yes
  * (See \ref tracing_tracing_options for details)
  */
 
@@ -100,9 +100,9 @@ int main(int argc, char *argv[])
     xbt_dynar_free (&link_variables);
   }
 
-  //create a customized triva graph configuration file
+  //create a customized viva graph configuration file
   FILE *fp;
-  fp = fopen ("triva_graph.plist", "w");
+  fp = fopen ("viva_graph.plist", "w");
   if (!fp){
     return 1;
   }
