@@ -1,6 +1,6 @@
 /* simple test trying to load a DAX file.                                   */
 
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -59,6 +59,12 @@ int main(int argc, char **argv)
 
   /* load the DAX file */
   dax = SD_daxload(argv[2]);
+  if (!dax){
+    XBT_ERROR("A problem occurred during DAX parsing (cycle or syntax). Do not continue this test");
+    free(tracefilename);
+    SD_exit();
+    exit(255);
+  }
 
   /* Display all the tasks */
   XBT_INFO
@@ -132,7 +138,7 @@ int main(int argc, char **argv)
     SD_task_destroy(task);
   }
   fclose(out);
-
+  xbt_dynar_free_container(&dax);
   /* exit */
   SD_exit();
   return 0;
