@@ -1,6 +1,6 @@
 /* simple test trying to load a DOT file.                                   */
 
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -32,6 +32,18 @@ int main(int argc, char **argv)
     XBT_INFO("example: %s ../2clusters.xml dag.dot dag.mytrace", argv[0]);
     exit(1);
   }
+
+  /* creation of the environment */
+  SD_create_environment(argv[1]);
+
+  /* load the DOT file */
+  dot = SD_dotload(argv[2]);
+  if(dot == NULL){
+    XBT_CRITICAL("No dot loaded. Do you have a cycle in your graph?");
+    SD_exit();
+    exit(2);
+  }
+
   char *tracefilename;
   if (argc == 3) {
     char *last = strrchr(argv[2], '.');
@@ -44,15 +56,6 @@ int main(int argc, char **argv)
     tracefilename = xbt_strdup(argv[3]);
   }
 
-  /* creation of the environment */
-  SD_create_environment(argv[1]);
-
-  /* load the DOT file */
-  dot = SD_dotload(argv[2]);
-  if(dot == NULL){
-    xbt_die("No dot load may be you have a cycle in your graph");
-  }
-
   /* Display all the tasks */
   XBT_INFO
       ("------------------- Display all tasks of the loaded DAG ---------------------------");
@@ -98,22 +101,19 @@ int main(int argc, char **argv)
     SD_workstation_t *wsl = SD_task_get_workstation_list(task);
     switch (kind) {
     case SD_TASK_COMP_SEQ:
-      fprintf(out, "[%f] %s compute %f # %s\n",
-              SD_task_get_start_time(task),
-              SD_workstation_get_name(wsl[0]), SD_task_get_amount(task),
-              SD_task_get_name(task));
+      fprintf(out, "[%f->%f] %s compute %f flops # %s\n",
+          SD_task_get_start_time(task),
+          SD_task_get_finish_time(task),
+          SD_workstation_get_name(wsl[0]), SD_task_get_amount(task),
+          SD_task_get_name(task));
       break;
     case SD_TASK_COMM_E2E:
-      fprintf(out, "[%f] %s send %s %f # %s\n",
-              SD_task_get_start_time(task),
-              SD_workstation_get_name(wsl[0]),
-              SD_workstation_get_name(wsl[1]), SD_task_get_amount(task),
-              SD_task_get_name(task));
-      fprintf(out, "[%f] %s recv %s %f # %s\n",
-              SD_task_get_finish_time(task),
-              SD_workstation_get_name(wsl[1]),
-              SD_workstation_get_name(wsl[0]), SD_task_get_amount(task),
-              SD_task_get_name(task));
+      fprintf(out, "[%f -> %f] %s -> %s transfer of %.0f bytes # %s\n",
+          SD_task_get_start_time(task),
+          SD_task_get_finish_time(task),
+          SD_workstation_get_name(wsl[0]),
+          SD_workstation_get_name(wsl[1]), SD_task_get_amount(task),
+          SD_task_get_name(task));
       break;
     default:
       xbt_die("Task %s is of unknown kind %d", SD_task_get_name(task),
