@@ -1,10 +1,15 @@
-/* GOAL loader prototype. Not ready for public usage yet */
-
-/* Copyright (c) 2011. The SimGrid Team.
- * All rights reserved.                                                     */
-
-/* This program is free software; you can redistribute it and/or modify it
- * under the terms of the license (GNU LGPL) which comes with this package. */
+/* Example of scatter communication, accepting a large amount of processes. 
+ * This based the experiment of Fig. 4 in http://hal.inria.fr/hal-00650233/
+ * That experiment is a comparison to the LogOPSim simulator, that takes 
+ *  GOAL files as an input, thus the file name. But there is no actual link
+ *  to the GOAL formalism beside of this.
+ * 
+ * Copyright (c) 2011-2013. The SimGrid Team.
+ * All rights reserved.
+ * 
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the license (GNU LGPL) which comes with this package.
+ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -67,7 +72,7 @@ int main(int argc, char **argv) {
   reclaimed = xbt_dynar_new(sizeof(bcast_task_t),xbt_free_ref);
   xbt_dynar_t done = NULL;
 
-  xbt_os_timer_start(timer);
+  xbt_os_cputimer_start(timer);
   send_one(0,SD_workstation_get_number());
   do {
     if (done != NULL && !xbt_dynar_is_empty(done)) {
@@ -93,8 +98,8 @@ int main(int argc, char **argv) {
     }
     done=SD_simulate(-1);
   } while(!xbt_dynar_is_empty(done));
-  xbt_os_timer_stop(timer);
-  printf("parse_time:%lf\n", xbt_os_timer_elapsed(timer) );
+  xbt_os_cputimer_stop(timer);
+  printf("exec_time:%f\n", xbt_os_timer_elapsed(timer) );
 
   xbt_dynar_free(&done);
   xbt_dynar_free(&reclaimed);
