@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,7 +26,7 @@ int main(int argc, char **argv)
   char *key, *data;
   char noexist[] = "NoProp";
   const char *value;
-  char exist[] = "SG_TEST_Hdd";
+  char exist[] = "Hdd";
 
   /* initialisation of SD */
   SD_init(&argc, argv);
