@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -23,6 +23,8 @@ XBT_PUBLIC(void) TRACE_category(const char *category);
 XBT_PUBLIC(void) TRACE_category_with_color (const char *category, const char *color);
 XBT_PUBLIC(xbt_dynar_t) TRACE_get_categories (void);
 XBT_PUBLIC(void) TRACE_smpi_set_category(const char *category);
+XBT_PUBLIC(void) TRACE_sd_set_task_category(SD_task_t task,
+    const char *category);
 
 /*
  * Functions to manage tracing marks (used for trace comparison experiments)
@@ -42,6 +44,18 @@ XBT_PUBLIC(int) TRACE_platform_graph_export_graphviz (const char *filename);
 /*
  * User-variables related functions
  */
+/* for VM variables */
+XBT_PUBLIC(void) TRACE_vm_variable_declare (const char *variable);
+XBT_PUBLIC(void) TRACE_vm_variable_declare_with_color (const char *variable, const char *color);
+XBT_PUBLIC(void) TRACE_vm_variable_set (const char *vm, const char *variable, double value);
+XBT_PUBLIC(void) TRACE_vm_variable_add (const char *vm, const char *variable, double value);
+XBT_PUBLIC(void) TRACE_vm_variable_sub (const char *vm, const char *variable, double value);
+XBT_PUBLIC(void) TRACE_vm_variable_set_with_time (double time, const char *vm, const char *variable, double value);
+XBT_PUBLIC(void) TRACE_vm_variable_add_with_time (double time, const char *vm, const char *variable, double value);
+XBT_PUBLIC(void) TRACE_vm_variable_sub_with_time (double time, const char *vm, const char *variable, double value);
+XBT_PUBLIC(xbt_dynar_t) TRACE_get_vm_variables (void);
+
+
 /* for host variables */
 XBT_PUBLIC(void) TRACE_host_variable_declare (const char *variable);
 XBT_PUBLIC(void) TRACE_host_variable_declare_with_color (const char *variable, const char *color);
@@ -78,7 +92,7 @@ XBT_PUBLIC(void) TRACE_host_push_state (const char *host, const char *state, con
 XBT_PUBLIC(void) TRACE_host_pop_state (const char *host, const char *state);
 XBT_PUBLIC(void) TRACE_host_reset_state (const char *host, const char *state);
 
-/* for creating graph configuration files for Triva by hand */
+/* for creating graph configuration files for Viva by hand */
 XBT_PUBLIC(xbt_dynar_t) TRACE_get_node_types (void);
 XBT_PUBLIC(xbt_dynar_t) TRACE_get_edge_types (void);
 XBT_PUBLIC(void) TRACE_pause (void);
@@ -98,6 +112,16 @@ XBT_PUBLIC(void) TRACE_resume (void);
 
 #define TRACE_platform_graph_export_graphviz(filename)
 
+#define TRACE_vm_variable_declare(var)
+#define TRACE_vm_variable_declare_with_color(var,color)
+#define TRACE_vm_variable_set(vm,var,value)
+#define TRACE_vm_variable_add(vm,var,value)
+#define TRACE_vm_variable_sub(vm,var,value)
+#define TRACE_vm_variable_set_with_time(time,vm,var,value)
+#define TRACE_vm_variable_add_with_time(time,vm,var,value)
+#define TRACE_vm_variable_sub_with_time(time,vm,var,value)
+#define TRACE_get_vm_variables()
+
 #define TRACE_host_variable_declare(var)
 #define TRACE_host_variable_declare_with_color(var,color)
 #define TRACE_host_variable_set(host,var,value)
