@@ -1,4 +1,5 @@
-/* Copyright (c) 2004-2012. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2004-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -6,7 +7,7 @@
 #ifndef MSG_DATATYPE_H
 #define MSG_DATATYPE_H
 #include "xbt/misc.h"
-#include "xbt/file_stat.h"
+#include "xbt/lib.h"
 #include "simgrid/simix.h"
 #include "simgrid_config.h"     // for HAVE_TRACING
 
@@ -26,16 +27,12 @@ SG_BEGIN_DECL()
  * want to send your task, but only the name of this mailbox. */
 typedef struct s_smx_rvpoint *msg_mailbox_t;
 
+/* ******************************** Environment ************************************ */
+typedef struct s_as *msg_as_t;
 
 /* ******************************** Host ************************************ */
 
-typedef struct msg_host {
-  xbt_swag_t vms;
-  smx_host_t smx_host;          /**< SIMIX representation of this host   */
-#ifdef MSG_USE_DEPRECATED
-  msg_mailbox_t *mailboxes;     /**< the channels  */
-#endif
-} s_msg_host_t;
+extern int MSG_HOST_LEVEL;
 
 /** @brief Host datatype.
     @ingroup m_host_management
@@ -47,7 +44,21 @@ typedef struct msg_host {
     some <em>private data</em> that can be only accessed by local
     process.
  */
-typedef struct msg_host *msg_host_t;
+typedef xbt_dictelm_t msg_host_t;
+typedef s_xbt_dictelm_t s_msg_host_t;
+
+typedef struct msg_host_priv {
+  xbt_swag_t vms;
+#ifdef MSG_USE_DEPRECATED
+  msg_mailbox_t *mailboxes;     /**< the channels  */
+#endif
+} s_msg_host_priv_t, *msg_host_priv_t;
+
+static inline msg_host_priv_t MSG_host_priv(msg_host_t host){
+  return (msg_host_priv_t )xbt_lib_get_level(host, MSG_HOST_LEVEL);
+}
+
+
 
 /* ******************************** Task ************************************ */
 
@@ -72,31 +83,65 @@ typedef struct msg_task {
  */
 typedef struct msg_task *msg_task_t;
 
+/* ********************************  VM ************************************* */
+typedef struct msg_vm *msg_vm_t;
+
+typedef enum {
+  msg_vm_state_suspended, msg_vm_state_running, msg_vm_state_migrating
+} e_msg_vm_state_t;
+
+typedef struct msg_vm {
+  char *name;
+  s_xbt_swag_hookup_t all_vms_hookup;
+  s_xbt_swag_hookup_t host_vms_hookup;
+  xbt_dynar_t processes;
+  e_msg_vm_state_t state;
+  msg_host_t location;
+  int coreAmount;
+} s_msg_vm_t;
 
 /* ******************************** File ************************************ */
 typedef struct simdata_file *simdata_file_t;
 
+typedef struct s_msg_file_info {
+  sg_storage_size_t size;
+  char* mount_point;
+  char* storageId;
+  char* storage_type;
+  char* content_type;
+} s_msg_file_info_t, *msg_file_info_t;
+
 typedef struct msg_file {
-  char *name;                   /**< @brief file name */
-  simdata_file_t simdata;                /**< @brief simulator data  */
-  void *data;                   /**< @brief user data */
+  char *fullname;               /**< @brief file full name (path+name)*/
+  simdata_file_t simdata;       /**< @brief simulator data  */
+  msg_file_info_t info;
 } s_msg_file_t;
 
 /** @brief File datatype.
-    @ingroup msg_file_management 
- 
-    You should consider this as an opaque object.
+ *  @ingroup msg_file_management
+ *
+ *  You should consider this as an opaque object.
  */
+
 typedef struct msg_file *msg_file_t;
 
+/* ******************************** Storage ************************************ */
+/* TODO: PV: to comment */
 
-/** @brief File datatype.
-    @ingroup msg_file_management
+extern int MSG_STORAGE_LEVEL;
 
-    You should consider this as an opaque object.
+/** @brief Storage datatype.
+ *  @ingroup msg_storage_management
+ *
+ *  You should consider this as an opaque object.
  */
-typedef s_file_stat_t s_msg_stat_t, *msg_stat_t;
+typedef xbt_dictelm_t msg_storage_t;
+typedef s_xbt_dictelm_t s_msg_storage_t;
 
+typedef struct msg_storage_priv  {
+  // TODO PV: fill it (or not) !
+  void * dummy;
+} s_msg_storage_priv_t, *msg_storage_priv_t;
 
 /*************** Begin GPU ***************/
 typedef struct simdata_gpu_task *simdata_gpu_task_t;
