@@ -1,4 +1,4 @@
-/* Copyright (c) 2006, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2006, 2008-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -6,6 +6,7 @@
 
 #ifndef SIMDAG_DATATYPES_H
 #define SIMDAG_DATATYPES_H
+#include "xbt/dict.h"
 
 /** @brief Workstation datatype
     @ingroup SD_datatypes_management
@@ -15,7 +16,7 @@
     resource with computing capabilities</em> and has a <em>name</em>.
 
     @see SD_workstation_management */
-typedef struct SD_workstation *SD_workstation_t;
+typedef xbt_dictelm_t SD_workstation_t;
 
 /** @brief Workstation access mode
     @ingroup SD_datatypes_management
@@ -88,4 +89,12 @@ typedef enum {
 } e_SD_task_kind_t;
 
 
+/** @brief Storage datatype
+    @ingroup SD_datatypes_management
+
+ TODO PV: comment it !
+
+    @see SD_storage_management */
+typedef xbt_dictelm_t SD_storage_t;
+
 #endif
