@@ -1,6 +1,7 @@
 /* platf.h - Public interface to the SimGrid platforms                      */
 
-/* Copyright (c) 2004-2012. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2004-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -33,7 +34,7 @@ typedef enum {
   SURF_TRACE_CONNECT_KIND_LINK_AVAIL = 2,
   SURF_TRACE_CONNECT_KIND_BANDWIDTH = 1,
   SURF_TRACE_CONNECT_KIND_LATENCY = 0
-} e_surf_trace_connect_kin_t;
+} e_surf_trace_connect_kind_t;
 
 typedef enum {
   SURF_PROCESS_ON_FAILURE_DIE = 1,
@@ -70,6 +71,18 @@ XBT_PUBLIC(probabilist_event_generator_t) tmgr_event_generator_new_exponential(c
 XBT_PUBLIC(probabilist_event_generator_t) tmgr_event_generator_new_weibull(const char* id,
                                                                            double scale,
                                                                            double shape);
+typedef xbt_dictelm_t sg_host_t;
+static inline char* sg_host_name(sg_host_t host) {
+  return host->key;
+}
+
+typedef xbt_dictelm_t sg_storage_t;
+static inline char* sg_storage_name(sg_storage_t storage) {
+  return storage->key;
+}
+/* Type for any integer storage size  */
+typedef uint64_t sg_storage_size_t;
+
 
 /*
  * Platform creation functions. Instead of passing 123 arguments to the creation functions
@@ -83,7 +96,8 @@ XBT_PUBLIC(probabilist_event_generator_t) tmgr_event_generator_new_weibull(const
 
 typedef struct {
   const char* id;
-  double power_peak;
+  xbt_dynar_t power_peak;
+  int pstate;
   int core_amount;
   double power_scale;
   tmgr_trace_t power_trace;
@@ -182,6 +196,10 @@ typedef struct s_sg_platf_cluster_cbarg {
   double lat;
   double bb_bw;
   double bb_lat;
+  double loopback_bw;
+  double loopback_lat;
+  double limiter_link;
+  xbt_dict_t properties;
   const char* router_id;
   e_surf_link_sharing_policy_t sharing_policy;
   e_surf_link_sharing_policy_t bb_sharing_policy;
@@ -190,7 +208,7 @@ typedef struct s_sg_platf_cluster_cbarg {
 } s_sg_platf_cluster_cbarg_t;
 
 #define SG_PLATF_CLUSTER_INITIALIZER {NULL,NULL,NULL,NULL,NULL,NULL \
-  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL}
+  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL}
 
 typedef struct s_sg_platf_cabinet_cbarg *sg_platf_cabinet_cbarg_t;
 typedef struct s_sg_platf_cabinet_cbarg {
@@ -209,6 +227,7 @@ typedef struct {
   const char* id;
   const char* type_id;
   const char* content;
+  const char* content_type;
   xbt_dict_t properties;
 } s_sg_platf_storage_cbarg_t, *sg_platf_storage_cbarg_t;
 
@@ -218,8 +237,9 @@ typedef struct {
   const char* id;
   const char* model;
   const char* content;
+  const char* content_type;
   xbt_dict_t properties;
-  unsigned long size; /* size in Gbytes */
+  sg_storage_size_t size;
 } s_sg_platf_storage_type_cbarg_t, *sg_platf_storage_type_cbarg_t;
 
 #define SG_PLATF_STORAGE_TYPE_INITIALIZER {NULL,NULL,NULL,NULL,NULL}
@@ -232,7 +252,7 @@ typedef struct {
 #define SG_PLATF_MSTORAGE_INITIALIZER {NULL,NULL}
 
 typedef struct {
-  const char* id;
+  const char* storageId;
   const char* name;
 } s_sg_platf_mount_cbarg_t, *sg_platf_mount_cbarg_t;
 
@@ -258,7 +278,7 @@ typedef struct s_sg_platf_trace_cbarg {
 
 typedef struct s_sg_platf_trace_connect_cbarg *sg_platf_trace_connect_cbarg_t;
 typedef struct s_sg_platf_trace_connect_cbarg {
-  e_surf_trace_connect_kin_t kind;
+  e_surf_trace_connect_kind_t kind;
   const char *trace;
   const char *element;
 } s_sg_platf_trace_connect_cbarg_t;
@@ -320,7 +340,7 @@ XBT_PUBLIC(void) sg_platf_new_bypassASroute (sg_platf_route_cbarg_t bypassASrout
 XBT_PUBLIC(void) sg_platf_new_prop (sg_platf_prop_cbarg_t prop); // Add a prop
 
 XBT_PUBLIC(void) sg_platf_new_trace(sg_platf_trace_cbarg_t trace);
-XBT_PUBLIC(void) sg_platf_new_trace_connect(sg_platf_trace_connect_cbarg_t trace_connect);
+XBT_PUBLIC(void) sg_platf_trace_connect(sg_platf_trace_connect_cbarg_t trace_connect);
 
 XBT_PUBLIC(void) sg_platf_new_storage(sg_platf_storage_cbarg_t storage); // Add a storage to the currently described AS
 XBT_PUBLIC(void) sg_platf_new_storage(sg_platf_storage_cbarg_t storage); // Add a storage to the currently described AS
