@@ -1,4 +1,4 @@
-/* Copyright (c) 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2006-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -8,42 +8,13 @@
 #define _SURF_SURFXML_PARSE_H
 
 #include <stdio.h>              /* to have FILE */
-#include "surf/simgrid_dtd.h"
+#include "xbt/misc.h"
 #include "xbt/function_types.h"
 #include "xbt/dict.h"
+#include "surf/simgrid_dtd.h"
 
 SG_BEGIN_DECL()
 
-/* Hook for the different tags. All the functions which pointer to are push into here are run when the tag is encountered */
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_route_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_route_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_link_ctn_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_link_ctn_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_process_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_process_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_argument_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_argument_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_prop_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_prop_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_trace_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_trace_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_trace_connect_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_trace_connect_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_random_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_random_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_ASroute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_ASroute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_bypassRoute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_bypassRoute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_bypassASroute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_bypassASroute_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_peer_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_peer_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_include_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_include_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) STag_surfxml_storage_cb_list;
-XBT_PUBLIC_DATA(xbt_dynar_t) ETag_surfxml_storage_cb_list;
-
 XBT_PUBLIC(void) surf_parse_open(const char *file);
 XBT_PUBLIC(void) surf_parse_close(void);
 XBT_PUBLIC(void) surf_parse_init_callbacks(void);
@@ -53,6 +24,11 @@ XBT_PUBLIC(void) surf_parse_error(const char *msg,...) _XBT_GNUC_PRINTF(1,2) _XB
 XBT_PUBLIC(void) surf_parse_warn(const char *msg,...) _XBT_GNUC_PRINTF(1,2);
 XBT_PUBLIC(double) surf_parse_get_double(const char *string);
 XBT_PUBLIC(int) surf_parse_get_int(const char *string);
+XBT_PUBLIC(double) surf_parse_get_time(const char *string);
+XBT_PUBLIC(double) surf_parse_get_size(const char *string);
+XBT_PUBLIC(double) surf_parse_get_bandwidth(const char *string);
+XBT_PUBLIC(double) surf_parse_get_power(const char *string);
+
 /* Prototypes of the functions offered by flex */
 XBT_PUBLIC(int) surf_parse_lex(void);
 XBT_PUBLIC(int) surf_parse_get_lineno(void);
@@ -97,10 +73,5 @@ extern unsigned int surfxml_buffer_stack_stack[1024];
     ETag_surfxml_##tag();                                               \
   } while(0)
 
-XBT_PUBLIC(void) surfxml_add_callback(xbt_dynar_t cb_list,
-                                      void_f_void_t function);
-XBT_PUBLIC(void) surfxml_del_callback(xbt_dynar_t cb_list,
-                                      void_f_void_t function);
-
 SG_END_DECL()
 #endif
