@@ -1,6 +1,6 @@
 /* xbt/lib.h - api to a generic library                                     */
 
-/* Copyright (c) 2011. The SimGrid Team.
+/* Copyright (c) 2011, 2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -28,6 +28,8 @@ XBT_PUBLIC(void) xbt_lib_set(xbt_lib_t lib, const char *name, int level,
                              void *obj);
 XBT_PUBLIC(void *) xbt_lib_get_or_null(xbt_lib_t lib, const char *name,
                                        int level);
+XBT_PUBLIC(xbt_dictelm_t) xbt_lib_get_elm_or_null(xbt_lib_t lib, const char *key);
+XBT_PUBLIC(void *) xbt_lib_get_level(xbt_dictelm_t elm, int level);
 
 #define xbt_lib_length(lib) xbt_dict_length((lib)->dict)
 
