@@ -1,6 +1,6 @@
 /* xbt/mallocator.h -- api to recycle allocated objects                     */
 
-/* Copyright (c) 2006, 2007, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2006-2007, 2009-2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -53,7 +53,7 @@ XBT_PUBLIC(void *) xbt_mallocator_get(xbt_mallocator_t mallocator);
 XBT_PUBLIC(void) xbt_mallocator_release(xbt_mallocator_t mallocator,
                                         void *object);
 
-XBT_PUBLIC(void) xbt_mallocator_initialization_is_done(void);
+XBT_PUBLIC(void) xbt_mallocator_initialization_is_done(int protect);
 /** @} */
 
 SG_END_DECL()
