@@ -1,7 +1,7 @@
 /* Copyright (C) 1991, 1992 Free Software Foundation, Inc.
    This file was then part of the GNU C Library. */
 
-/* Copyright (c) 2010-2012. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -19,6 +19,7 @@
 #endif
 
 #include "xbt/dynar.h"
+#include "xbt/dict.h"
 
 /* Datatype representing a separate heap. The whole point of the mmalloc module
  * is to allow several such heaps in the process. It thus works by redefining
@@ -30,46 +31,39 @@
 typedef struct mdesc *xbt_mheap_t;
 
 /* Allocate SIZE bytes of memory (and memset it to 0).  */
-extern void *mmalloc(xbt_mheap_t md, size_t size);
+XBT_PUBLIC( void ) *mmalloc(xbt_mheap_t md, size_t size);
 
 /* Allocate SIZE bytes of memory (and don't mess with it) */
 void *mmalloc_no_memset(xbt_mheap_t mdp, size_t size);
 
 /* Re-allocate the previously allocated block in void*, making the new block
    SIZE bytes long.  */
-extern void *mrealloc(xbt_mheap_t md, void *ptr, size_t size);
+XBT_PUBLIC( void ) *mrealloc(xbt_mheap_t md, void *ptr, size_t size);
 
 /* Free a block allocated by `mmalloc', `mrealloc' or `mcalloc'.  */
-extern void mfree(xbt_mheap_t md, void *ptr);
+XBT_PUBLIC( void ) mfree(xbt_mheap_t md, void *ptr);
 
-/* Allocate SIZE bytes allocated to ALIGNMENT bytes.  */
-extern void *mmemalign(xbt_mheap_t md, size_t alignment, size_t size);
+XBT_PUBLIC( xbt_mheap_t ) xbt_mheap_new(int fd, void *baseaddr);
 
-/* Allocate SIZE bytes on a page boundary.  */
-extern void *mvalloc(xbt_mheap_t md, size_t size);
+XBT_PUBLIC( void ) xbt_mheap_destroy_no_free(xbt_mheap_t md);
 
-extern xbt_mheap_t xbt_mheap_new(int fd, void *baseaddr);
-
-extern void xbt_mheap_destroy_no_free(xbt_mheap_t md);
-
-extern void *xbt_mheap_destroy(xbt_mheap_t md);
+XBT_PUBLIC( void ) *xbt_mheap_destroy(xbt_mheap_t md);
 
 /* return the heap used when NULL is passed as first argument to any mm* function */
-extern xbt_mheap_t mmalloc_get_default_md(void);
+XBT_PUBLIC( xbt_mheap_t ) mmalloc_get_default_md(void);
 
 /* To change the heap used when using the legacy version malloc/free/realloc and such */
 void mmalloc_set_current_heap(xbt_mheap_t new_heap);
 xbt_mheap_t mmalloc_get_current_heap(void);
 
-int mmalloc_compare_heap(xbt_mheap_t heap1, xbt_mheap_t heap2, xbt_dynar_t *stacks1, xbt_dynar_t *stacks2, xbt_dynar_t *equals);
+int mmalloc_compare_heap(xbt_mheap_t heap1, xbt_mheap_t heap2, xbt_dict_t all_types, xbt_dict_t other_types);
 int mmalloc_linear_compare_heap(xbt_mheap_t heap1, xbt_mheap_t heap2);
+int init_heap_information(xbt_mheap_t heap1, xbt_mheap_t heap2, xbt_dynar_t to_ignore1, xbt_dynar_t to_ignore2);
+int compare_heap_area(void *area1, void* area2, xbt_dynar_t previous, xbt_dict_t all_types, xbt_dict_t other_types, char *type, int pointer_level);
+void reset_heap_information(void);
+int get_pointed_area_size(void *area, int heap);
 
-void mmalloc_backtrace_block_display(void* heapinfo, int block);
-void mmalloc_backtrace_fragment_display(void* heapinfo, int block, int frag);
-void mmalloc_backtrace_display(void *addr);
-
-int is_free_area(void *area, xbt_mheap_t heap);
-
-
+size_t mmalloc_get_bytes_used(xbt_mheap_t);
+ssize_t mmalloc_get_busy_size(xbt_mheap_t, void *ptr);
 
 #endif                          /* MMALLOC_H */
