@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -62,10 +62,10 @@ static int l_comm_wait(lua_State* L) {
     timeout = luaL_checknumber(L, 2);
   }
                                   /* comm ... */
-  MSG_error_t res = MSG_comm_wait(comm, timeout);
+  msg_error_t res = MSG_comm_wait(comm, timeout);
 
   if (res == MSG_OK) {
-    m_task_t task = MSG_comm_get_task(comm);
+    msg_task_t task = MSG_comm_get_task(comm);
     if (MSG_task_get_sender(task) == MSG_process_self()) {
       /* I'm the sender */
       return 0;
@@ -111,10 +111,10 @@ static int l_comm_test(lua_State* L) {
   }
   else {
     /* finished but may have failed */
-    MSG_error_t res = MSG_comm_get_status(comm);
+    msg_error_t res = MSG_comm_get_status(comm);
 
     if (res == MSG_OK) {
-      m_task_t task = MSG_comm_get_task(comm);
+      msg_task_t task = MSG_comm_get_task(comm);
       if (MSG_task_get_sender(task) == MSG_process_self()) {
         /* I'm the sender */
         lua_pushboolean(L, 1);
