@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -29,7 +29,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(lua_process, bindings, "Lua Bindings (process mo
 static int l_process_sleep(lua_State* L)
 {
   double duration = luaL_checknumber(L, 1);
-  MSG_error_t res = MSG_process_sleep(duration);
+  msg_error_t res = MSG_process_sleep(duration);
 
   switch (res) {
 
