@@ -4,28 +4,8 @@
 #ifndef SIMGRID_LUA_H
 #define SIMGRID_LUA_H
 
-#include "xbt/dict.h"
-#include "xbt/dynar.h"
 #include <lua.h>
 
-extern xbt_dict_t process_function_set;
-extern xbt_dynar_t process_list;
-extern xbt_dict_t machine_set;
-
-typedef struct s_process_t {
-  int argc;
-  char **argv;
-  char *host;
-} s_process_t;
-
-void s_process_free(void *process);
-
-/* UNIX files */
-void generate_sim(const char *project);
-void generate_rl(const char *project);
-void generate_makefile_am(const char *project);
-void generate_makefile_local(const char *project);
-
 /* ********************************************************************************* */
 /*                           Plaftorm functions                                      */
 /* ********************************************************************************* */
