@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -23,11 +23,13 @@ typedef struct surf_model *surf_model_t;
  */
 typedef struct surf_action *surf_action_t;
 typedef struct surf_file *surf_file_t;
+typedef struct surf_storage *surf_storage_t;
 typedef struct surf_stat *surf_stat_t;
 
 typedef struct lmm_element *lmm_element_t;
 typedef struct lmm_variable *lmm_variable_t;
 typedef struct lmm_constraint *lmm_constraint_t;
+typedef struct lmm_constraint_light *lmm_constraint_light_t;
 typedef struct lmm_system *lmm_system_t;
 
 typedef struct tmgr_history *tmgr_history_t;
