@@ -1,10 +1,13 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
   * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "instr/instr_private.h"
+#include "xbt/lib.h"
+#include "surf/surf.h"
+#include "surf/surf_routing.h"
 
 #ifdef HAVE_TRACING
 
@@ -104,6 +107,7 @@ container_t PJ_container_new (const char *name, e_container_types kind, containe
       case INSTR_ROUTER:      snprintf (typename, INSTR_DEFAULT_STR_SIZE, "ROUTER");      break;
       case INSTR_SMPI:        snprintf (typename, INSTR_DEFAULT_STR_SIZE, "MPI");         break;
       case INSTR_MSG_PROCESS: snprintf (typename, INSTR_DEFAULT_STR_SIZE, "MSG_PROCESS"); break;
+      case INSTR_MSG_VM: snprintf (typename, INSTR_DEFAULT_STR_SIZE, "MSG_VM"); break;
       case INSTR_MSG_TASK:    snprintf (typename, INSTR_DEFAULT_STR_SIZE, "MSG_TASK");    break;
       default: THROWF (tracing_error, 0, "new container kind is unknown."); break;
     }
@@ -225,7 +229,7 @@ void PJ_container_free_all ()
   rootContainer = NULL;
 
   //checks
-  if (xbt_dict_length(allContainers) != 0){
+  if (!xbt_dict_is_empty(allContainers)){
     THROWF(tracing_error, 0, "some containers still present even after destroying all of them");
   }
 }
