@@ -1,4 +1,4 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -12,7 +12,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(instr_paje_header, instr, "Paje tracing event sy
 
 extern FILE *tracing_file;
 
-static void TRACE_header_PajeDefineContainerType (int basic)
+static void TRACE_header_PajeDefineContainerType (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineContainerType %d\n", PAJE_DefineContainerType);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -25,7 +25,7 @@ static void TRACE_header_PajeDefineContainerType (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDefineVariableType (int basic)
+static void TRACE_header_PajeDefineVariableType (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineVariableType %d\n", PAJE_DefineVariableType);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -39,7 +39,7 @@ static void TRACE_header_PajeDefineVariableType (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDefineStateType (int basic)
+static void TRACE_header_PajeDefineStateType (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineStateType %d\n", PAJE_DefineStateType);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -52,7 +52,7 @@ static void TRACE_header_PajeDefineStateType (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDefineEventType (int basic)
+static void TRACE_header_PajeDefineEventType (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineEventType %d\n", PAJE_DefineEventType);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -65,7 +65,7 @@ static void TRACE_header_PajeDefineEventType (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDefineLinkType (int basic)
+static void TRACE_header_PajeDefineLinkType (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineLinkType %d\n", PAJE_DefineLinkType);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -82,7 +82,7 @@ static void TRACE_header_PajeDefineLinkType (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDefineEntityValue (int basic)
+static void TRACE_header_PajeDefineEntityValue (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDefineEntityValue %d\n", PAJE_DefineEntityValue);
   fprintf(tracing_file, "%%       Alias string\n");
@@ -96,7 +96,7 @@ static void TRACE_header_PajeDefineEntityValue (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeCreateContainer (int basic)
+static void TRACE_header_PajeCreateContainer (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeCreateContainer %d\n", PAJE_CreateContainer);
   fprintf(tracing_file, "%%       Time date\n");
@@ -107,7 +107,7 @@ static void TRACE_header_PajeCreateContainer (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeDestroyContainer (int basic)
+static void TRACE_header_PajeDestroyContainer (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeDestroyContainer %d\n", PAJE_DestroyContainer);
   fprintf(tracing_file, "%%       Time date\n");
@@ -116,7 +116,7 @@ static void TRACE_header_PajeDestroyContainer (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeSetVariable (int basic)
+static void TRACE_header_PajeSetVariable (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeSetVariable %d\n", PAJE_SetVariable);
   fprintf(tracing_file, "%%       Time date\n");
@@ -126,7 +126,7 @@ static void TRACE_header_PajeSetVariable (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeAddVariable (int basic)
+static void TRACE_header_PajeAddVariable (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeAddVariable %d\n", PAJE_AddVariable);
   fprintf(tracing_file, "%%       Time date\n");
@@ -136,7 +136,7 @@ static void TRACE_header_PajeAddVariable (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeSubVariable (int basic)
+static void TRACE_header_PajeSubVariable (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeSubVariable %d\n", PAJE_SubVariable);
   fprintf(tracing_file, "%%       Time date\n");
@@ -147,7 +147,7 @@ static void TRACE_header_PajeSubVariable (int basic)
 }
 
 
-static void TRACE_header_PajeSetState (int basic)
+static void TRACE_header_PajeSetState (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeSetState %d\n", PAJE_SetState);
   fprintf(tracing_file, "%%       Time date\n");
@@ -157,17 +157,18 @@ static void TRACE_header_PajeSetState (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajePushState (int basic)
+static void TRACE_header_PajePushState (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajePushState %d\n", PAJE_PushState);
   fprintf(tracing_file, "%%       Time date\n");
   fprintf(tracing_file, "%%       Type string\n");
   fprintf(tracing_file, "%%       Container string\n");
   fprintf(tracing_file, "%%       Value string\n");
+  if (size) fprintf(tracing_file, "%%       Size int\n");
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajePopState (int basic)
+static void TRACE_header_PajePopState (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajePopState %d\n", PAJE_PopState);
   fprintf(tracing_file, "%%       Time date\n");
@@ -176,7 +177,7 @@ static void TRACE_header_PajePopState (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeResetState (int basic)
+static void TRACE_header_PajeResetState (int basic, int size)
 {
   if (basic) return;
 
@@ -187,7 +188,7 @@ static void TRACE_header_PajeResetState (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeStartLink (int basic)
+static void TRACE_header_PajeStartLink (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeStartLink %d\n", PAJE_StartLink);
   fprintf(tracing_file, "%%       Time date\n");
@@ -200,10 +201,11 @@ static void TRACE_header_PajeStartLink (int basic)
     fprintf(tracing_file, "%%       StartContainer string\n");
   }
   fprintf(tracing_file, "%%       Key string\n");
+  if (size) fprintf(tracing_file, "%%       Size int\n");
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeEndLink (int basic)
+static void TRACE_header_PajeEndLink (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeEndLink %d\n", PAJE_EndLink);
   fprintf(tracing_file, "%%       Time date\n");
@@ -219,7 +221,7 @@ static void TRACE_header_PajeEndLink (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-static void TRACE_header_PajeNewEvent (int basic)
+static void TRACE_header_PajeNewEvent (int basic, int size)
 {
   fprintf(tracing_file, "%%EventDef PajeNewEvent %d\n", PAJE_NewEvent);
   fprintf(tracing_file, "%%       Time date\n");
@@ -229,29 +231,28 @@ static void TRACE_header_PajeNewEvent (int basic)
   fprintf(tracing_file, "%%EndEventDef\n");
 }
 
-void TRACE_header(int basic)
+void TRACE_header(int basic, int size)
 {
   XBT_DEBUG ("Define paje header");
-  TRACE_header_PajeDefineContainerType (basic);
-  TRACE_header_PajeDefineVariableType (basic);
-  TRACE_header_PajeDefineStateType (basic);
-  TRACE_header_PajeDefineEventType (basic);
-  TRACE_header_PajeDefineLinkType (basic);
-  TRACE_header_PajeDefineEntityValue (basic);
-  TRACE_header_PajeCreateContainer (basic);
-  TRACE_header_PajeDestroyContainer (basic);
-  TRACE_header_PajeSetVariable (basic);
-  TRACE_header_PajeAddVariable (basic);
-  TRACE_header_PajeSubVariable (basic);
-  TRACE_header_PajeSetState (basic);
-  TRACE_header_PajePushState (basic);
-  TRACE_header_PajePopState (basic);
-  TRACE_header_PajeResetState (basic);
-  TRACE_header_PajeStartLink (basic);
-  TRACE_header_PajeEndLink (basic);
-  TRACE_header_PajeNewEvent (basic);
+  TRACE_header_PajeDefineContainerType (basic, size);
+  TRACE_header_PajeDefineVariableType (basic, size);
+  TRACE_header_PajeDefineStateType (basic, size);
+  TRACE_header_PajeDefineEventType (basic, size);
+  TRACE_header_PajeDefineLinkType (basic, size);
+  TRACE_header_PajeDefineEntityValue (basic, size);
+  TRACE_header_PajeCreateContainer (basic, size);
+  TRACE_header_PajeDestroyContainer (basic, size);
+  TRACE_header_PajeSetVariable (basic, size);
+  TRACE_header_PajeAddVariable (basic, size);
+  TRACE_header_PajeSubVariable (basic, size);
+  TRACE_header_PajeSetState (basic, size);
+  TRACE_header_PajePushState (basic, size);
+  TRACE_header_PajePopState (basic, size);
+  TRACE_header_PajeResetState (basic, size);
+  TRACE_header_PajeStartLink (basic, size);
+  TRACE_header_PajeEndLink (basic, size);
+  TRACE_header_PajeNewEvent (basic, size);
 }
 
 #endif
 
-
