@@ -1,4 +1,4 @@
-/* Copyright (c) 2012. The SimGrid Team.
+/* Copyright (c) 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -30,16 +30,34 @@ val_t PJ_value_new (const char *name, const char *color, type_t father)
   return ret;
 }
 
+val_t PJ_value_get_or_new (const char *name, const char *color, type_t father)
+{
+  val_t ret = 0;
+  xbt_ex_t e;
+  TRY {
+    ret = PJ_value_get(name, father);
+  }
+  CATCH(e) {
+    xbt_ex_free(e);
+    ret = PJ_value_new(name, color, father);
+  }
+  return ret;
+}
+
 val_t PJ_value_get (const char *name, type_t father)
 {
   if (name == NULL || father == NULL){
     THROWF (tracing_error, 0, "can't get a value with a NULL name (or a NULL father)");
   }
 
-  if (father->kind == TYPE_VARIABLE) return NULL; //Variables can't have different values
+  if (father->kind == TYPE_VARIABLE)
+    THROWF(tracing_error, 0,
+           "variables can't have different values (%s)", father->name);
   val_t ret = (val_t)xbt_dict_get_or_null (father->values, name);
-  if (ret == NULL){
-    return NULL;
+  if (ret == NULL) {
+    THROWF(tracing_error, 2,
+           "value with name (%s) not found in father type (%s)",
+           name, father->name);
   }
   return ret;
 }
