@@ -53,7 +53,7 @@ void jed_event_add_info(jed_event_t event, char *key, char *value) {
 void create_jed_event(jed_event_t *event, char *name, double start_time,
     double end_time, const char *type) {
 
-  *event = (jed_event_t) calloc(1, sizeof(s_jed_event_t));
+  *event = xbt_new0(s_jed_event_t,1);
   (*event)->name = xbt_strdup(name);
 
   (*event)->start_time = start_time;
@@ -61,7 +61,7 @@ void create_jed_event(jed_event_t *event, char *name, double start_time,
 
   (*event)->type = xbt_strdup(type);
 
-  (*event)->resource_subsets = xbt_dynar_new(sizeof(jed_res_subset_t), NULL);
+  (*event)->resource_subsets = xbt_dynar_new(sizeof(jed_res_subset_t), xbt_free_ref);
   (*event)->characteristics_list = xbt_dynar_new(sizeof(char*), NULL);
   (*event)->info_hash = xbt_dict_new_homogeneous(NULL);
 
