@@ -1,4 +1,5 @@
-/* Copyright (c) 2007-2012 Da SimGrid Team. All rights reserved.            */
+/* Copyright (c) 2007-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -8,7 +9,9 @@
 
 #include "simgrid_config.h"
 #include <stdio.h>
+#ifndef WIN32
 #include <sys/mman.h>
+#endif
 #include "mc/mc.h"
 #include "mc/datatypes.h"
 #include "xbt/fifo.h"
@@ -21,50 +24,76 @@
 #include "msg/msg.h"
 #include "msg/datatypes.h"
 #include "xbt/strbuff.h"
+#include "xbt/parmap.h"
 
 /****************************** Snapshots ***********************************/
 
+#define NB_REGIONS 3 /* binary data (data + BSS) (type = 2), libsimgrid data (data + BSS) (type = 1), std_heap (type = 0)*/ 
+
 typedef struct s_mc_mem_region{
-  int type;
   void *start_addr;
   void *data;
   size_t size;
 } s_mc_mem_region_t, *mc_mem_region_t;
 
 typedef struct s_mc_snapshot{
-  unsigned int num_reg;
-  mc_mem_region_t *regions;
+  size_t heap_bytes_used;
+  mc_mem_region_t regions[NB_REGIONS];
+  int nb_processes;
+  size_t *stack_sizes;
   xbt_dynar_t stacks;
+  xbt_dynar_t to_ignore;
+  char hash_global[41];
+  char hash_local[41];
 } s_mc_snapshot_t, *mc_snapshot_t;
 
 typedef struct s_mc_snapshot_stack{
-  xbt_strbuff_t local_variables;
+  xbt_dynar_t local_variables;
   void *stack_pointer;
+  void *real_address;
 }s_mc_snapshot_stack_t, *mc_snapshot_stack_t;
 
-void MC_take_snapshot(mc_snapshot_t);
-void MC_take_snapshot_liveness(mc_snapshot_t s);
+typedef struct s_mc_global_t{
+  mc_snapshot_t snapshot;
+  int raw_mem_set;
+  int prev_pair;
+  char *prev_req;
+}s_mc_global_t, *mc_global_t;
+
+typedef struct s_mc_checkpoint_ignore_region{
+  void *addr;
+  size_t size;
+}s_mc_checkpoint_ignore_region_t, *mc_checkpoint_ignore_region_t;
+
+mc_snapshot_t SIMIX_pre_mc_snapshot(smx_simcall_t simcall);
+mc_snapshot_t MC_take_snapshot(int num_state);
 void MC_restore_snapshot(mc_snapshot_t);
 void MC_free_snapshot(mc_snapshot_t);
 
+extern xbt_dynar_t mc_checkpoint_ignore;
 
 /********************************* MC Global **********************************/
-extern double *mc_time;
 
-/* Bound of the MC depth-first search algorithm */
-#define MAX_DEPTH 1000
-#define MAX_DEPTH_LIVENESS 500
+extern double *mc_time;
+extern FILE *dot_output;
+extern const char* colors[13];
+extern xbt_parmap_t parmap;
+
+extern int user_max_depth_reached;
 
 int MC_deadlock_check(void);
 void MC_replay(xbt_fifo_t stack, int start);
 void MC_replay_liveness(xbt_fifo_t stack, int all_stack);
 void MC_wait_for_requests(void);
-void MC_get_enabled_processes();
 void MC_show_deadlock(smx_simcall_t req);
 void MC_show_stack_safety(xbt_fifo_t stack);
 void MC_dump_stack_safety(xbt_fifo_t stack);
+void MC_init(void);
+int SIMIX_pre_mc_random(smx_simcall_t simcall, int min, int max);
+
 
 /********************************* Requests ***********************************/
+
 int MC_request_depend(smx_simcall_t req1, smx_simcall_t req2);
 char* MC_request_to_string(smx_simcall_t req, int value);
 unsigned int MC_request_testany_fail(smx_simcall_t req);
@@ -73,13 +102,16 @@ int MC_request_is_visible(smx_simcall_t req);
 int MC_request_is_enabled(smx_simcall_t req);
 int MC_request_is_enabled_by_idx(smx_simcall_t req, unsigned int idx);
 int MC_process_is_enabled(smx_process_t process);
+char *MC_request_get_dot_output(smx_simcall_t req, int value);
 
 
 /******************************** States **************************************/
+
 /* Possible exploration status of a process in a state */
 typedef enum {
   MC_NOT_INTERLEAVE=0,      /* Do not interleave (do not execute) */
   MC_INTERLEAVE,            /* Interleave the process (one or more request) */
+  MC_MORE_INTERLEAVE,       /* Interleave twice the process (for mc_random simcall) */
   MC_DONE                   /* Already interleaved */
 } e_mc_process_state_t;
 
@@ -100,10 +132,9 @@ typedef struct mc_state {
   int req_num;                      /* The request number (in the case of a
                                        multi-request like waitany ) */
   mc_snapshot_t system_state;      /* Snapshot of system state */
+  int num;
 } s_mc_state_t, *mc_state_t;
 
-extern xbt_fifo_t mc_stack_safety_stateless;
-
 mc_state_t MC_state_new(void);
 void MC_state_delete(mc_state_t state);
 void MC_state_interleave_process(mc_state_t state, smx_process_t process);
@@ -113,27 +144,24 @@ void MC_state_set_executed_request(mc_state_t state, smx_simcall_t req, int valu
 smx_simcall_t MC_state_get_executed_request(mc_state_t state, int *value);
 smx_simcall_t MC_state_get_internal_request(mc_state_t state);
 smx_simcall_t MC_state_get_request(mc_state_t state, int *value);
+void MC_state_remove_interleave_process(mc_state_t state, smx_process_t process);
+
 
 /****************************** Statistics ************************************/
+
 typedef struct mc_stats {
   unsigned long state_size;
   unsigned long visited_states;
+  unsigned long visited_pairs;
   unsigned long expanded_states;
+  unsigned long expanded_pairs;
   unsigned long executed_transitions;
 } s_mc_stats_t, *mc_stats_t;
 
-typedef struct mc_stats_pair {
-  //unsigned long pair_size;
-  unsigned long visited_pairs;
-  unsigned long expanded_pairs;
-  unsigned long executed_transitions;
-} s_mc_stats_pair_t, *mc_stats_pair_t;
-
 extern mc_stats_t mc_stats;
-extern mc_stats_pair_t mc_stats_pair;
 
 void MC_print_statistics(mc_stats_t);
-void MC_print_statistics_pairs(mc_stats_pair_t);
+
 
 /********************************** MEMORY ******************************/
 /* The possible memory modes for the modelchecker are standard and raw. */
@@ -159,7 +187,6 @@ extern void *raw_heap;
 #define MC_SET_RAW_MEM    mmalloc_set_current_heap(raw_heap)
 #define MC_UNSET_RAW_MEM  mmalloc_set_current_heap(std_heap)
 
-extern int raw_mem_set;
 
 /******************************* MEMORY MAPPINGS ***************************/
 /* These functions and data structures implements a binary interface for   */
@@ -171,7 +198,7 @@ typedef struct s_map_region {
   void *start_addr;             /* Start address of the map */
   void *end_addr;               /* End address of the map */
   int prot;                     /* Memory protection */
-  int flags;                    /* Aditional memory flags */
+  int flags;                    /* Additional memory flags */
   void *offset;                 /* Offset in the file/whatever */
   char dev_major;               /* Major of the device */
   char dev_minor;               /* Minor of the device */
@@ -187,14 +214,57 @@ typedef struct s_memory_map {
 
 } s_memory_map_t, *memory_map_t;
 
-memory_map_t get_memory_map(void);
-void free_memory_map(memory_map_t map);
-void get_libsimgrid_plt_section(void);
-void get_binary_plt_section(void);
+
+void MC_init_memory_map_info(void);
+memory_map_t MC_get_memory_map(void);
+void MC_free_memory_map(memory_map_t map);
+void MC_get_libsimgrid_plt_section(void);
+void MC_get_binary_plt_section(void);
 
 extern void *start_data_libsimgrid;
+extern void *start_data_binary;
+extern void *start_bss_binary;
+extern char *libsimgrid_path;
+extern void *start_text_libsimgrid;
+extern void *start_text_binary;
+extern void *start_bss_libsimgrid;
+extern void *start_plt_libsimgrid;
+extern void *end_plt_libsimgrid;
+extern void *start_plt_binary;
+extern void *end_plt_binary;
+extern void *start_got_plt_libsimgrid;
+extern void *end_got_plt_libsimgrid;
+extern void *start_got_plt_binary;
+extern void *end_got_plt_binary;
+
+
+/********************************** Snapshot comparison **********************************/
+
+typedef struct s_mc_comparison_times{
+  double nb_processes_comparison_time;
+  double bytes_used_comparison_time;
+  double stacks_sizes_comparison_time;
+  double binary_global_variables_comparison_time;
+  double libsimgrid_global_variables_comparison_time;
+  double heap_comparison_time;
+  double stacks_comparison_time;
+  double hash_global_variables_comparison_time;
+  double hash_local_variables_comparison_time;
+}s_mc_comparison_times_t, *mc_comparison_times_t;
+
+extern __thread mc_comparison_times_t mc_comp_times;
+extern __thread double mc_snapshot_comparison_time;
+
+int snapshot_compare(void *state1, void *state2);
+int SIMIX_pre_mc_compare_snapshots(smx_simcall_t simcall, mc_snapshot_t s1, mc_snapshot_t s2);
+void print_comparison_times(void);
+
+//#define MC_DEBUG 1
+#define MC_VERBOSE 1
+
+
+/********************************** DPOR for safety property **************************************/
 
-/********************************** DPOR for safety  **************************************/
 typedef enum {
   e_mc_reduce_unset,
   e_mc_reduce_none,
@@ -202,73 +272,73 @@ typedef enum {
 } e_mc_reduce_t;
 
 extern e_mc_reduce_t mc_reduce_kind;
+extern mc_global_t initial_state_safety;
+extern xbt_fifo_t mc_stack_safety;
+extern xbt_dict_t first_enabled_state;
 
 void MC_dpor_init(void);
 void MC_dpor(void);
-void MC_dpor_exit(void);
-void MC_init_safety(void);
+
+typedef struct s_mc_visited_state{
+  mc_snapshot_t system_state;
+  size_t heap_bytes_used;
+  int nb_processes;
+  int num;
+  int other_num; // dot_output for
+}s_mc_visited_state_t, *mc_visited_state_t;
 
 
-/********************************** Double-DFS for liveness property**************************************/
+/********************************** Double-DFS for liveness property **************************************/
 
 extern xbt_fifo_t mc_stack_liveness;
-extern mc_snapshot_t initial_snapshot_liveness;
+extern mc_global_t initial_state_liveness;
 extern xbt_automaton_t _mc_property_automaton;
 extern int compare;
-extern void *start_plt_libsimgrid;
-extern void *end_plt_libsimgrid;
-extern void *start_plt_binary;
-extern void *end_plt_binary;
+extern xbt_dynar_t mc_stack_comparison_ignore;
+extern xbt_dynar_t mc_data_bss_comparison_ignore;
 
 typedef struct s_mc_pair{
-  mc_snapshot_t system_state;
-  mc_state_t graph_state;
-  xbt_state_t automaton_state;
+  int num;
+  int search_cycle;
+  mc_state_t graph_state; /* System state included */
+  xbt_automaton_state_t automaton_state;
+  xbt_dynar_t atomic_propositions;
+  int requests;
 }s_mc_pair_t, *mc_pair_t;
 
-typedef struct s_mc_pair_reached{
-  int nb;
-  xbt_state_t automaton_state;
-  xbt_dynar_t prop_ato;
-  mc_snapshot_t system_state;
-}s_mc_pair_reached_t, *mc_pair_reached_t;
+typedef struct s_mc_visited_pair{
+  int num;
+  int other_num; /* Dot output for */
+  int acceptance_pair;
+  mc_state_t graph_state; /* System state included */
+  xbt_automaton_state_t automaton_state;
+  xbt_dynar_t atomic_propositions;
+  size_t heap_bytes_used;
+  int nb_processes;
+  int acceptance_removed;
+  int visited_removed;
+}s_mc_visited_pair_t, *mc_visited_pair_t;
 
-int MC_automaton_evaluate_label(xbt_exp_label_t l);
-mc_pair_t new_pair(mc_snapshot_t sn, mc_state_t sg, xbt_state_t st);
+mc_pair_t MC_pair_new(void);
+void MC_pair_delete(mc_pair_t);
+void mc_pair_free_voidp(void *p);
+mc_visited_pair_t MC_visited_pair_new(int pair_num, xbt_automaton_state_t automaton_state, xbt_dynar_t atomic_propositions);
+void MC_visited_pair_delete(mc_visited_pair_t p);
 
-int reached(xbt_state_t st);
-void set_pair_reached(xbt_state_t st);
-int snapshot_compare(mc_snapshot_t s1, mc_snapshot_t s2);
-void MC_pair_delete(mc_pair_t pair);
-void MC_exit_liveness(void);
-mc_state_t MC_state_pair_new(void);
-
-/* **** Double-DFS stateless **** */
-
-typedef struct s_mc_pair_stateless{
-  mc_state_t graph_state;
-  xbt_state_t automaton_state;
-  int requests;
-}s_mc_pair_stateless_t, *mc_pair_stateless_t;
-
-mc_pair_stateless_t new_pair_stateless(mc_state_t sg, xbt_state_t st, int r);
 void MC_ddfs_init(void);
-void MC_ddfs(int search_cycle);
+void MC_ddfs(void);
 void MC_show_stack_liveness(xbt_fifo_t stack);
 void MC_dump_stack_liveness(xbt_fifo_t stack);
-void MC_pair_stateless_delete(mc_pair_stateless_t pair);
 
-/********************************** Configuration of MC **************************************/
-extern xbt_fifo_t mc_stack_safety;
 
-extern int _surf_mc_checkpoint;
-extern char* _surf_mc_property_file;
+/********************************** Variables with DWARF **********************************/
 
-/****** Core dump ******/
-
-int create_dump(int pair);
-
-/****** Local variables with DWARF ******/
+extern xbt_dict_t mc_local_variables_libsimgrid;
+extern xbt_dict_t mc_local_variables_binary;
+extern xbt_dynar_t mc_global_variables_libsimgrid;
+extern xbt_dynar_t mc_global_variables_binary;
+extern xbt_dict_t mc_variables_type_libsimgrid;
+extern xbt_dict_t mc_variables_type_binary;
 
 typedef enum {
   e_dw_loclist,
@@ -332,37 +402,36 @@ typedef struct s_dw_location_entry{
   dw_location_t location;
 }s_dw_location_entry_t, *dw_location_entry_t;
 
-typedef struct s_dw_local_variable{
+typedef struct s_dw_variable{
+  int global;
   char *name;
-  dw_location_t location;
-}s_dw_local_variable_t, *dw_local_variable_t;
+  char *type_origin;
+  union{
+    dw_location_t location;
+    void *address;
+  }address;
+}s_dw_variable_t, *dw_variable_t;
 
 typedef struct s_dw_frame{
   char *name;
   void *low_pc;
   void *high_pc;
   dw_location_t frame_base;
-  xbt_dict_t variables;
+  xbt_dynar_t variables; /* Cannot use dict, there may be several variables with the same name (in different lexical blocks)*/
   unsigned long int start;
   unsigned long int end;
 }s_dw_frame_t, *dw_frame_t;
 
-/* FIXME : implement free functions for each structure */
+/********************************** Miscellaneous **********************************/
 
-extern xbt_dict_t mc_local_variables;
-
-typedef struct s_variable_value{
+typedef struct s_local_variable{
+  char *frame;
+  unsigned long ip;
+  char *name;
   char *type;
-  
-  union{
-    void *address;
-    long int res;
-  }value;
-}s_variable_value_t, *variable_value_t;
-
-void MC_get_local_variables(const char *elf_file, xbt_dict_t location_list, xbt_dict_t *variables);
-void print_local_variables(xbt_dict_t list);
-char *get_libsimgrid_path(void);
-xbt_dict_t MC_get_location_list(const char *elf_file);
+  void *address;
+  int region;
+}s_local_variable_t, *local_variable_t;
 
 #endif
+
