@@ -1,4 +1,5 @@
-/* Copyright (c) 2008-2012 Da SimGrid Team. All rights reserved.            */
+/* Copyright (c) 2008-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -18,20 +19,8 @@ mc_state_t MC_state_new()
   state->max_pid = simix_process_maxpid;
   state->proc_status = xbt_new0(s_mc_procstate_t, state->max_pid);
   state->system_state = NULL;
-
-  mc_stats->expanded_states++;
-  return state;
-}
-
-mc_state_t MC_state_pair_new(void)
-{
-  mc_state_t state = NULL;
+  state->num = ++mc_stats->expanded_states;
   
-  state = xbt_new0(s_mc_state_t, 1);
-  state->max_pid = simix_process_maxpid;
-  state->proc_status = xbt_new0(s_mc_procstate_t, state->max_pid);
-  
-  //mc_stats->expanded_states++;
   return state;
 }
 
@@ -41,6 +30,8 @@ mc_state_t MC_state_pair_new(void)
  */
 void MC_state_delete(mc_state_t state)
 {
+  if(state->system_state)
+    MC_free_snapshot(state->system_state);
   xbt_free(state->proc_status);
   xbt_free(state);
 }
@@ -51,12 +42,18 @@ void MC_state_interleave_process(mc_state_t state, smx_process_t process)
   state->proc_status[process->pid].interleave_count = 0;
 }
 
+void MC_state_remove_interleave_process(mc_state_t state, smx_process_t process)
+{
+  if(state->proc_status[process->pid].state == MC_INTERLEAVE)
+    state->proc_status[process->pid].state = MC_DONE;
+}
+
 unsigned int MC_state_interleave_size(mc_state_t state)
 {
   unsigned int i, size=0;
 
   for(i=0; i < state->max_pid; i++){
-    if(state->proc_status[i].state == MC_INTERLEAVE)
+    if((state->proc_status[i].state == MC_INTERLEAVE) || (state->proc_status[i].state == MC_MORE_INTERLEAVE))
       size++;
   }
 
@@ -71,6 +68,8 @@ void MC_state_set_executed_request(mc_state_t state, smx_simcall_t req, int valu
 {
   state->executed_req = *req;
   state->req_num = value;
+  smx_process_t process = NULL;
+  mc_procstate_t procstate = NULL;
 
   /* The waitany and testany request are transformed into a wait or test request over the
    * corresponding communication action so it can be treated later by the dependence
@@ -79,9 +78,9 @@ void MC_state_set_executed_request(mc_state_t state, smx_simcall_t req, int valu
     case SIMCALL_COMM_WAITANY:
       state->internal_req.call = SIMCALL_COMM_WAIT;
       state->internal_req.issuer = req->issuer;
-      state->internal_comm = *xbt_dynar_get_as(req->comm_waitany.comms, value, smx_action_t);
-      state->internal_req.comm_wait.comm = &state->internal_comm;
-      state->internal_req.comm_wait.timeout = 0;
+      state->internal_comm = *xbt_dynar_get_as(simcall_comm_waitany__get__comms(req), value, smx_action_t);
+      simcall_comm_wait__set__comm(&state->internal_req, &state->internal_comm);
+      simcall_comm_wait__set__timeout(&state->internal_req, 0);
       break;
 
     case SIMCALL_COMM_TESTANY:
@@ -89,24 +88,37 @@ void MC_state_set_executed_request(mc_state_t state, smx_simcall_t req, int valu
       state->internal_req.issuer = req->issuer;
 
       if(value > 0)
-        state->internal_comm = *xbt_dynar_get_as(req->comm_testany.comms, value, smx_action_t);
+        state->internal_comm = *xbt_dynar_get_as(simcall_comm_testany__get__comms(req), value, smx_action_t);
 
-      state->internal_req.comm_wait.comm = &state->internal_comm;
-      state->internal_req.comm_test.result = value;
+      simcall_comm_test__set__comm(&state->internal_req, &state->internal_comm);
+      simcall_comm_test__set__result(&state->internal_req, value);
       break;
 
     case SIMCALL_COMM_WAIT:
       state->internal_req = *req;
-      state->internal_comm = *(req->comm_wait.comm);
-      state->executed_req.comm_wait.comm = &state->internal_comm;
-      state->internal_req.comm_wait.comm = &state->internal_comm;
+      state->internal_comm = *(simcall_comm_wait__get__comm(req));
+      simcall_comm_wait__set__comm(&state->executed_req, &state->internal_comm);
+      simcall_comm_wait__set__comm(&state->internal_req, &state->internal_comm);
       break;
 
     case SIMCALL_COMM_TEST:
       state->internal_req = *req;
-      state->internal_comm = *req->comm_test.comm;
-      state->executed_req.comm_test.comm = &state->internal_comm;
-      state->internal_req.comm_test.comm = &state->internal_comm;
+      state->internal_comm = *simcall_comm_test__get__comm(req);
+      simcall_comm_test__set__comm(&state->executed_req, &state->internal_comm);
+      simcall_comm_test__set__comm(&state->internal_req, &state->internal_comm);
+      break;
+
+    case SIMCALL_MC_RANDOM:
+      state->internal_req = *req;
+      if(value != simcall_mc_random__get__max(req)){
+        xbt_swag_foreach(process, simix_global->process_list){
+          procstate = &state->proc_status[process->pid];
+          if(process->pid == req->issuer->pid){
+            procstate->state = MC_MORE_INTERLEAVE;
+            break;
+          }        
+        }
+      }
       break;
 
     default:
@@ -114,7 +126,7 @@ void MC_state_set_executed_request(mc_state_t state, smx_simcall_t req, int valu
       break;
   }
 }
-
+ 
 smx_simcall_t MC_state_get_executed_request(mc_state_t state, int *value)
 {
   *value = state->req_num;
@@ -135,19 +147,19 @@ smx_simcall_t MC_state_get_request(mc_state_t state, int *value)
   xbt_swag_foreach(process, simix_global->process_list){
     procstate = &state->proc_status[process->pid];
 
-    if(procstate->state == MC_INTERLEAVE){
+    if(procstate->state == MC_INTERLEAVE || procstate->state == MC_MORE_INTERLEAVE){
       if(MC_process_is_enabled(process)){
         switch(process->simcall.call){
           case SIMCALL_COMM_WAITANY:
             *value = -1;
-            while(procstate->interleave_count < xbt_dynar_length(process->simcall.comm_waitany.comms)){
+            while(procstate->interleave_count < xbt_dynar_length(simcall_comm_waitany__get__comms(&process->simcall))){
               if(MC_request_is_enabled_by_idx(&process->simcall, procstate->interleave_count++)){
                 *value = procstate->interleave_count-1;
                 break;
               }
             }
 
-            if(procstate->interleave_count >= xbt_dynar_length(process->simcall.comm_waitany.comms))
+            if(procstate->interleave_count >= xbt_dynar_length(simcall_comm_waitany__get__comms(&process->simcall)))
               procstate->state = MC_DONE;
 
             if(*value != -1)
@@ -158,14 +170,14 @@ smx_simcall_t MC_state_get_request(mc_state_t state, int *value)
           case SIMCALL_COMM_TESTANY:
             start_count = procstate->interleave_count;
             *value = -1;
-            while(procstate->interleave_count < xbt_dynar_length(process->simcall.comm_testany.comms)){
+            while(procstate->interleave_count < xbt_dynar_length(simcall_comm_testany__get__comms(&process->simcall))){
               if(MC_request_is_enabled_by_idx(&process->simcall, procstate->interleave_count++)){
                 *value = procstate->interleave_count - 1;
                 break;
               }
             }
 
-            if(procstate->interleave_count >= xbt_dynar_length(process->simcall.comm_testany.comms))
+            if(procstate->interleave_count >= xbt_dynar_length(simcall_comm_testany__get__comms(&process->simcall)))
               procstate->state = MC_DONE;
 
             if(*value != -1 || start_count == 0)
@@ -174,8 +186,8 @@ smx_simcall_t MC_state_get_request(mc_state_t state, int *value)
             break;
 
           case SIMCALL_COMM_WAIT:
-            if(process->simcall.comm_wait.comm->comm.src_proc
-               && process->simcall.comm_wait.comm->comm.dst_proc){
+            if(simcall_comm_wait__get__comm(&process->simcall)->comm.src_proc
+               && simcall_comm_wait__get__comm(&process->simcall)->comm.dst_proc){
               *value = 0;
             }else{
               *value = -1;
@@ -185,6 +197,17 @@ smx_simcall_t MC_state_get_request(mc_state_t state, int *value)
 
             break;
 
+          case SIMCALL_MC_RANDOM:
+            if(procstate->state == MC_INTERLEAVE)
+              *value = 0;
+            else{
+              if(state->req_num < simcall_mc_random__get__max(&process->simcall))
+                *value = state->req_num + 1;
+            }
+            procstate->state = MC_DONE;
+            return &process->simcall;
+            break;
+          
           default:
             procstate->state = MC_DONE;
             *value = 0;
