@@ -1,16 +1,16 @@
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "simgrid_config.h" //For getline, keep that include first
-
 #include "msg_private.h"
 #include "xbt/str.h"
 #include "xbt/dynar.h"
 #include "xbt/replay.h"
 
+#include <errno.h>
+
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(msg_action, msg,
                                 "MSG actions for trace driven simulation");
 
