@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -27,7 +27,7 @@ void MSG_launch_application(const char *file)
 {
 
   xbt_assert(msg_global,
-              "MSG_global_init_args has to be called before MSG_launch_application.");
+              "MSG_init has to be called before MSG_launch_application.");
 
   SIMIX_launch_application(file);
 
