@@ -1,9 +1,10 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+#include "instr/instr_interface.h"
 #include "msg_private.h"
 #include "msg_mailbox.h"
 #include "mc/mc.h"
@@ -12,6 +13,8 @@
 #include "xbt/virtu.h"
 #include "xbt/ex.h"             /* ex_backtrace_display */
 #include "xbt/replay.h"
+#include "simgrid/sg_config.h" /* Configuration mechanism of SimGrid */
+
 
 XBT_LOG_NEW_CATEGORY(msg, "All MSG categories");
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(msg_kernel, msg,
@@ -44,7 +47,6 @@ void MSG_init_nocheck(int *argc, char **argv) {
 #ifdef MSG_USE_DEPRECATED
     msg_global->max_channel = 0;
 #endif
-    msg_global->PID = 1;
     msg_global->sent_msg = 0;
     msg_global->task_copy_callback = NULL;
     msg_global->process_data_cleanup = NULL;
@@ -61,7 +63,7 @@ void MSG_init_nocheck(int *argc, char **argv) {
   
   if(MC_is_active()){
     /* Ignore total amount of messages sent during the simulation for heap comparison */
-    MC_ignore(&(msg_global->sent_msg), sizeof(msg_global->sent_msg));
+    MC_ignore_heap(&(msg_global->sent_msg), sizeof(msg_global->sent_msg));
   }
 
 #ifdef HAVE_TRACING
@@ -70,8 +72,9 @@ void MSG_init_nocheck(int *argc, char **argv) {
 
   XBT_DEBUG("ADD MSG LEVELS");
   MSG_HOST_LEVEL = xbt_lib_add_level(host_lib, (void_f_pvoid_t) __MSG_host_destroy);
+  MSG_STORAGE_LEVEL = xbt_lib_add_level(storage_lib, (void_f_pvoid_t) __MSG_storage_destroy);
 
-  atexit(MSG_exit);
+  if(sg_cfg_get_boolean("clean_atexit")) atexit(MSG_exit);
 }
 
 #ifdef MSG_USE_DEPRECATED
@@ -127,6 +130,20 @@ msg_error_t MSG_main(void)
   return MSG_OK;
 }
 
+/** \ingroup msg_simulation
+ * \brief set a configuration variable
+ *
+ * Do --help on any simgrid binary to see the list of currently existing configuration variables, and see Section @ref options.
+ *
+ * Example:
+ * MSG_config("workstation/model","ptask_L07");
+ */
+void MSG_config(const char *key, const char *value){
+  xbt_assert(msg_global,"ERROR: Please call MSG_init() before using MSG_config()");
+  xbt_cfg_set_as_string(_sg_cfg_set, key, value);
+}
+
+
 /** \ingroup msg_simulation
  * \brief Kill all running process
 
@@ -136,14 +153,12 @@ msg_error_t MSG_main(void)
  */
 int MSG_process_killall(int reset_PIDs)
 {
-  simcall_process_killall();
+  simcall_process_killall(reset_PIDs);
 
-  if (reset_PIDs > 0) {
-    msg_global->PID = reset_PIDs;
+  if (reset_PIDs > 0)
     msg_global->session++;
-  }
 
-  return msg_global->PID;
+  return 0;
 
 }
 
@@ -152,7 +167,7 @@ static void MSG_exit(void) {
     return;
 
 #ifdef HAVE_TRACING
-  TRACE_surf_release();
+  TRACE_surf_resource_utilization_release();
 #endif
 
   /* initialization of the action module */
