@@ -1,16 +1,16 @@
 /* portable -- header loading to write portable code                         */
 /* loads much more stuff than sysdep.h since the latter is in public interface*/
 
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2010, 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
   * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#ifndef GRAS_PORTABLE_H
-#define GRAS_PORTABLE_H
+#ifndef SIMGRID_PORTABLE_H
+#define SIMGRID_PORTABLE_H
 
-#include "gras_config.h"
+#include "internal_config.h"
 #include "xbt/misc.h"
 /* 
  * win32 or win64 (__XBT_WIN32 is defined for win32 and win64 applications, __TOS_WIN__ is defined by xlC).
@@ -38,34 +38,6 @@
 # include <sys/sysctl.h>
 #endif
 
-/****
- **** Networking 
- ****/
-
-
-#ifdef HAVE_SYS_SOCKET_H
-#  include <sys/socket.h>
-#  include <netinet/in.h>       /* sometimes required for #include <arpa/inet.h> */
-#  include <netinet/tcp.h>      /* TCP_NODELAY */
-#  include <netdb.h>            /* getprotobyname() */
-#  include <arpa/inet.h>        /* inet_ntoa() */
-#  include <sys/types.h>        /* sometimes required for fd_set */
-# endif
-
-
-#ifndef HAVE_WINSOCK_H
-#       define tcp_read( s, buf, len)   read( s, buf, len )
-#       define tcp_write( s, buf, len)  write( s, buf, len )
-#       define sock_errno        errno
-#       define sock_errstr(err)  strerror(err)
-
-#       ifdef SHUT_RDWR
-#               define tcp_close( s )   (shutdown( s, SHUT_RDWR ), close( s ))
-#       else
-#               define tcp_close( s )   close( s )
-#       endif
-#endif                          /* windows or unix ? */
-
 /****
  **** File handling
  ****/
@@ -154,10 +126,4 @@ extern int vasnprintf(char **ptr, size_t str_m, const char *fmt,
 #  include <execinfo.h>
 #endif
 
-/****
- **** Some debugging functions. Can't we find a better place for this??
- ****/
-void hexa_print(const char *name, unsigned char *data, int size);
-const char *hexa_str(unsigned char *data, int size, int downside);
-
-#endif                          /* GRAS_PORTABLE_H */
+#endif                          /* SIMGRID_PORTABLE_H */
