@@ -1,18 +1,19 @@
-/* Copyright (c) 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2006-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "private.h"
+#include "instr/instr_interface.h"
 #include "xbt/sysdep.h"
 #include "xbt/dynar.h"
 #include "surf/surf.h"
+#include "simgrid/sg_config.h"
 #include "xbt/ex.h"
 #include "xbt/log.h"
 #include "xbt/str.h"
 #include "xbt/config.h"
-#include "instr/instr_private.h"
 #include "surf/surfxml_parse.h"
 #ifdef HAVE_LUA
 #include <lua.h>
@@ -31,7 +32,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(sd_kernel, sd,
 SD_global_t sd_global = NULL;
 
 /**
- * \brief Initialises SD internal data
+ * \brief Initializes SD internal data
  *
  * This function must be called before any other SD function. Then you
  * should call SD_create_environment().
@@ -80,7 +81,7 @@ void SD_init(int *argc, char **argv)
 
   surf_init(argc, argv);
 
-  xbt_cfg_setdefault_string(_surf_cfg_set, "workstation/model",
+  xbt_cfg_setdefault_string(_sg_cfg_set, "workstation/model",
                             "ptask_L07");
 
 #ifdef HAVE_TRACING
@@ -94,8 +95,27 @@ void SD_init(int *argc, char **argv)
   XBT_DEBUG("ADD SD LEVELS");
   SD_HOST_LEVEL = xbt_lib_add_level(host_lib,__SD_workstation_destroy);
   SD_LINK_LEVEL = xbt_lib_add_level(link_lib,__SD_link_destroy);
+  SD_STORAGE_LEVEL = xbt_lib_add_level(storage_lib,__SD_storage_destroy);
+
+  if (_sg_cfg_exit_asap) {
+    SD_exit();
+    exit(0);
+  }
 }
 
+/** \brief set a configuration variable
+ *
+ * Do --help on any simgrid binary to see the list of currently existing configuration variables, and see Section @ref options.
+ *
+ * Example:
+ * SD_config("workstation/model","default");
+ */
+void SD_config(const char *key, const char *value){
+  xbt_assert(sd_global,"ERROR: Please call SD_init() before using SD_config()");
+  xbt_cfg_set_as_string(_sg_cfg_set, key, value);
+}
+
+
 /**
  * \brief Reinits the application part of the simulation (experimental feature)
  *
@@ -113,20 +133,37 @@ void SD_application_reinit(void)
 
   s_SD_task_t task;
 
-  XBT_DEBUG("Recreating the swags...");
+  SD_task_t done_task, next_done_task;
+
+   XBT_DEBUG("Recreating the swags...");
   xbt_swag_free(sd_global->not_scheduled_task_set);
   xbt_swag_free(sd_global->schedulable_task_set);
   xbt_swag_free(sd_global->scheduled_task_set);
   xbt_swag_free(sd_global->runnable_task_set);
   xbt_swag_free(sd_global->in_fifo_task_set);
   xbt_swag_free(sd_global->running_task_set);
-  xbt_swag_free(sd_global->done_task_set);
   xbt_swag_free(sd_global->failed_task_set);
 
   sd_global->not_scheduled_task_set =
       xbt_swag_new(xbt_swag_offset(task, state_hookup));
   sd_global->schedulable_task_set =
       xbt_swag_new(xbt_swag_offset(task, state_hookup));
+  xbt_swag_foreach_safe(done_task, next_done_task, sd_global->done_task_set){
+    if (xbt_dynar_is_empty(done_task->tasks_before)){
+      __SD_task_set_state(done_task, SD_SCHEDULABLE);
+    } else{
+      __SD_task_set_state(done_task, SD_NOT_SCHEDULED);
+      done_task->unsatisfied_dependencies =
+        xbt_dynar_length(done_task->tasks_before);
+      done_task->is_not_ready = done_task->unsatisfied_dependencies;
+    }
+    free(done_task->workstation_list);
+    done_task->workstation_list = NULL;
+    done_task->workstation_nb = 0;
+  }
+
+  xbt_swag_free(sd_global->done_task_set);
+
   sd_global->scheduled_task_set =
       xbt_swag_new(xbt_swag_offset(task, state_hookup));
   sd_global->runnable_task_set =
@@ -141,7 +178,9 @@ void SD_application_reinit(void)
       xbt_swag_new(xbt_swag_offset(task, state_hookup));
   sd_global->task_number = 0;
 
+
 #ifdef HAVE_JEDULE
+  jedule_sd_dump();
   jedule_sd_cleanup();
   jedule_sd_init();
 #endif
@@ -150,9 +189,9 @@ void SD_application_reinit(void)
 /**
  * \brief Creates the environment
  *
- * The environment (i.e. the \ref SD_workstation_management "workstations" and the
- * \ref SD_link_management "links") is created with the data stored in the given XML
- * platform file.
+ * The environment (i.e. the \ref SD_workstation_management "workstations"
+ * and the \ref SD_link_management "links") is created with the data stored
+ * in the given XML platform file.
  *
  * \param platform_file name of an XML file describing the environment to create
  * \see SD_workstation_management, SD_link_management
@@ -171,10 +210,11 @@ void SD_create_environment(const char *platform_file)
   char *name = NULL;
   void **surf_workstation = NULL;
   void **surf_link = NULL;
+  void **surf_storage = NULL;
 
   parse_platform_file(platform_file);
 
-  /* now let's create the SD wrappers for workstations and links */
+  /* now let's create the SD wrappers for workstations, storages and links */
   xbt_lib_foreach(host_lib, cursor, name, surf_workstation){
     if(surf_workstation[SURF_WKS_LEVEL])
       __SD_workstation_create(surf_workstation[SURF_WKS_LEVEL], NULL);
@@ -185,6 +225,12 @@ void SD_create_environment(const char *platform_file)
     __SD_link_create(surf_link[SURF_LINK_LEVEL], NULL);
   }
 
+  xbt_lib_foreach(storage_lib, cursor, name, surf_storage) {
+  if(surf_storage[SURF_STORAGE_LEVEL])
+    __SD_storage_create(surf_storage[SURF_STORAGE_LEVEL], NULL);
+  }
+
+
   XBT_DEBUG("Workstation number: %d, link number: %d",
          SD_workstation_get_number(), SD_link_get_number());
 #ifdef HAVE_JEDULE
@@ -198,7 +244,7 @@ void SD_create_environment(const char *platform_file)
  * The function will execute the \ref SD_RUNNABLE runnable tasks.
  * If \a how_long is positive, then the simulation will be stopped either
  * when time reaches \a how_long or when a watch point is reached.
- * A nonpositive value for \a how_long means no time limit, in which case
+ * A non-positive value for \a how_long means no time limit, in which case
  * the simulation will be stopped either when a watch point is reached or
  * when no more task can be executed.
  * Then you can call SD_simulate() again.
@@ -289,6 +335,10 @@ xbt_swag_t SD_simulate_swag(double how_long) {
           if (dst->is_not_ready > 0)
             dst->is_not_ready--;
 
+          XBT_DEBUG("Released a dependency on %s: %d remain(s). Became schedulable if %d=0",
+             SD_task_get_name(dst), dst->unsatisfied_dependencies,
+             dst->is_not_ready);
+
           if (!(dst->unsatisfied_dependencies)) {
             if (__SD_task_is_scheduled(dst))
               __SD_task_set_state(dst, SD_RUNNABLE);
@@ -296,6 +346,10 @@ xbt_swag_t SD_simulate_swag(double how_long) {
               __SD_task_set_state(dst, SD_SCHEDULABLE);
           }
 
+          if (__SD_task_is_not_scheduled(dst) && !(dst->is_not_ready)) {
+            __SD_task_set_state(dst, SD_SCHEDULABLE);
+          }
+
           if (SD_task_get_kind(dst) == SD_TASK_COMM_E2E) {
             SD_dependency_t comm_dep;
             SD_task_t comm_dst;
@@ -305,6 +359,10 @@ xbt_swag_t SD_simulate_swag(double how_long) {
                 comm_dst->is_not_ready > 0) {
               comm_dst->is_not_ready--;
 
+            XBT_DEBUG("%s is a transfer, %s may be ready now if %d=0",
+               SD_task_get_name(dst), SD_task_get_name(comm_dst),
+               comm_dst->is_not_ready);
+
               if (!(comm_dst->is_not_ready)) {
                 __SD_task_set_state(comm_dst, SD_SCHEDULABLE);
               }
@@ -373,15 +431,15 @@ double SD_get_clock(void) {
 /**
  * \brief Destroys all SD internal data
  *
- * This function should be called when the simulation is over. Don't forget also to destroy
- * the tasks.
+ * This function should be called when the simulation is over. Don't forget
+ * to destroy too.
  *
  * \see SD_init(), SD_task_destroy()
  */
 void SD_exit(void)
 {
 #ifdef HAVE_TRACING
-  TRACE_surf_release();
+  TRACE_surf_resource_utilization_release();
 #endif
 
   xbt_mallocator_free(sd_global->task_mallocator);
@@ -406,35 +464,15 @@ void SD_exit(void)
   TRACE_end();
 #endif
 
-  XBT_DEBUG("Exiting Surf...");
-  surf_exit();
-
   xbt_free(sd_global);
   sd_global = NULL;
 
 #ifdef HAVE_JEDULE
   jedule_sd_dump();
   jedule_sd_cleanup();
+  jedule_sd_exit();
 #endif
-}
 
-/**
- * \brief load script file
- */
-
-void SD_load_environment_script(const char *script_file)
-{
-#ifdef HAVE_LUA
-  lua_State *L = lua_open();
-  luaL_openlibs(L);
-
-  if (luaL_loadfile(L, script_file) || lua_pcall(L, 0, 0, 0)) {
-    printf("error: %s\n", lua_tostring(L, -1));
-    return;
-  }
-#else
-  xbt_die
-      ("Lua is not available!! to call SD_load_environment_script, lua should be available...");
-#endif
-  return;
+  XBT_DEBUG("Exiting Surf...");
+  surf_exit();
 }
