@@ -1,6 +1,6 @@
 /* a fast and simple context switching library                              */
 
-/* Copyright (c) 2009 - 2011. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -11,7 +11,8 @@
 #include "xbt/swag.h"
 #include "xbt/xbt_os_thread.h"
 #include "smx_private.h"
-#include "gras_config.h"
+#include "simgrid/sg_config.h"
+#include "internal_config.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(simix_context, simix,
                                 "Context switching mechanism");
@@ -19,7 +20,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(simix_context, simix,
 char* smx_context_factory_name = NULL; /* factory name specified by --cfg=contexts/factory:value */
 smx_ctx_factory_initializer_t smx_factory_initializer_to_use = NULL;
 int smx_context_stack_size = 128 * 1024;
-
+int smx_context_stack_size_was_set = 0;
 #ifdef HAVE_THREAD_LOCAL_STORAGE
 static __thread smx_context_t smx_current_context_parallel;
 #else
@@ -35,6 +36,11 @@ static e_xbt_parmap_mode_t smx_parallel_synchronization_mode = XBT_PARMAP_DEFAUL
  */
 void SIMIX_context_mod_init(void)
 {
+#if defined(CONTEXT_THREADS) && !defined(HAVE_THREAD_LOCAL_STORAGE)
+  /* the __thread storage class is not available on this platform:
+   * use getspecific/setspecific instead to store the current context in each thread */
+  xbt_os_thread_key_create(&smx_current_context_key);
+#endif
   if (!simix_global->context_factory) {
     /* select the context factory to use to create the contexts */
     if (smx_factory_initializer_to_use) {
@@ -42,41 +48,32 @@ void SIMIX_context_mod_init(void)
     }
     else { /* use the factory specified by --cfg=contexts/factory:value */
 
-    if (smx_context_factory_name == NULL) {
-        /* use the default factory */
-  #ifdef HAVE_RAWCTX
-      SIMIX_ctx_raw_factory_init(&simix_global->context_factory);
-  #elif CONTEXT_UCONTEXT
-    SIMIX_ctx_sysv_factory_init(&simix_global->context_factory);
-  #else
-    SIMIX_ctx_thread_factory_init(&simix_global->context_factory);
-  #endif
-    }
-    else if (!strcmp(smx_context_factory_name, "ucontext")) {
-        /* use ucontext */
-#ifdef CONTEXT_UCONTEXT
-        SIMIX_ctx_sysv_factory_init(&simix_global->context_factory);
-#else
-        xbt_die("The context factory 'ucontext' unavailable on your system");
-#endif
-      }
-      else if (!strcmp(smx_context_factory_name, "thread")) {
-  /* use os threads (either pthreads or windows ones) */
+
+      if (!strcmp(smx_context_factory_name, "thread")) {
+        /* use os threads (either pthreads or windows ones) */
         SIMIX_ctx_thread_factory_init(&simix_global->context_factory);
       }
-      else if (!strcmp(smx_context_factory_name, "raw")) {
-  /* use raw contexts */
-  SIMIX_ctx_raw_factory_init(&simix_global->context_factory);
+#ifdef CONTEXT_UCONTEXT
+      else if (!strcmp(smx_context_factory_name, "ucontext")) {
+        /* use ucontext */
+        SIMIX_ctx_sysv_factory_init(&simix_global->context_factory);
       }
+#endif
+#ifdef HAVE_RAWCTX
+      else if (!strcmp(smx_context_factory_name, "raw")) {
+        /* use raw contexts */
+        SIMIX_ctx_raw_factory_init(&simix_global->context_factory);
+      }
+#endif
       else {
         XBT_ERROR("Invalid context factory specified. Valid factories on this machine:");
 #ifdef HAVE_RAWCTX
         XBT_ERROR("  raw: high performance context factory implemented specifically for SimGrid");
 #else
-        XBT_ERROR("  (raw contextes are disabled at compilation time on this machine -- check configure logs for details)");
+        XBT_ERROR("  (raw contexts are disabled at compilation time on this machine -- check configure logs for details)");
 #endif
 #ifdef CONTEXT_UCONTEXT
-        XBT_ERROR("  ucontext: classical system V contextes (implemented with makecontext, swapcontext and friends)");
+        XBT_ERROR("  ucontext: classical system V contexts (implemented with makecontext, swapcontext and friends)");
 #else
         XBT_ERROR("  (ucontext is disabled at compilation time on this machine -- check configure logs for details)");
 #endif
@@ -85,16 +82,10 @@ void SIMIX_context_mod_init(void)
       }
     }
   }
-
-#if defined(CONTEXT_THREADS) && !defined(HAVE_THREAD_LOCAL_STORAGE)
-  /* the __thread storage class is not available on this platform:
-   * use getspecific/setspecific instead to store the current context in each thread */
-  xbt_os_thread_key_create(&smx_current_context_key);
-#endif
 }
 
 /**
- * This function is call by SIMIX_clean() to finalize the context module.
+ * This function is called by SIMIX_clean() to finalize the context module.
  */
 void SIMIX_context_mod_exit(void)
 {
@@ -105,7 +96,7 @@ void SIMIX_context_mod_exit(void)
     finalize_factory = simix_global->context_factory->finalize;
     finalize_factory(&simix_global->context_factory);
   }
-  xbt_dict_remove((xbt_dict_t) _surf_cfg_set,"contexts/factory");
+  xbt_dict_remove((xbt_dict_t) _sg_cfg_set,"contexts/factory");
 }
 
 /**
@@ -136,8 +127,7 @@ XBT_INLINE int SIMIX_context_get_nthreads(void) {
  *
  * \param nb_threads the number of threads to use
  */
-XBT_INLINE void SIMIX_context_set_nthreads(int nb_threads) {
-
+void SIMIX_context_set_nthreads(int nb_threads) {
   if (nb_threads<=0) {  
      nb_threads = xbt_os_get_numcores();
      XBT_INFO("Auto-setting contexts/nthreads to %d",nb_threads);
@@ -148,7 +138,6 @@ XBT_INLINE void SIMIX_context_set_nthreads(int nb_threads) {
     THROWF(arg_error, 0, "The thread factory cannot be run in parallel");
 #endif
   }
- 
   smx_parallel_contexts = nb_threads;
 }
 
@@ -231,4 +220,3 @@ XBT_INLINE void SIMIX_context_set_current(smx_context_t context)
     smx_current_context_serial = context;
   }
 }
-
