@@ -1,6 +1,6 @@
 /* context_base - Code factorization across context switching implementations */
 
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -12,7 +12,7 @@
 #include "smx_private.h"
 #include "mc/mc.h"
 
-XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(bindings);
+XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(simix_context);
 
 void smx_ctx_base_factory_init(smx_context_factory_t *factory)
 {
@@ -49,7 +49,7 @@ smx_ctx_base_factory_create_context_sized(size_t size,
 
   /* Store the address of the stack in heap to compare it apart of heap comparison */
   if(MC_is_active())
-    MC_ignore(context, size);
+    MC_ignore_heap(context, size);
 
   /* If the user provided a function for the process then use it.
      Otherwise, it is the context for maestro and we should set it as the
