@@ -1,6 +1,6 @@
-/* context_raw - fast context switching inspired from System V ucontextes   */
+/* context_raw - fast context switching inspired from System V ucontexts   */
 
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -31,7 +31,7 @@ typedef struct s_smx_ctx_raw {
 
 #ifdef CONTEXT_THREADS
 static xbt_parmap_t raw_parmap;
-static raw_stack_t* raw_workers_stacks;       /* space to save the worker stack in each thread */
+static smx_ctx_raw_t* raw_workers_context;    /* space to save the worker context in each thread */
 static unsigned long raw_threads_working;     /* number of threads that have started their work */
 static xbt_os_thread_key_t raw_worker_id_key; /* thread-specific storage for the thread id */
 #endif
@@ -44,7 +44,86 @@ extern raw_stack_t raw_makecontext(char* malloced_stack, int stack_size,
                                    rawctx_entry_point_t entry_point, void* arg);
 extern void raw_swapcontext(raw_stack_t* old, raw_stack_t new);
 
-#ifdef PROCESSOR_i686
+#if PROCESSOR_x86_64
+__asm__ (
+#if defined(APPLE)
+   ".text\n"
+   ".globl _raw_makecontext\n"
+   "_raw_makecontext:\n"
+#elif defined(_WIN32)
+   ".text\n"
+   ".globl raw_makecontext\n"
+   "raw_makecontext:\n"
+#else
+   ".text\n"
+   ".globl raw_makecontext\n"
+   ".type raw_makecontext,@function\n"
+   "raw_makecontext:\n"/* Calling convention sets the arguments in rdi, rsi, rdx and rcx, respectively */
+#endif
+   "   mov %rdi,%rax\n"      /* stack */
+   "   add %rsi,%rax\n"      /* size  */
+   "   movq $0,   -8(%rax)\n" /* @return for func */
+   "   mov %rdx,-16(%rax)\n" /* func */
+   "   mov %rcx,-24(%rax)\n" /* arg/rdi */
+   "   movq $0,  -32(%rax)\n" /* rsi */
+   "   movq $0,  -40(%rax)\n" /* rdx */
+   "   movq $0,  -48(%rax)\n" /* rcx */
+   "   movq $0,  -56(%rax)\n" /* r8  */
+   "   movq $0,  -64(%rax)\n" /* r9  */
+   "   movq $0,  -72(%rax)\n" /* rbp */
+   "   movq $0,  -80(%rax)\n" /* rbx */
+   "   movq $0,  -88(%rax)\n" /* r12 */
+   "   movq $0,  -96(%rax)\n" /* r13 */
+   "   movq $0, -104(%rax)\n" /* r14 */
+   "   movq $0, -112(%rax)\n" /* r15 */
+   "   sub $112,%rax\n"
+   "   ret\n"
+);
+
+__asm__ (
+#if defined(APPLE)
+   ".text\n"
+   ".globl _raw_swapcontext\n"
+   "_raw_swapcontext:\n"
+#elif defined(_WIN32)
+   ".text\n"
+   ".globl raw_swapcontext\n"
+   "raw_swapcontext:\n"
+#else
+   ".text\n"
+   ".globl raw_swapcontext\n"
+   ".type raw_swapcontext,@function\n"
+   "raw_swapcontext:\n" /* Calling convention sets the arguments in rdi and rsi, respectively */
+#endif
+   "   push %rdi\n"
+   "   push %rsi\n"
+   "   push %rdx\n"
+   "   push %rcx\n"
+   "   push %r8\n"
+   "   push %r9\n"
+   "   push %rbp\n"
+   "   push %rbx\n"
+   "   push %r12\n"
+   "   push %r13\n"
+   "   push %r14\n"
+   "   push %r15\n"
+   "   mov %rsp,(%rdi)\n" /* old */
+   "   mov %rsi,%rsp\n" /* new */
+   "   pop %r15\n"
+   "   pop %r14\n"
+   "   pop %r13\n"
+   "   pop %r12\n"
+   "   pop %rbx\n"
+   "   pop %rbp\n"
+   "   pop %r9\n"
+   "   pop %r8\n"
+   "   pop %rcx\n"
+   "   pop %rdx\n"
+   "   pop %rsi\n"
+   "   pop %rdi\n"
+   "   ret\n"
+);
+#elif PROCESSOR_i686
 __asm__ (
 #if defined(APPLE) || defined(_WIN32)
    ".text\n"
@@ -96,86 +175,8 @@ __asm__ (
    "   popl %ebp\n"
    "   retl\n"
 );
-#elif PROCESSOR_x86_64
-__asm__ (
-#if defined(APPLE)
-   ".text\n"
-   ".globl _raw_makecontext\n"
-   "_raw_makecontext:\n"
-#elif defined(_WIN32)
-   ".text\n"
-   ".globl raw_makecontext\n"
-   "raw_makecontext:\n"
 #else
-   ".text\n"
-   ".globl raw_makecontext\n"
-   ".type raw_makecontext,@function\n"
-   "raw_makecontext:\n"/* Calling convention sets the arguments in rdi, rsi, rdx and rcx, respectively */
-#endif
-   "   movq %rdi,%rax\n"      /* stack */
-   "   addq %rsi,%rax\n"      /* size  */
-   "   movq $0,   -8(%rax)\n" /* @return for func */
-   "   movq %rdx,-16(%rax)\n" /* func */
-   "   movq %rcx,-24(%rax)\n" /* arg/rdi */
-   "   movq $0,  -32(%rax)\n" /* rsi */
-   "   movq $0,  -40(%rax)\n" /* rdx */
-   "   movq $0,  -48(%rax)\n" /* rcx */
-   "   movq $0,  -56(%rax)\n" /* r8  */
-   "   movq $0,  -64(%rax)\n" /* r9  */
-   "   movq $0,  -72(%rax)\n" /* rbp */
-   "   movq $0,  -80(%rax)\n" /* rbx */
-   "   movq $0,  -88(%rax)\n" /* r12 */
-   "   movq $0,  -96(%rax)\n" /* r13 */
-   "   movq $0, -104(%rax)\n" /* r14 */
-   "   movq $0, -112(%rax)\n" /* r15 */
-   "   subq $112,%rax\n"
-   "   retq\n"
-);
 
-__asm__ (
-#if defined(APPLE)
-   ".text\n"
-   ".globl _raw_swapcontext\n"
-   "_raw_swapcontext:\n"
-#elif defined(_WIN32)
-   ".text\n"
-   ".globl raw_swapcontext\n"
-   "raw_swapcontext:\n"
-#else
-   ".text\n"
-   ".globl raw_swapcontext\n"
-   ".type raw_swapcontext,@function\n"
-   "raw_swapcontext:\n" /* Calling convention sets the arguments in rdi and rsi, respectively */
-#endif
-   "   pushq %rdi\n"
-   "   pushq %rsi\n"
-   "   pushq %rdx\n"
-   "   pushq %rcx\n"
-   "   pushq %r8\n"
-   "   pushq %r9\n"
-   "   pushq %rbp\n"
-   "   pushq %rbx\n"
-   "   pushq %r12\n"
-   "   pushq %r13\n"
-   "   pushq %r14\n"
-   "   pushq %r15\n"
-   "   movq %rsp,(%rdi)\n" /* old */
-   "   movq %rsi,%rsp\n" /* new */
-   "   popq %r15\n"
-   "   popq %r14\n"
-   "   popq %r13\n"
-   "   popq %r12\n"
-   "   popq %rbx\n"
-   "   popq %rbp\n"
-   "   popq %r9\n"
-   "   popq %r8\n"
-   "   popq %rcx\n"
-   "   popq %rdx\n"
-   "   popq %rsi\n"
-   "   popq %rdi\n"
-   "   retq\n"
-);
-#else
 
 /* If you implement raw contexts for other processors, don't forget to
    update the definition of HAVE_RAWCTX in buildtools/Cmake/CompleteInFiles.cmake */
@@ -241,9 +242,11 @@ void SIMIX_ctx_raw_factory_init(smx_context_factory_t *factory)
   if (SIMIX_context_is_parallel()) {
 #ifdef CONTEXT_THREADS
     int nthreads = SIMIX_context_get_nthreads();
-    raw_parmap = xbt_parmap_new(nthreads, SIMIX_context_get_parallel_mode());
-    raw_workers_stacks = xbt_new(raw_stack_t, nthreads);
     xbt_os_thread_key_create(&raw_worker_id_key);
+    raw_parmap = xbt_parmap_new(nthreads, SIMIX_context_get_parallel_mode());
+    raw_workers_context = xbt_new(smx_ctx_raw_t, nthreads);
+    raw_maestro_context=NULL;
+
 #endif
     if (SIMIX_context_get_parallel_threshold() > 1) {
       /* choose dynamically */
@@ -273,14 +276,14 @@ void SIMIX_ctx_raw_factory_init(smx_context_factory_t *factory)
 static int smx_ctx_raw_factory_finalize(smx_context_factory_t *factory)
 {
 #ifdef TIME_BENCH_PER_SR
-  XBT_CRITICAL("Total wasted time in %u SR: %lf", sr_count, time_wasted_sr);
-  XBT_CRITICAL("Total wasted time in %u SSR: %lf", ssr_count, time_wasted_ssr);
+  XBT_CRITICAL("Total wasted time in %u SR: %f", sr_count, time_wasted_sr);
+  XBT_CRITICAL("Total wasted time in %u SSR: %f", ssr_count, time_wasted_ssr);
 #endif
 
 #ifdef CONTEXT_THREADS
   if (raw_parmap)
     xbt_parmap_destroy(raw_parmap);
-  xbt_free(raw_workers_stacks);
+  xbt_free(raw_workers_context);
 #endif
   return smx_ctx_base_factory_finalize(factory);
 }
@@ -325,10 +328,11 @@ smx_ctx_raw_create_context(xbt_main_func_t code, int argc, char **argv,
 #endif                          /* HAVE_VALGRIND_VALGRIND_H */
 
      } else {
-       raw_maestro_context = context;
+       if(data != NULL && raw_maestro_context==NULL)
+         raw_maestro_context = context;
 
        if(MC_is_active())
-         MC_ignore(&(raw_maestro_context->stack_top), sizeof(raw_maestro_context->stack_top));
+         MC_ignore_heap(&(raw_maestro_context->stack_top), sizeof(raw_maestro_context->stack_top));
 
      }
 
@@ -444,9 +448,9 @@ static void smx_ctx_raw_runall_serial(xbt_dynar_t processes)
       time_thread_sr[t] = 0;
     }
 
-    xbt_os_timer_start(timer);
+    xbt_os_cputimer_start(timer);
     smx_ctx_raw_resume(process);
-    xbt_os_timer_stop(timer);
+    xbt_os_cputimer_stop(timer);
     elapsed = xbt_os_timer_elapsed(timer);
     time_thread_ssr[t] += elapsed;
     time_thread_sr[((smx_ctx_raw_t)process->context)->thread] += elapsed;
@@ -459,7 +463,7 @@ static void smx_ctx_raw_runall_serial(xbt_dynar_t processes)
     tmax = time_thread_ssr[t];
 
   for(cursor=0; cursor <= t; cursor++){
-    XBT_VERB("Time SSR thread %u = %lf (max %lf)", cursor, time_thread_ssr[cursor], tmax);
+    XBT_VERB("Time SSR thread %u = %f (max %f)", cursor, time_thread_ssr[cursor], tmax);
     time_wasted_ssr += tmax - time_thread_ssr[cursor];
   }
 }
@@ -477,7 +481,7 @@ void smx_ctx_raw_new_sr(void)
   }
 
   for(i=0; i < NUM_THREADS; i++){
-    XBT_VERB("Time SR thread %u = %lf (max %lf)", i, time_thread_sr[i], tmax);
+    XBT_VERB("Time SR thread %u = %f (max %f)", i, time_thread_sr[i], tmax);
     time_wasted_sr += tmax - time_thread_sr[i];
   }
 
@@ -521,12 +525,14 @@ static void smx_ctx_raw_suspend_parallel(smx_context_t context)
   else {
     /* all processes were run, go to the barrier */
     XBT_DEBUG("No more processes to run");
-    next_context = (smx_context_t) raw_maestro_context;
+
     unsigned long worker_id =
         (unsigned long) xbt_os_thread_get_specific(raw_worker_id_key);
+
+    next_context = (smx_context_t)raw_workers_context[worker_id];
     XBT_DEBUG("Restoring worker stack %lu (working threads = %lu)",
         worker_id, raw_threads_working);
-    next_stack = raw_workers_stacks[worker_id];
+    next_stack = ((smx_ctx_raw_t)next_context)->stack_top;
   }
 
   SIMIX_context_set_current(next_context);
@@ -544,8 +550,11 @@ static void smx_ctx_raw_resume_parallel(smx_process_t first_process)
 #ifdef CONTEXT_THREADS
   unsigned long worker_id = __sync_fetch_and_add(&raw_threads_working, 1);
   xbt_os_thread_set_specific(raw_worker_id_key, (void*) worker_id);
+  smx_ctx_raw_t worker_context = (smx_ctx_raw_t)SIMIX_context_self();
+  raw_workers_context[worker_id] = worker_context;
   XBT_DEBUG("Saving worker stack %lu", worker_id);
-  raw_stack_t* worker_stack = &raw_workers_stacks[worker_id];
+  raw_stack_t* worker_stack = &(worker_context)->stack_top;
+
 
   smx_context_t context = first_process->context;
   SIMIX_context_set_current(context);
@@ -563,7 +572,7 @@ static void smx_ctx_raw_runall_parallel(void)
   xbt_parmap_apply(raw_parmap, (void_f_pvoid_t) smx_ctx_raw_resume_parallel,
       simix_global->process_to_run);
 #else
-  xbt_die("You asked for a parallel execution, but you don't have any threads.")
+  xbt_die("You asked for a parallel execution, but you don't have any threads.");
 #endif
 }
 
