@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2012. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -38,6 +38,9 @@ int SIMIX_process_has_pending_comms(smx_process_t process) {
   return xbt_fifo_size(process->comms) > 0;
 }
 
+void SIMIX_pre_process_cleanup(smx_simcall_t simcall, smx_process_t process) {
+  SIMIX_process_cleanup(process);
+}
 /**
  * \brief Moves a process to the list of processes to destroy.
  */
@@ -92,9 +95,8 @@ void SIMIX_process_cleanup(smx_process_t process)
     }
   }
 
-  /*xbt_swag_remove(process, simix_global->process_to_run);*/
   xbt_swag_remove(process, simix_global->process_list);
-  xbt_swag_remove(process, process->smx_host->process_list);
+  xbt_swag_remove(process, SIMIX_host_priv(process->smx_host)->process_list);
   xbt_swag_insert(process, simix_global->process_to_destroy);
   process->context->iwannadie = 0;
 }
@@ -120,8 +122,8 @@ void SIMIX_process_empty_trash(void)
 
     xbt_dynar_free(&process->on_exit);
 
-    free(process->name);
-    free(process);
+    xbt_free(process->name);
+    xbt_free(process);
   }
 }
 
@@ -141,6 +143,12 @@ void SIMIX_create_maestro_process()
   maestro->context = SIMIX_context_new(NULL, 0, NULL, NULL, maestro);
   maestro->simcall.issuer = maestro;
 
+  if (SIMIX_process_self()) {
+    maestro->ppid = SIMIX_process_get_PID(SIMIX_process_self());
+  } else {
+    maestro->ppid = -1;
+  }
+
   simix_global->maestro_process = maestro;
   return;
 }
@@ -159,12 +167,12 @@ void SIMIX_process_stop(smx_process_t arg) {
    */
   if (arg->auto_restart && !SIMIX_host_get_state(arg->smx_host)) {
     SIMIX_host_add_auto_restart_process(arg->smx_host,arg->name,arg->code, arg->data,
-                                        arg->smx_host->name,
+                                        sg_host_name(arg->smx_host),
                                         arg->kill_time,
                                         arg->argc,arg->argv,arg->properties,
                                         arg->auto_restart);
   }
-  XBT_DEBUG("Process %s (%s) is dead",arg->name,arg->smx_host->name);
+  XBT_DEBUG("Process %s (%s) is dead",arg->name,sg_host_name(arg->smx_host));
   /* stop the context */
   SIMIX_context_stop(arg->context);
 }
@@ -192,6 +200,20 @@ smx_process_t SIMIX_process_create_from_wrapper(smx_process_arg_t args) {
   return process;
 }
 
+
+void SIMIX_pre_process_create(smx_simcall_t simcall,
+		          smx_process_t *process,
+                          const char *name,
+                          xbt_main_func_t code,
+                          void *data,
+                          const char *hostname,
+                          double kill_time,
+                          int argc, char **argv,
+                          xbt_dict_t properties,
+                          int auto_restart){
+  return SIMIX_process_create(process, name, code, data, hostname,
+                              kill_time, argc, argv, properties, auto_restart);
+}
 /**
  * \brief Internal function to create a process.
  *
@@ -214,11 +236,15 @@ void SIMIX_process_create(smx_process_t *process,
   *process = NULL;
   smx_host_t host = SIMIX_host_get_by_name(hostname);
 
-  XBT_DEBUG("Start process %s on host %s", name, hostname);
+  XBT_DEBUG("Start process %s on host '%s'", name, hostname);
 
   if (!SIMIX_host_get_state(host)) {
+    int i;
     XBT_WARN("Cannot launch process '%s' on failed host '%s'", name,
           hostname);
+    for (i = 0; i < argc; i++)
+      xbt_free(argv[i]);
+    xbt_free(argv);
   }
   else {
     *process = xbt_new0(s_smx_process_t, 1);
@@ -231,6 +257,13 @@ void SIMIX_process_create(smx_process_t *process,
     (*process)->data = data;
     (*process)->comms = xbt_fifo_new();
     (*process)->simcall.issuer = *process;
+    
+     if (SIMIX_process_self()) {
+       (*process)->ppid = SIMIX_process_get_PID(SIMIX_process_self());
+     } else {
+       (*process)->ppid = -1;
+     }
+
     /* Process data for auto-restart */
     (*process)->auto_restart = auto_restart;
     (*process)->code = code;
@@ -246,24 +279,28 @@ void SIMIX_process_create(smx_process_t *process,
     (*process)->running_ctx = xbt_new(xbt_running_ctx_t, 1);
     XBT_RUNNING_CTX_INITIALIZE((*process)->running_ctx);
 
+    if(MC_is_active()){
+      MC_ignore_heap((*process)->running_ctx, sizeof(*(*process)->running_ctx));
+    }
+
     /* Add properties */
     (*process)->properties = properties;
 
     /* Add the process to it's host process list */
-    xbt_swag_insert(*process, host->process_list);
+    xbt_swag_insert(*process, SIMIX_host_priv(host)->process_list);
 
     XBT_DEBUG("Start context '%s'", (*process)->name);
 
     /* Now insert it in the global process list and in the process to run list */
     xbt_swag_insert(*process, simix_global->process_list);
-    XBT_DEBUG("Inserting %s(%s) in the to_run list", (*process)->name, host->name);
+    XBT_DEBUG("Inserting %s(%s) in the to_run list", (*process)->name, sg_host_name(host));
     xbt_dynar_push_as(simix_global->process_to_run, smx_process_t, *process);
   }
 
   if (kill_time > SIMIX_get_clock()) {
     if (simix_global->kill_process_function) {
       XBT_DEBUG("Process %s(%s) will be kill at time %f", (*process)->name,
-          (*process)->smx_host->name, kill_time);
+          sg_host_name((*process)->smx_host), kill_time);
       SIMIX_timer_set(kill_time, simix_global->kill_process_function, *process);
     }
   }
@@ -288,6 +325,9 @@ void SIMIX_process_runall(void)
   xbt_dynar_reset(simix_global->process_to_run);
 }
 
+void SIMIX_pre_process_kill(smx_simcall_t simcall, smx_process_t process) {
+  SIMIX_process_kill(process, simcall->issuer);
+}
 /**
  * \brief Internal function to kill a SIMIX process.
  *
@@ -299,7 +339,7 @@ void SIMIX_process_runall(void)
  */
 void SIMIX_process_kill(smx_process_t process, smx_process_t issuer) {
 
-  XBT_DEBUG("Killing process %s on %s", process->name, process->smx_host->name);
+  XBT_DEBUG("Killing process %s on %s", process->name, sg_host_name(process->smx_host));
 
   process->context->iwannadie = 1;
   process->blocked = 0;
@@ -311,35 +351,36 @@ void SIMIX_process_kill(smx_process_t process, smx_process_t issuer) {
 
     switch (process->waiting_action->type) {
 
-      case SIMIX_ACTION_EXECUTE:          
-      case SIMIX_ACTION_PARALLEL_EXECUTE:
-        SIMIX_host_execution_destroy(process->waiting_action);
-        break;
+    case SIMIX_ACTION_EXECUTE:
+    case SIMIX_ACTION_PARALLEL_EXECUTE:
+      SIMIX_host_execution_destroy(process->waiting_action);
+      break;
 
-      case SIMIX_ACTION_COMMUNICATE:
-        xbt_fifo_remove(process->comms, process->waiting_action);
-        SIMIX_comm_cancel(process->waiting_action);
-        break;
+    case SIMIX_ACTION_COMMUNICATE:
+      xbt_fifo_remove(process->comms, process->waiting_action);
+      SIMIX_comm_cancel(process->waiting_action);
+      SIMIX_comm_destroy(process->waiting_action);
+      break;
 
-        case SIMIX_ACTION_SLEEP:
-          SIMIX_process_sleep_destroy(process->waiting_action);
-          break;
+    case SIMIX_ACTION_SLEEP:
+      SIMIX_process_sleep_destroy(process->waiting_action);
+      break;
 
-        case SIMIX_ACTION_SYNCHRO:
-          SIMIX_synchro_stop_waiting(process, &process->simcall);
-          SIMIX_synchro_destroy(process->waiting_action);
-          break;
+    case SIMIX_ACTION_SYNCHRO:
+      SIMIX_synchro_stop_waiting(process, &process->simcall);
+      SIMIX_synchro_destroy(process->waiting_action);
+      break;
 
-        case SIMIX_ACTION_IO:
-          SIMIX_io_destroy(process->waiting_action);
-          break;
+    case SIMIX_ACTION_IO:
+      SIMIX_io_destroy(process->waiting_action);
+      break;
 
-        /* **************************************/
-        /* TUTORIAL: New API                    */
-        case SIMIX_ACTION_NEW_API:
-          SIMIX_new_api_destroy(process->waiting_action);
-          break;
-        /* **************************************/
+      /* **************************************/
+      /* TUTORIAL: New API                    */
+    case SIMIX_ACTION_NEW_API:
+      SIMIX_new_api_destroy(process->waiting_action);
+      break;
+      /* **************************************/
 
     }
   }
@@ -349,11 +390,14 @@ void SIMIX_process_kill(smx_process_t process, smx_process_t issuer) {
 
 }
 
+void SIMIX_pre_process_killall(smx_simcall_t simcall, int reset_pid) {
+  SIMIX_process_killall(simcall->issuer, reset_pid);
+}
 /**
  * \brief Kills all running processes.
  * \param issuer this one will not be killed
  */
-void SIMIX_process_killall(smx_process_t issuer)
+void SIMIX_process_killall(smx_process_t issuer, int reset_pid)
 {
   smx_process_t p = NULL;
 
@@ -363,28 +407,31 @@ void SIMIX_process_killall(smx_process_t issuer)
     }
   }
 
+  if (reset_pid > 0)
+    simix_process_maxpid = reset_pid;
+
   SIMIX_context_runall();
 
   SIMIX_process_empty_trash();
 }
 
+void SIMIX_pre_process_change_host(smx_simcall_t simcall, smx_process_t process,
+		                   smx_host_t dest)
+{
+  process->new_host = dest;
+}
 void SIMIX_process_change_host(smx_process_t process,
              smx_host_t dest)
 {
   xbt_assert((process != NULL), "Invalid parameters");
-  xbt_swag_remove(process, process->smx_host->process_list);
+  xbt_swag_remove(process, SIMIX_host_priv(process->smx_host)->process_list);
   process->smx_host = dest;
-  xbt_swag_insert(process, dest->process_list);
+  xbt_swag_insert(process, SIMIX_host_priv(dest)->process_list);
 }
 
-void SIMIX_pre_process_change_host(smx_process_t process, smx_host_t dest)
-{
-  process->new_host = dest;
-}
 
-void SIMIX_pre_process_suspend(smx_simcall_t simcall)
+void SIMIX_pre_process_suspend(smx_simcall_t simcall, smx_process_t process)
 {
-  smx_process_t process = simcall->process_suspend.process;
   smx_action_t action_suspend =
       SIMIX_process_suspend(process, simcall->issuer);
 
@@ -448,6 +495,10 @@ smx_action_t SIMIX_process_suspend(smx_process_t process, smx_process_t issuer)
   }
 }
 
+void SIMIX_pre_process_resume(smx_simcall_t simcall, smx_process_t process){
+  SIMIX_process_resume(process, simcall->issuer);
+}
+
 void SIMIX_process_resume(smx_process_t process, smx_process_t issuer)
 {
   xbt_assert((process != NULL), "Invalid parameters");
@@ -502,11 +553,40 @@ int SIMIX_process_get_maxpid(void) {
   return simix_process_maxpid;
 }
 
+int SIMIX_pre_process_count(smx_simcall_t simcall){
+  return SIMIX_process_count();
+}
 int SIMIX_process_count(void)
 {
   return xbt_swag_size(simix_global->process_list);
 }
 
+int SIMIX_pre_process_get_PID(smx_simcall_t simcall, smx_process_t self){
+   return SIMIX_process_get_PID(self);
+}
+
+int SIMIX_process_get_PID(smx_process_t self){
+  if (self == NULL)
+    return 0;
+  else
+    return self->pid;
+}
+
+int SIMIX_pre_process_get_PPID(smx_simcall_t simcall, smx_process_t self){
+  return SIMIX_process_get_PPID(self);
+}
+
+int SIMIX_process_get_PPID(smx_process_t self){
+  if (self == NULL)
+    return 0;
+  else
+    return self->ppid;
+}
+
+void* SIMIX_pre_process_self_get_data(smx_simcall_t simcall, smx_process_t self){
+  return SIMIX_process_self_get_data(self);	
+}
+
 void* SIMIX_process_self_get_data(smx_process_t self)
 {
   xbt_assert(self == SIMIX_process_self(), "This is not the current process");
@@ -517,6 +597,10 @@ void* SIMIX_process_self_get_data(smx_process_t self)
   return SIMIX_process_get_data(self);
 }
 
+void SIMIX_pre_process_set_data(smx_simcall_t simcall, smx_process_t process,
+                                void *data){
+  SIMIX_process_set_data(process, data);
+}
 void SIMIX_process_self_set_data(smx_process_t self, void *data)
 {
   xbt_assert(self == SIMIX_process_self(), "This is not the current process");
@@ -524,6 +608,9 @@ void SIMIX_process_self_set_data(smx_process_t self, void *data)
   SIMIX_process_set_data(self, data);
 }
 
+void* SIMIX_pre_process_get_data(smx_simcall_t simcall, smx_process_t process){
+  return SIMIX_process_get_data(process);
+}
 void* SIMIX_process_get_data(smx_process_t process)
 {
   return process->data;
@@ -534,6 +621,9 @@ void SIMIX_process_set_data(smx_process_t process, void *data)
   process->data = data;
 }
 
+smx_host_t SIMIX_pre_process_get_host(smx_simcall_t simcall, smx_process_t process){
+  return SIMIX_process_get_host(process);
+}
 smx_host_t SIMIX_process_get_host(smx_process_t process)
 {
   return process->smx_host;
@@ -550,6 +640,9 @@ const char* SIMIX_process_self_get_name(void) {
   return SIMIX_process_get_name(process);
 }
 
+const char* SIMIX_pre_process_get_name(smx_simcall_t simcall, smx_process_t process) {
+  return SIMIX_process_get_name(process);
+}
 const char* SIMIX_process_get_name(smx_process_t process)
 {
   return process->name;
@@ -567,25 +660,31 @@ smx_process_t SIMIX_process_get_by_name(const char* name)
   return NULL;
 }
 
+int SIMIX_pre_process_is_suspended(smx_simcall_t simcall, smx_process_t process){
+  return SIMIX_process_is_suspended(process);
+}
 int SIMIX_process_is_suspended(smx_process_t process)
 {
   return process->suspended;
 }
 
+xbt_dict_t SIMIX_pre_process_get_properties(smx_simcall_t simcall, smx_process_t process){
+  return SIMIX_process_get_properties(process);
+}
 xbt_dict_t SIMIX_process_get_properties(smx_process_t process)
 {
   return process->properties;
 }
 
-void SIMIX_pre_process_sleep(smx_simcall_t simcall)
+void SIMIX_pre_process_sleep(smx_simcall_t simcall, double duration)
 {
   if (MC_is_active()) {
-    MC_process_clock_add(simcall->issuer, simcall->process_sleep.duration);
-    simcall->process_sleep.result = SIMIX_DONE;
+    MC_process_clock_add(simcall->issuer, duration);
+    simcall_process_sleep__set__result(simcall, SIMIX_DONE);
     SIMIX_simcall_answer(simcall);
     return;
   }
-  smx_action_t action = SIMIX_process_sleep(simcall->issuer, simcall->process_sleep.duration);
+  smx_action_t action = SIMIX_process_sleep(simcall->issuer, duration);
   xbt_fifo_push(action->simcalls, simcall);
   simcall->issuer->waiting_action = action;
 }
@@ -597,9 +696,9 @@ smx_action_t SIMIX_process_sleep(smx_process_t process, double duration)
 
   /* check if the host is active */
   if (surf_workstation_model->extension.
-      workstation.get_state(host->host) != SURF_RESOURCE_ON) {
+      workstation.get_state(host) != SURF_RESOURCE_ON) {
     THROWF(host_error, 0, "Host %s failed, you cannot call this function",
-           host->name);
+           sg_host_name(host));
   }
 
   action = xbt_mallocator_get(simix_global->action_mallocator);
@@ -611,7 +710,7 @@ smx_action_t SIMIX_process_sleep(smx_process_t process, double duration)
 
   action->sleep.host = host;
   action->sleep.surf_sleep =
-      surf_workstation_model->extension.workstation.sleep(host->host, duration);
+      surf_workstation_model->extension.workstation.sleep(host, duration);
 
   surf_workstation_model->action_data_set(action->sleep.surf_sleep, action);
   XBT_DEBUG("Create sleep action %p", action);
@@ -630,6 +729,7 @@ void SIMIX_post_process_sleep(smx_action_t action)
       case SURF_ACTION_FAILED:
         simcall->issuer->context->iwannadie = 1;
         //SMX_EXCEPTION(simcall->issuer, host_error, 0, "Host failed");
+        state = SIMIX_SRC_HOST_FAILURE;
         break;
 
       case SURF_ACTION_DONE:
@@ -641,10 +741,10 @@ void SIMIX_post_process_sleep(smx_action_t action)
         break;
     }
     if (surf_workstation_model->extension.
-        workstation.get_state(simcall->issuer->smx_host->host) != SURF_RESOURCE_ON) {
+        workstation.get_state(simcall->issuer->smx_host) != SURF_RESOURCE_ON) {
       simcall->issuer->context->iwannadie = 1;
     }
-    simcall->process_sleep.result = state;
+    simcall_process_sleep__set__result(simcall, state);
     simcall->issuer->waiting_action = NULL;
     SIMIX_simcall_answer(simcall);
 
@@ -709,6 +809,7 @@ void SIMIX_process_yield(smx_process_t self)
     self->doexception = 0;
     SMX_THROW();
   }
+
 }
 
 /* callback: context fetching */
@@ -763,6 +864,8 @@ xbt_dynar_t SIMIX_processes_as_dynar(void) {
   }
   return res;
 }
+
+
 void SIMIX_process_on_exit_runall(smx_process_t process) {
   s_smx_process_exit_fun_t exit_fun;
 
@@ -771,6 +874,12 @@ void SIMIX_process_on_exit_runall(smx_process_t process) {
     (exit_fun.fun)(exit_fun.arg);
   }
 }
+
+void SIMIX_pre_process_on_exit(smx_simcall_t simcall, smx_process_t process,
+	                       int_f_pvoid_t fun, void *data) {
+  SIMIX_process_on_exit(process, fun, data);
+}
+
 void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data) {
   xbt_assert(process, "current process not found: are you in maestro context ?");
 
@@ -782,6 +891,11 @@ void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data)
 
   xbt_dynar_push_as(process->on_exit,s_smx_process_exit_fun_t,exit_fun);
 }
+
+void SIMIX_pre_process_auto_restart_set(smx_simcall_t simcall, smx_process_t process,
+		                        int auto_restart) {
+  SIMIX_process_auto_restart_set(process, auto_restart);	
+}
 /**
  * \brief Sets the auto-restart status of the process.
  * If set to 1, the process will be automatically restarted when its host
@@ -790,17 +904,21 @@ void SIMIX_process_on_exit(smx_process_t process, int_f_pvoid_t fun, void *data)
 void SIMIX_process_auto_restart_set(smx_process_t process, int auto_restart) {
   process->auto_restart = auto_restart;
 }
+
+smx_process_t SIMIX_pre_process_restart(smx_simcall_t simcall, smx_process_t process) {
+  return SIMIX_process_restart(process, simcall->issuer);	
+}
 /**
  * \brief Restart a process.
  * Restart a process, starting it again from the beginning.
  */
 smx_process_t SIMIX_process_restart(smx_process_t process, smx_process_t issuer) {
-  XBT_DEBUG("Restarting process %s on %s", process->name, process->smx_host->name);
+  XBT_DEBUG("Restarting process %s on %s", process->name, sg_host_name(process->smx_host));
   //retrieve the arguments of the old process
   //FIXME: Factorise this with SIMIX_host_add_auto_restart_process ?
   s_smx_process_arg_t arg;
   arg.code = process->code;
-  arg.hostname = process->smx_host->name;
+  arg.hostname = sg_host_name(process->smx_host);
   arg.kill_time = process->kill_time;
   arg.argc = process->argc;
   arg.data = process->data;
