@@ -1,7 +1,7 @@
 /* smpi_mpi_dt.c -- MPI primitives to handle datatypes                        */
 /* FIXME: a very incomplete implementation                                    */
 
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -13,6 +13,8 @@
 
 #include "private.h"
 #include "smpi_mpi_dt_private.h"
+#include "mc/mc.h"
+#include "simgrid/modelchecker.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(smpi_mpi_dt, smpi,
                                 "Logging specific to SMPI (datatype)");
@@ -44,6 +46,14 @@ typedef struct {
   float value;
   int index;
 } float_int;
+typedef struct {
+  float value;
+  float index;
+} float_float;
+typedef struct {
+  double value;
+  double index;
+} double_double;
 typedef struct {
   long value;
   int index;
@@ -101,6 +111,9 @@ CREATE_MPI_DATATYPE(MPI_LONG_INT, long_int);
 CREATE_MPI_DATATYPE(MPI_DOUBLE_INT, double_int);
 CREATE_MPI_DATATYPE(MPI_SHORT_INT, short_int);
 CREATE_MPI_DATATYPE(MPI_2INT, int_int);
+CREATE_MPI_DATATYPE(MPI_2FLOAT, float_float);
+CREATE_MPI_DATATYPE(MPI_2DOUBLE, double_double);
+
 CREATE_MPI_DATATYPE(MPI_LONG_DOUBLE_INT, long_double_int);
 
 CREATE_MPI_DATATYPE_NULL(MPI_UB);
@@ -130,27 +143,22 @@ MPI_Aint smpi_datatype_ub(MPI_Datatype datatype)
 int smpi_datatype_extent(MPI_Datatype datatype, MPI_Aint * lb,
                          MPI_Aint * extent)
 {
-  int retval;
+  *lb = datatype->lb;
+  *extent = datatype->ub - datatype->lb;
+  return MPI_SUCCESS;
+}
 
-  if ((datatype->flags & DT_FLAG_COMMITED) != DT_FLAG_COMMITED) {
-    retval = MPI_ERR_TYPE;
-  } else {
-    *lb = datatype->lb;
-    *extent = datatype->ub - datatype->lb;
-    retval = MPI_SUCCESS;
-  }
-  return retval;
+MPI_Aint smpi_datatype_get_extent(MPI_Datatype datatype){
+  return datatype->ub - datatype->lb;
 }
 
 int smpi_datatype_copy(void *sendbuf, int sendcount, MPI_Datatype sendtype,
                        void *recvbuf, int recvcount, MPI_Datatype recvtype)
 {
-  int retval, count;
+  int count;
 
   /* First check if we really have something to do */
-  if (recvcount == 0) {
-    retval = sendcount == 0 ? MPI_SUCCESS : MPI_ERR_TRUNCATE;
-  } else {
+  if (recvcount > 0 && recvbuf != sendbuf) {
     /* FIXME: treat packed cases */
     sendcount *= smpi_datatype_size(sendtype);
     recvcount *= smpi_datatype_size(recvtype);
@@ -171,20 +179,18 @@ int smpi_datatype_copy(void *sendbuf, int sendcount, MPI_Datatype sendtype,
     }else{
       s_smpi_subtype_t *subtype =  sendtype->substruct;
 
-      s_smpi_mpi_vector_t* type_c = (s_smpi_mpi_vector_t*)sendtype;
 
-      void * buf_tmp = malloc(count * type_c->size_oldtype);
+      void * buf_tmp = xbt_malloc(count);
 
-      subtype->serialize( sendbuf, buf_tmp,1, subtype);
+      subtype->serialize( sendbuf, buf_tmp,count/smpi_datatype_size(sendtype), subtype);
       subtype =  recvtype->substruct;
-      subtype->unserialize(recvbuf, buf_tmp,1, subtype);
+      subtype->unserialize( buf_tmp, recvbuf,count/smpi_datatype_size(recvtype), subtype);
 
       free(buf_tmp);
     }
-    retval = sendcount > recvcount ? MPI_ERR_TRUNCATE : MPI_SUCCESS;
   }
 
-  return retval;
+  return sendcount > recvcount ? MPI_ERR_TRUNCATE : MPI_SUCCESS;
 }
 
 /*
@@ -207,11 +213,20 @@ void serialize_vector( const void *noncontiguous_vector,
   char* noncontiguous_vector_char = (char*)noncontiguous_vector;
 
   for (i = 0; i < type_c->block_count * count; i++) {
-    memcpy(contiguous_vector_char,
-           noncontiguous_vector_char, type_c->block_length * type_c->size_oldtype);
+      if (type_c->old_type->has_subtype == 0)
+        memcpy(contiguous_vector_char,
+               noncontiguous_vector_char, type_c->block_length * type_c->size_oldtype);
+      else
+        ((s_smpi_subtype_t*)type_c->old_type->substruct)->serialize( noncontiguous_vector_char,
+                                                                     contiguous_vector_char,
+                                                                     type_c->block_length,
+                                                                     type_c->old_type->substruct);
 
     contiguous_vector_char += type_c->block_length*type_c->size_oldtype;
-    noncontiguous_vector_char += type_c->block_stride*type_c->size_oldtype;
+    if((i+1)%type_c->block_count ==0)
+    noncontiguous_vector_char += type_c->block_length*smpi_datatype_get_extent(type_c->old_type);
+    else
+    noncontiguous_vector_char += type_c->block_stride*smpi_datatype_get_extent(type_c->old_type);
   }
 }
 
@@ -236,11 +251,19 @@ void unserialize_vector( const void *contiguous_vector,
   char* noncontiguous_vector_char = (char*)noncontiguous_vector;
 
   for (i = 0; i < type_c->block_count * count; i++) {
-    memcpy(noncontiguous_vector_char,
-           contiguous_vector_char, type_c->block_length * type_c->size_oldtype);
-
+    if (type_c->old_type->has_subtype == 0)
+      memcpy(noncontiguous_vector_char,
+             contiguous_vector_char, type_c->block_length * type_c->size_oldtype);
+    else
+      ((s_smpi_subtype_t*)type_c->old_type->substruct)->unserialize( contiguous_vector_char,
+                                                                     noncontiguous_vector_char,
+                                                                     type_c->block_length,
+                                                                     type_c->old_type->substruct);
     contiguous_vector_char += type_c->block_length*type_c->size_oldtype;
-    noncontiguous_vector_char += type_c->block_stride*type_c->size_oldtype;
+    if((i+1)%type_c->block_count ==0)
+    noncontiguous_vector_char += type_c->block_length*smpi_datatype_get_extent(type_c->old_type);
+    else
+    noncontiguous_vector_char += type_c->block_stride*smpi_datatype_get_extent(type_c->old_type);
   }
 }
 
@@ -267,31 +290,156 @@ s_smpi_mpi_vector_t* smpi_datatype_vector_create( int block_stride,
   return new_t;
 }
 
-void smpi_datatype_create(MPI_Datatype* new_type, int size,int extent, int has_subtype,
+void smpi_datatype_create(MPI_Datatype* new_type, int size,int lb, int ub, int has_subtype,
                           void *struct_type, int flags){
   MPI_Datatype new_t= xbt_new(s_smpi_mpi_datatype_t,1);
   new_t->size = size;
-  new_t->has_subtype = has_subtype;
-  new_t->lb = 0;
-  new_t->ub = extent;
+  new_t->has_subtype = size>0? has_subtype:0;
+  new_t->lb = lb;
+  new_t->ub = ub;
   new_t->flags = flags;
   new_t->substruct = struct_type;
+  new_t->in_use=0;
   *new_type = new_t;
+
+#ifdef HAVE_MC
+  if(MC_is_active())
+    MC_ignore(&(new_t->in_use), sizeof(new_t->in_use));
+#endif
 }
 
 void smpi_datatype_free(MPI_Datatype* type){
+
+  if((*type)->flags & DT_FLAG_PREDEFINED)return;
+
+  //if still used, mark for deletion
+  if((*type)->in_use!=0){
+      (*type)->flags |=DT_FLAG_DESTROYED;
+      return;
+  }
+
   if ((*type)->has_subtype == 1){
     ((s_smpi_subtype_t *)(*type)->substruct)->subtype_free(type);  
+    xbt_free((*type)->substruct);
   }
   xbt_free(*type);
+
 }
 
-int smpi_datatype_contiguous(int count, MPI_Datatype old_type, MPI_Datatype* new_type)
+void smpi_datatype_use(MPI_Datatype type){
+  if(type)type->in_use++;
+
+#ifdef HAVE_MC
+  if(MC_is_active())
+    MC_ignore(&(type->in_use), sizeof(type->in_use));
+#endif
+}
+
+
+void smpi_datatype_unuse(MPI_Datatype type){
+  if(type && type->in_use-- == 0 && (type->flags & DT_FLAG_DESTROYED))
+    smpi_datatype_free(&type);
+  
+#ifdef HAVE_MC
+  if(MC_is_active())
+    MC_ignore(&(type->in_use), sizeof(type->in_use));
+#endif
+}
+
+
+
+
+/*
+Contiguous Implementation
+*/
+
+
+/*
+ *  Copies noncontiguous data into contiguous memory.
+ *  @param contiguous_hvector - output hvector
+ *  @param noncontiguous_hvector - input hvector
+ *  @param type - pointer contening :
+ *      - stride - stride of between noncontiguous data, in bytes
+ *      - block_length - the width or height of blocked matrix
+ *      - count - the number of rows of matrix
+ */
+void serialize_contiguous( const void *noncontiguous_hvector,
+                       void *contiguous_hvector,
+                       size_t count,
+                       void *type)
+{
+  s_smpi_mpi_contiguous_t* type_c = (s_smpi_mpi_contiguous_t*)type;
+  char* contiguous_vector_char = (char*)contiguous_hvector;
+  char* noncontiguous_vector_char = (char*)noncontiguous_hvector+type_c->lb;
+  memcpy(contiguous_vector_char,
+           noncontiguous_vector_char, count* type_c->block_count * type_c->size_oldtype);
+}
+/*
+ *  Copies contiguous data into noncontiguous memory.
+ *  @param noncontiguous_vector - output hvector
+ *  @param contiguous_vector - input hvector
+ *  @param type - pointer contening :
+ *      - stride - stride of between noncontiguous data, in bytes
+ *      - block_length - the width or height of blocked matrix
+ *      - count - the number of rows of matrix
+ */
+void unserialize_contiguous( const void *contiguous_vector,
+                         void *noncontiguous_vector,
+                         size_t count,
+                         void *type)
+{
+  s_smpi_mpi_contiguous_t* type_c = (s_smpi_mpi_contiguous_t*)type;
+  char* contiguous_vector_char = (char*)contiguous_vector;
+  char* noncontiguous_vector_char = (char*)noncontiguous_vector+type_c->lb;
+
+  memcpy(noncontiguous_vector_char,
+           contiguous_vector_char, count*  type_c->block_count * type_c->size_oldtype);
+}
+
+void free_contiguous(MPI_Datatype* d){
+}
+
+/*
+ * Create a Sub type contiguous to be able to serialize and unserialize it
+ * the structure s_smpi_mpi_contiguous_t is derived from s_smpi_subtype which
+ * required the functions unserialize and serialize
+ *
+ */
+s_smpi_mpi_contiguous_t* smpi_datatype_contiguous_create( MPI_Aint lb,
+                                                  int block_count,
+                                                  MPI_Datatype old_type,
+                                                  int size_oldtype){
+  s_smpi_mpi_contiguous_t *new_t= xbt_new(s_smpi_mpi_contiguous_t,1);
+  new_t->base.serialize = &serialize_contiguous;
+  new_t->base.unserialize = &unserialize_contiguous;
+  new_t->base.subtype_free = &free_contiguous;
+  new_t->lb = lb;
+  new_t->block_count = block_count;
+  new_t->old_type = old_type;
+  new_t->size_oldtype = size_oldtype;
+  return new_t;
+}
+
+
+
+
+int smpi_datatype_contiguous(int count, MPI_Datatype old_type, MPI_Datatype* new_type, MPI_Aint lb)
 {
   int retval;
-  smpi_datatype_create(new_type, count *
-                       smpi_datatype_size(old_type),count *
-                       smpi_datatype_size(old_type),0,NULL, DT_FLAG_CONTIGUOUS);
+  if(old_type->has_subtype){
+	  //handle this case as a hvector with stride equals to the extent of the datatype
+	  return smpi_datatype_hvector(count, 1, smpi_datatype_get_extent(old_type), old_type, new_type);
+  }
+  
+  s_smpi_mpi_contiguous_t* subtype = smpi_datatype_contiguous_create( lb,
+                                                                count,
+                                                                old_type,
+                                                                smpi_datatype_size(old_type));
+                                                                
+  smpi_datatype_create(new_type,
+					  count * smpi_datatype_size(old_type),
+					  lb,lb + count * smpi_datatype_size(old_type),
+					  1,subtype, DT_FLAG_CONTIGUOUS);
   retval=MPI_SUCCESS;
   return retval;
 }
@@ -299,18 +447,24 @@ int smpi_datatype_contiguous(int count, MPI_Datatype old_type, MPI_Datatype* new
 int smpi_datatype_vector(int count, int blocklen, int stride, MPI_Datatype old_type, MPI_Datatype* new_type)
 {
   int retval;
-  if (blocklen<=0) return MPI_ERR_ARG;
-  if(stride != blocklen){
-  if (old_type->has_subtype == 1)
-    XBT_WARN("vector contains a complex type - not yet handled");
+  if (blocklen<0) return MPI_ERR_ARG;
+  MPI_Aint lb = 0;
+  MPI_Aint ub = 0;
+  if(count>0){
+    lb=smpi_datatype_lb(old_type);
+    ub=((count-1)*stride+blocklen-1)*smpi_datatype_get_extent(old_type)+smpi_datatype_ub(old_type);
+  }
+  if(old_type->has_subtype || stride != blocklen){
+
+
     s_smpi_mpi_vector_t* subtype = smpi_datatype_vector_create( stride,
                                                                 blocklen,
                                                                 count,
                                                                 old_type,
                                                                 smpi_datatype_size(old_type));
-    smpi_datatype_create(new_type, count * (blocklen) *
-                         smpi_datatype_size(old_type),
-                        ((count -1) * stride + blocklen) * smpi_datatype_size(old_type),
+    smpi_datatype_create(new_type,
+                         count * (blocklen) * smpi_datatype_size(old_type), lb,
+                         ub,
                          1,
                          subtype,
                          DT_FLAG_VECTOR);
@@ -319,7 +473,7 @@ int smpi_datatype_vector(int count, int blocklen, int stride, MPI_Datatype old_t
     /* in this situation the data are contignous thus it's not
      * required to serialize and unserialize it*/
     smpi_datatype_create(new_type, count * blocklen *
-                         smpi_datatype_size(old_type), ((count -1) * stride + blocklen)*
+                         smpi_datatype_size(old_type), 0, ((count -1) * stride + blocklen)*
                          smpi_datatype_size(old_type),
                          0,
                          NULL,
@@ -357,10 +511,19 @@ void serialize_hvector( const void *noncontiguous_hvector,
   char* noncontiguous_vector_char = (char*)noncontiguous_hvector;
 
   for (i = 0; i < type_c->block_count * count; i++) {
-    memcpy(contiguous_vector_char,
+    if (type_c->old_type->has_subtype == 0)
+      memcpy(contiguous_vector_char,
            noncontiguous_vector_char, type_c->block_length * type_c->size_oldtype);
+    else
+      ((s_smpi_subtype_t*)type_c->old_type->substruct)->serialize( noncontiguous_vector_char,
+                                                                   contiguous_vector_char,
+                                                                   type_c->block_length,
+                                                                   type_c->old_type->substruct);
 
     contiguous_vector_char += type_c->block_length*type_c->size_oldtype;
+    if((i+1)%type_c->block_count ==0)
+    noncontiguous_vector_char += type_c->block_length*type_c->size_oldtype;
+    else
     noncontiguous_vector_char += type_c->block_stride;
   }
 }
@@ -385,10 +548,18 @@ void unserialize_hvector( const void *contiguous_vector,
   char* noncontiguous_vector_char = (char*)noncontiguous_vector;
 
   for (i = 0; i < type_c->block_count * count; i++) {
-    memcpy(noncontiguous_vector_char,
+    if (type_c->old_type->has_subtype == 0)
+      memcpy(noncontiguous_vector_char,
            contiguous_vector_char, type_c->block_length * type_c->size_oldtype);
-
+    else
+      ((s_smpi_subtype_t*)type_c->old_type->substruct)->unserialize( contiguous_vector_char,
+                                                                     noncontiguous_vector_char,
+                                                                     type_c->block_length,
+                                                                     type_c->old_type->substruct);
     contiguous_vector_char += type_c->block_length*type_c->size_oldtype;
+    if((i+1)%type_c->block_count ==0)
+    noncontiguous_vector_char += type_c->block_length*type_c->size_oldtype;
+    else
     noncontiguous_vector_char += type_c->block_stride;
   }
 }
@@ -423,26 +594,29 @@ void free_hvector(MPI_Datatype* d){
 int smpi_datatype_hvector(int count, int blocklen, MPI_Aint stride, MPI_Datatype old_type, MPI_Datatype* new_type)
 {
   int retval;
-  if (blocklen<=0) return MPI_ERR_ARG;
-  if (old_type->has_subtype == 1)
-      XBT_WARN("hvector contains a complex type - not yet handled");
-  if(stride != blocklen*smpi_datatype_size(old_type)){
+  if (blocklen<0) return MPI_ERR_ARG;
+  MPI_Aint lb = 0;
+  MPI_Aint ub = 0;
+  if(count>0){
+    lb=smpi_datatype_lb(old_type);
+    ub=((count-1)*stride)+(blocklen-1)*smpi_datatype_get_extent(old_type)+smpi_datatype_ub(old_type);
+  }
+  if(old_type->has_subtype || stride != blocklen*smpi_datatype_get_extent(old_type)){
     s_smpi_mpi_hvector_t* subtype = smpi_datatype_hvector_create( stride,
                                                                   blocklen,
                                                                   count,
                                                                   old_type,
                                                                   smpi_datatype_size(old_type));
 
-    smpi_datatype_create(new_type, count * blocklen *
-                         smpi_datatype_size(old_type), (count-1) * stride + blocklen *
-                         smpi_datatype_size(old_type),
+    smpi_datatype_create(new_type, count * blocklen * smpi_datatype_size(old_type),
+						 lb,ub,
                          1,
                          subtype,
                          DT_FLAG_VECTOR);
     retval=MPI_SUCCESS;
   }else{
     smpi_datatype_create(new_type, count * blocklen *
-                                             smpi_datatype_size(old_type),count * blocklen *
+                                             smpi_datatype_size(old_type),0,count * blocklen *
                                              smpi_datatype_size(old_type),
                                             0,
                                             NULL,
@@ -474,15 +648,22 @@ void serialize_indexed( const void *noncontiguous_indexed,
   s_smpi_mpi_indexed_t* type_c = (s_smpi_mpi_indexed_t*)type;
   int i,j;
   char* contiguous_indexed_char = (char*)contiguous_indexed;
-  char* noncontiguous_indexed_char = (char*)noncontiguous_indexed;
+  char* noncontiguous_indexed_char = (char*)noncontiguous_indexed+type_c->block_indices[0] * type_c->size_oldtype;
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(contiguous_indexed_char,
-             noncontiguous_indexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      if (type_c->old_type->has_subtype == 0)
+        memcpy(contiguous_indexed_char,
+                     noncontiguous_indexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      else
+        ((s_smpi_subtype_t*)type_c->old_type->substruct)->serialize( noncontiguous_indexed_char,
+                                                                     contiguous_indexed_char,
+                                                                     type_c->block_lengths[i],
+                                                                     type_c->old_type->substruct);
+
 
       contiguous_indexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
-      if (i<type_c->block_count-1)noncontiguous_indexed_char = (char*)noncontiguous_indexed + type_c->block_indices[i+1]*type_c->size_oldtype;
-      else noncontiguous_indexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
+      if (i<type_c->block_count-1)noncontiguous_indexed_char = (char*)noncontiguous_indexed + type_c->block_indices[i+1]*smpi_datatype_get_extent(type_c->old_type);
+      else noncontiguous_indexed_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_type);
     }
     noncontiguous_indexed=(void*)noncontiguous_indexed_char;
   }
@@ -501,19 +682,26 @@ void unserialize_indexed( const void *contiguous_indexed,
                          size_t count,
                          void *type)
 {
+
   s_smpi_mpi_indexed_t* type_c = (s_smpi_mpi_indexed_t*)type;
   int i,j;
-
   char* contiguous_indexed_char = (char*)contiguous_indexed;
-  char* noncontiguous_indexed_char = (char*)noncontiguous_indexed;
+  char* noncontiguous_indexed_char = (char*)noncontiguous_indexed+type_c->block_indices[0]*smpi_datatype_get_extent(type_c->old_type);
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(noncontiguous_indexed_char,
+      if (type_c->old_type->has_subtype == 0)
+        memcpy(noncontiguous_indexed_char ,
              contiguous_indexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      else
+        ((s_smpi_subtype_t*)type_c->old_type->substruct)->unserialize( contiguous_indexed_char,
+                                                                       noncontiguous_indexed_char,
+                                                                       type_c->block_lengths[i],
+                                                                       type_c->old_type->substruct);
 
       contiguous_indexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
-      if (i<type_c->block_count-1)noncontiguous_indexed_char = (char*)noncontiguous_indexed + type_c->block_indices[i+1]*type_c->size_oldtype;
-      else noncontiguous_indexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
+      if (i<type_c->block_count-1)
+        noncontiguous_indexed_char = (char*)noncontiguous_indexed + type_c->block_indices[i+1]*smpi_datatype_get_extent(type_c->old_type);
+      else noncontiguous_indexed_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_type);
     }
     noncontiguous_indexed=(void*)noncontiguous_indexed_char;
   }
@@ -559,15 +747,27 @@ int smpi_datatype_indexed(int count, int* blocklens, int* indices, MPI_Datatype
   int retval;
   int size = 0;
   int contiguous=1;
+  MPI_Aint lb = 0;
+  MPI_Aint ub = 0;
+  if(count>0){
+    lb=indices[0]*smpi_datatype_get_extent(old_type);
+    ub=indices[0]*smpi_datatype_get_extent(old_type) + blocklens[0]*smpi_datatype_ub(old_type);
+  }
+
   for(i=0; i< count; i++){
-    if   (blocklens[i]<=0)
+    if   (blocklens[i]<0)
       return MPI_ERR_ARG;
     size += blocklens[i];
 
+    if(indices[i]*smpi_datatype_get_extent(old_type)+smpi_datatype_lb(old_type)<lb)
+    	lb = indices[i]*smpi_datatype_get_extent(old_type)+smpi_datatype_lb(old_type);
+    if(indices[i]*smpi_datatype_get_extent(old_type)+blocklens[i]*smpi_datatype_ub(old_type)>ub)
+    	ub = indices[i]*smpi_datatype_get_extent(old_type)+blocklens[i]*smpi_datatype_ub(old_type);
+
     if ( (i< count -1) && (indices[i]+blocklens[i] != indices[i+1]) )contiguous=0;
   }
   if (old_type->has_subtype == 1)
-    XBT_WARN("indexed contains a complex type - not yet handled");
+    contiguous=0;
 
   if(!contiguous){
     s_smpi_mpi_indexed_t* subtype = smpi_datatype_indexed_create( blocklens,
@@ -576,11 +776,14 @@ int smpi_datatype_indexed(int count, int* blocklens, int* indices, MPI_Datatype
                                                                   old_type,
                                                                   smpi_datatype_size(old_type));
      smpi_datatype_create(new_type,  size *
-                         smpi_datatype_size(old_type),(indices[count-1]+blocklens[count-1])*smpi_datatype_size(old_type),1, subtype, DT_FLAG_DATA);
+                         smpi_datatype_size(old_type),lb,ub,1, subtype, DT_FLAG_DATA);
   }else{
+    s_smpi_mpi_contiguous_t* subtype = smpi_datatype_contiguous_create( lb,
+                                                                  size,
+                                                                  old_type,
+                                                                  smpi_datatype_size(old_type));
     smpi_datatype_create(new_type,  size *
-                         smpi_datatype_size(old_type),size *
-                         smpi_datatype_size(old_type),0, NULL, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
+                         smpi_datatype_size(old_type),lb,ub,1, subtype, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
   }
   retval=MPI_SUCCESS;
   return retval;
@@ -608,15 +811,21 @@ void serialize_hindexed( const void *noncontiguous_hindexed,
   s_smpi_mpi_hindexed_t* type_c = (s_smpi_mpi_hindexed_t*)type;
   int i,j;
   char* contiguous_hindexed_char = (char*)contiguous_hindexed;
-  char* noncontiguous_hindexed_char = (char*)noncontiguous_hindexed;
+  char* noncontiguous_hindexed_char = (char*)noncontiguous_hindexed+ type_c->block_indices[0];
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(contiguous_hindexed_char,
-             noncontiguous_hindexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      if (type_c->old_type->has_subtype == 0)
+        memcpy(contiguous_hindexed_char,
+                     noncontiguous_hindexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      else
+        ((s_smpi_subtype_t*)type_c->old_type->substruct)->serialize( noncontiguous_hindexed_char,
+                                                                     contiguous_hindexed_char,
+                                                                     type_c->block_lengths[i],
+                                                                     type_c->old_type->substruct);
 
       contiguous_hindexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
       if (i<type_c->block_count-1)noncontiguous_hindexed_char = (char*)noncontiguous_hindexed + type_c->block_indices[i+1];
-      else noncontiguous_hindexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
+      else noncontiguous_hindexed_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_type);
     }
     noncontiguous_hindexed=(void*)noncontiguous_hindexed_char;
   }
@@ -639,15 +848,21 @@ void unserialize_hindexed( const void *contiguous_hindexed,
   int i,j;
 
   char* contiguous_hindexed_char = (char*)contiguous_hindexed;
-  char* noncontiguous_hindexed_char = (char*)noncontiguous_hindexed;
+  char* noncontiguous_hindexed_char = (char*)noncontiguous_hindexed+ type_c->block_indices[0];
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(noncontiguous_hindexed_char,
-             contiguous_hindexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      if (type_c->old_type->has_subtype == 0)
+        memcpy(noncontiguous_hindexed_char,
+               contiguous_hindexed_char, type_c->block_lengths[i] * type_c->size_oldtype);
+      else
+        ((s_smpi_subtype_t*)type_c->old_type->substruct)->unserialize( contiguous_hindexed_char,
+                                                                       noncontiguous_hindexed_char,
+                                                                       type_c->block_lengths[i],
+                                                                       type_c->old_type->substruct);
 
       contiguous_hindexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
       if (i<type_c->block_count-1)noncontiguous_hindexed_char = (char*)noncontiguous_hindexed + type_c->block_indices[i+1];
-      else noncontiguous_hindexed_char += type_c->block_lengths[i]*type_c->size_oldtype;
+      else noncontiguous_hindexed_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_type);
     }
     noncontiguous_hindexed=(void*)noncontiguous_hindexed_char;
   }
@@ -693,27 +908,43 @@ int smpi_datatype_hindexed(int count, int* blocklens, MPI_Aint* indices, MPI_Dat
   int retval;
   int size = 0;
   int contiguous=1;
+  MPI_Aint lb = 0;
+  MPI_Aint ub = 0;
+  if(count>0){
+    lb=indices[0] + smpi_datatype_lb(old_type);
+    ub=indices[0] + blocklens[0]*smpi_datatype_ub(old_type);
+  }
   for(i=0; i< count; i++){
-    if   (blocklens[i]<=0)
+    if   (blocklens[i]<0)
       return MPI_ERR_ARG;
     size += blocklens[i];
+
+    if(indices[i]+smpi_datatype_lb(old_type)<lb) lb = indices[i]+smpi_datatype_lb(old_type);
+    if(indices[i]+blocklens[i]*smpi_datatype_ub(old_type)>ub) ub = indices[i]+blocklens[i]*smpi_datatype_ub(old_type);
+
     if ( (i< count -1) && (indices[i]+blocklens[i]*smpi_datatype_size(old_type) != indices[i+1]) )contiguous=0;
   }
-  if (old_type->has_subtype == 1)
-    XBT_WARN("hindexed contains a complex type - not yet handled");
+  if (old_type->has_subtype == 1 || lb!=0)
+    contiguous=0;
+
   if(!contiguous){
     s_smpi_mpi_hindexed_t* subtype = smpi_datatype_hindexed_create( blocklens,
                                                                   indices,
                                                                   count,
                                                                   old_type,
                                                                   smpi_datatype_size(old_type));
-    smpi_datatype_create(new_type,  size *
-                         smpi_datatype_size(old_type),indices[count-1]+blocklens[count-1]*smpi_datatype_size(old_type)
+    smpi_datatype_create(new_type,  size * smpi_datatype_size(old_type),
+						 lb,
+                         ub
                          ,1, subtype, DT_FLAG_DATA);
   }else{
-    smpi_datatype_create(new_type,  size *
-                         smpi_datatype_size(old_type),size *
-                         smpi_datatype_size(old_type),0, NULL, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
+    s_smpi_mpi_contiguous_t* subtype = smpi_datatype_contiguous_create( lb,
+                                                                  size,
+                                                                  old_type,
+                                                                  smpi_datatype_size(old_type));
+    smpi_datatype_create(new_type,  size * smpi_datatype_size(old_type),
+					     0,size * smpi_datatype_size(old_type),
+					     1, subtype, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
   }
   retval=MPI_SUCCESS;
   return retval;
@@ -741,14 +972,22 @@ void serialize_struct( const void *noncontiguous_struct,
   s_smpi_mpi_struct_t* type_c = (s_smpi_mpi_struct_t*)type;
   int i,j;
   char* contiguous_struct_char = (char*)contiguous_struct;
-  char* noncontiguous_struct_char = (char*)noncontiguous_struct;
+  char* noncontiguous_struct_char = (char*)noncontiguous_struct+ type_c->block_indices[0];
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(contiguous_struct_char,
+      if (type_c->old_types[i]->has_subtype == 0)
+        memcpy(contiguous_struct_char,
              noncontiguous_struct_char, type_c->block_lengths[i] * smpi_datatype_size(type_c->old_types[i]));
+      else
+        ((s_smpi_subtype_t*)type_c->old_types[i]->substruct)->serialize( noncontiguous_struct_char,
+                                                                         contiguous_struct_char,
+                                                                         type_c->block_lengths[i],
+                                                                         type_c->old_types[i]->substruct);
+
+
       contiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_size(type_c->old_types[i]);
       if (i<type_c->block_count-1)noncontiguous_struct_char = (char*)noncontiguous_struct + type_c->block_indices[i+1];
-      else noncontiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_size(type_c->old_types[i]);//let's hope this is MPI_UB ?
+      else noncontiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_types[i]);//let's hope this is MPI_UB ?
     }
     noncontiguous_struct=(void*)noncontiguous_struct_char;
   }
@@ -771,14 +1010,21 @@ void unserialize_struct( const void *contiguous_struct,
   int i,j;
 
   char* contiguous_struct_char = (char*)contiguous_struct;
-  char* noncontiguous_struct_char = (char*)noncontiguous_struct;
+  char* noncontiguous_struct_char = (char*)noncontiguous_struct+ type_c->block_indices[0];
   for(j=0; j<count;j++){
     for (i = 0; i < type_c->block_count; i++) {
-      memcpy(noncontiguous_struct_char,
+      if (type_c->old_types[i]->has_subtype == 0)
+        memcpy(noncontiguous_struct_char,
              contiguous_struct_char, type_c->block_lengths[i] * smpi_datatype_size(type_c->old_types[i]));
+      else
+        ((s_smpi_subtype_t*)type_c->old_types[i]->substruct)->unserialize( contiguous_struct_char,
+                                                                           noncontiguous_struct_char,
+                                                                           type_c->block_lengths[i],
+                                                                           type_c->old_types[i]->substruct);
+
       contiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_size(type_c->old_types[i]);
       if (i<type_c->block_count-1)noncontiguous_struct_char =  (char*)noncontiguous_struct + type_c->block_indices[i+1];
-      else noncontiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_size(type_c->old_types[i]);
+      else noncontiguous_struct_char += type_c->block_lengths[i]*smpi_datatype_get_extent(type_c->old_types[i]);
     }
     noncontiguous_struct=(void*)noncontiguous_struct_char;
     
@@ -828,12 +1074,32 @@ int smpi_datatype_struct(int count, int* blocklens, MPI_Aint* indices, MPI_Datat
   size_t size = 0;
   int contiguous=1;
   size = 0;
+  MPI_Aint lb = 0;
+  MPI_Aint ub = 0;
+  if(count>0){
+    lb=indices[0] + smpi_datatype_lb(old_types[0]);
+    ub=indices[0] + blocklens[0]*smpi_datatype_ub(old_types[0]);
+  }
+  int forced_lb=0;
+  int forced_ub=0;
   for(i=0; i< count; i++){
-    if (blocklens[i]<=0)
+    if (blocklens[i]<0)
       return MPI_ERR_ARG;
     if (old_types[i]->has_subtype == 1)
-      XBT_WARN("Struct contains a complex type - not yet handled");
+      contiguous=0;
+
     size += blocklens[i]*smpi_datatype_size(old_types[i]);
+    if (old_types[i]==MPI_LB){
+      lb=indices[i];
+      forced_lb=1;
+    }
+    if (old_types[i]==MPI_UB){
+      ub=indices[i];
+      forced_ub=1;
+    }
+
+    if(!forced_lb && indices[i]+smpi_datatype_lb(old_types[i])<lb) lb = indices[i];
+    if(!forced_ub && indices[i]+blocklens[i]*smpi_datatype_ub(old_types[i])>ub) ub = indices[i]+blocklens[i]*smpi_datatype_ub(old_types[i]);
 
     if ( (i< count -1) && (indices[i]+blocklens[i]*smpi_datatype_size(old_types[i]) != indices[i+1]) )contiguous=0;
   }
@@ -844,9 +1110,13 @@ int smpi_datatype_struct(int count, int* blocklens, MPI_Aint* indices, MPI_Datat
                                                               count,
                                                               old_types);
 
-    smpi_datatype_create(new_type,  size, indices[count-1] + blocklens[count-1]*smpi_datatype_size(old_types[count-1]),1, subtype, DT_FLAG_DATA);
+    smpi_datatype_create(new_type,  size, lb, ub,1, subtype, DT_FLAG_DATA);
   }else{
-    smpi_datatype_create(new_type,  size, indices[count-1] + blocklens[count-1]*smpi_datatype_size(old_types[count-1]),0, NULL, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
+    s_smpi_mpi_contiguous_t* subtype = smpi_datatype_contiguous_create( lb,
+                                                                  size,
+                                                                  MPI_CHAR,
+                                                                  1);
+    smpi_datatype_create(new_type,  size, lb, ub,1, subtype, DT_FLAG_DATA|DT_FLAG_CONTIGUOUS);
   }
   return MPI_SUCCESS;
 }
@@ -858,6 +1128,7 @@ void smpi_datatype_commit(MPI_Datatype *datatype)
 
 typedef struct s_smpi_mpi_op {
   MPI_User_function *func;
+  int is_commute;
 } s_smpi_mpi_op_t;
 
 #define MAX_OP(a, b)  (b) = (a) < (b) ? (b) : (a)
@@ -1148,6 +1419,10 @@ static void minloc_func(void *a, void *b, int *length,
     APPLY_FUNC(a, b, length, int_int, MINLOC_OP);
   } else if (*datatype == MPI_LONG_DOUBLE_INT) {
     APPLY_FUNC(a, b, length, long_double_int, MINLOC_OP);
+  } else if (*datatype == MPI_2FLOAT) {
+    APPLY_FUNC(a, b, length, float_float, MINLOC_OP);
+  } else if (*datatype == MPI_2DOUBLE) {
+    APPLY_FUNC(a, b, length, double_double, MINLOC_OP);
   }
 }
 
@@ -1166,12 +1441,16 @@ static void maxloc_func(void *a, void *b, int *length,
     APPLY_FUNC(a, b, length, int_int, MAXLOC_OP);
   } else if (*datatype == MPI_LONG_DOUBLE_INT) {
     APPLY_FUNC(a, b, length, long_double_int, MAXLOC_OP);
+  } else if (*datatype == MPI_2FLOAT) {
+    APPLY_FUNC(a, b, length, float_float, MAXLOC_OP);
+  } else if (*datatype == MPI_2DOUBLE) {
+    APPLY_FUNC(a, b, length, double_double, MAXLOC_OP);
   }
 }
 
 
 #define CREATE_MPI_OP(name, func)                             \
-  static s_smpi_mpi_op_t mpi_##name = { &(func) /* func */ }; \
+  static s_smpi_mpi_op_t mpi_##name = { &(func) /* func */, TRUE }; \
 MPI_Op name = &mpi_##name;
 
 CREATE_MPI_OP(MPI_MAX, max_func);
@@ -1190,13 +1469,17 @@ CREATE_MPI_OP(MPI_MINLOC, minloc_func);
 MPI_Op smpi_op_new(MPI_User_function * function, int commute)
 {
   MPI_Op op;
-
-  //FIXME: add commute param
   op = xbt_new(s_smpi_mpi_op_t, 1);
   op->func = function;
+  op-> is_commute = commute;
   return op;
 }
 
+int smpi_op_is_commute(MPI_Op op)
+{
+  return (op==MPI_OP_NULL) ? 1 : op-> is_commute;
+}
+
 void smpi_op_destroy(MPI_Op op)
 {
   xbt_free(op);
