@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -178,7 +178,7 @@ void lagrange_solve(lmm_system_t sys)
   lmm_variable_t var = NULL;
 
   /*
-   * Auxiliar variables.
+   * Auxiliary variables.
    */
   int iteration = 0;
   double tmp = 0;
@@ -268,8 +268,8 @@ void lagrange_solve(lmm_system_t sys)
       if (var->bound >= 0) {
         XBT_DEBUG("Working on var (%p)", var);
         var->new_mu = new_mu(var);
-/* 	dual_updated += (fabs(var->new_mu-var->mu)>dichotomy_min_error); */
-/* 	XBT_DEBUG("dual_updated (%d) : %1.20f",dual_updated,fabs(var->new_mu-var->mu)); */
+/*   dual_updated += (fabs(var->new_mu-var->mu)>dichotomy_min_error); */
+/*   XBT_DEBUG("dual_updated (%d) : %1.20f",dual_updated,fabs(var->new_mu-var->mu)); */
         XBT_DEBUG("Updating mu : var->mu (%p) : %1.20f -> %1.20f", var,
                var->mu, var->new_mu);
         var->mu = var->new_mu;
@@ -438,16 +438,16 @@ static double dichotomy(double init, double diff(double, void *),
         min = middle;
         overall_error = max_diff - middle_diff;
         min_diff = middle_diff;
-/* 	SHOW_EXPR(overall_error); */
+/*   SHOW_EXPR(overall_error); */
       } else if (middle_diff > 0) {
         XBT_CDEBUG(surf_lagrange_dichotomy, "Decreasing max");
         max = middle;
         overall_error = max_diff - middle_diff;
         max_diff = middle_diff;
-/* 	SHOW_EXPR(overall_error); */
+/*   SHOW_EXPR(overall_error); */
       } else {
         overall_error = 0;
-/* 	SHOW_EXPR(overall_error); */
+/*   SHOW_EXPR(overall_error); */
       }
     } else if (min_diff == 0) {
       max = min;
@@ -460,12 +460,12 @@ static double dichotomy(double init, double diff(double, void *),
     } else if (min_diff > 0 && max_diff < 0) {
       XBT_CWARN(surf_lagrange_dichotomy,
              "The impossible happened, partial_diff(min) > 0 && partial_diff(max) < 0");
-      abort();
+      xbt_abort();
     } else {
       XBT_CWARN(surf_lagrange_dichotomy,
              "diffmin (%1.20f) or diffmax (%1.20f) are something I don't know, taking no action.",
              min_diff, max_diff);
-      abort();
+      xbt_abort();
     }
   }
 
