@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2008-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -18,7 +18,6 @@ typedef struct surf_action_network_Constant {
 } s_surf_action_network_Constant_t, *surf_action_network_Constant_t;
 
 XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(surf_network);
-static random_data_t random_latency = NULL;
 static int host_number_int = 0;
 
 static void netcste_count_hosts(sg_platf_host_cbarg_t h) {
@@ -126,7 +125,7 @@ static surf_action_t netcste_communicate(sg_routing_edge_t src,
 
   action->suspended = 0;
 
-  action->latency = sg_latency_factor;          //random_generate(random_latency);
+  action->latency = sg_latency_factor;
   action->lat_init = action->latency;
 
   if (action->latency <= 0.0) {
@@ -150,19 +149,19 @@ static void netcste_action_set_category(surf_action_t action, const char *catego
 static double netcste_get_link_bandwidth(const void *link)
 {
   DIE_IMPOSSIBLE;
-  return -1.0;
+  return -1.0; /* useless since DIE actually abort(), but eclipse prefer to have a useless and harmless return */
 }
 
 static double netcste_get_link_latency(const void *link)
 {
   DIE_IMPOSSIBLE;
-  return -1.0;
+  return -1.0; /* useless since DIE actually abort(), but eclipse prefer to have a useless and harmless return */
 }
 
 static int link_shared(const void *link)
 {
   DIE_IMPOSSIBLE;
-  return -1;
+  return -1; /* useless since DIE actually abort(), but eclipse prefer to have a useless and harmless return */
 }
 
 static void netcste_action_suspend(surf_action_t action)
@@ -200,7 +199,7 @@ void surf_network_model_init_Constant()
   surf_network_model->action_unref = netcste_action_unref;
   surf_network_model->action_cancel = netcste_action_cancel;
   surf_network_model->action_recycle = net_action_recycle;
-  surf_network_model->get_remains = net_action_get_remains;
+  surf_network_model->get_remains = surf_action_get_remains;
 #ifdef HAVE_LATENCY_BOUND_TRACKING
   surf_network_model->get_latency_limited = net_get_link_latency_limited;
 #endif
@@ -229,8 +228,6 @@ void surf_network_model_init_Constant()
   surf_network_model->set_category = netcste_action_set_category;
 #endif
 
-  if (!random_latency)
-    random_latency = random_new(RAND, 100, 0.0, 1.0, .125, .034);
   netcste_define_callbacks();
   xbt_dynar_push(model_list, &surf_network_model);
 
