@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -362,7 +362,7 @@ void surf_network_model_init_NS3()
 
   /* Added the initialization for NS3 interface */
 
-  if (ns3_initialize(xbt_cfg_get_string(_surf_cfg_set,"ns3/TcpModel"))) {
+  if (ns3_initialize(xbt_cfg_get_string(_sg_cfg_set, "ns3/TcpModel"))) {
     xbt_die("Impossible to initialize NS3 interface");
   }
 
