@@ -1,4 +1,4 @@
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -6,7 +6,7 @@
 
 #include "surf/random_mgr.h"
 #include "xbt/sysdep.h"
-#include "gras_config.h" /*_XBT_WIN32*/
+#include "internal_config.h" /*_XBT_WIN32*/
 #include <math.h>
 #include <stdlib.h>
 
