@@ -1,16 +1,16 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012. The SimGrid Team.
- * All rights reserved.                                                                 */
+/* Copyright (c) 2004-2013. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
- * under the terms of the license (GNU LGPL) which comes with this package.             */
+ * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "xbt/ex.h"
 #include "xbt/dict.h"
-#include "xbt/file_stat.h"
 #include "portable.h"
 #include "surf_private.h"
 #include "storage_private.h"
 #include "surf/surf_resource.h"
+#include <inttypes.h>
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_storage, surf,
                                 "Logging specific to the SURF storage module");
@@ -35,34 +35,26 @@ static xbt_dynar_t storage_list;
 #define GENERIC_LMM_ACTION(action) action->generic_lmm_action
 #define GENERIC_ACTION(action) GENERIC_LMM_ACTION(action).generic_action
 
-static xbt_dict_t parse_storage_content(char *filename, unsigned long *used_size);
+static xbt_dict_t parse_storage_content(char *filename, sg_storage_size_t *used_size);
+static int storage_action_unref(surf_action_t action);
 static void storage_action_state_set(surf_action_t action, e_surf_action_state_t state);
-static surf_action_t storage_action_execute (void *storage, double size, e_surf_action_storage_type_t type);
-static void free_storage_content(void *p);
-
-static surf_action_t storage_action_stat(void *storage, surf_file_t stream)
-{
-  surf_action_t action = storage_action_execute(storage,0, STAT);
-  action->file = stream;
-  action->stat = stream->content->stat;
-  return action;
-}
+static surf_action_t storage_action_execute (void *storage, sg_storage_size_t size, e_surf_action_storage_type_t type);
 
 static surf_action_t storage_action_ls(void *storage, const char* path)
 {
   surf_action_t action = storage_action_execute(storage,0, LS);
   action->ls_dict = NULL;
-  xbt_dict_t ls_dict = xbt_dict_new();
+  xbt_dict_t ls_dict = xbt_dict_new_homogeneous(xbt_free);
 
   char* key;
-  surf_stat_t data = NULL;
+  sg_storage_size_t size = 0;
   xbt_dict_cursor_t cursor = NULL;
 
   xbt_dynar_t dyn = NULL;
   char* file = NULL;
 
-  // foreach file int the storage
-  xbt_dict_foreach(((storage_t)storage)->content,cursor,key,data){
+  // for each file in the storage content
+  xbt_dict_foreach(((storage_t)storage)->content,cursor,key,size){
     // Search if file start with the prefix 'path'
     if(xbt_str_start_with(key,path)){
       file = &key[strlen(path)];
@@ -73,12 +65,14 @@ static surf_action_t storage_action_ls(void *storage, const char* path)
 
       // file
       if(xbt_dynar_length(dyn) == 1){
-        xbt_dict_set(ls_dict,file,&(data->stat),NULL);
+        sg_storage_size_t *psize = xbt_new(sg_storage_size_t,1);
+        *psize=size;
+        xbt_dict_set(ls_dict,file,psize,NULL);
       }
       // Directory
       else
       {
-        // if directory does not exist yet in dict
+        // if directory does not exist yet in the dictionary
         if(!xbt_dict_get_or_null(ls_dict,file))
           xbt_dict_set(ls_dict,file,NULL,NULL);
       }
@@ -90,80 +84,69 @@ static surf_action_t storage_action_ls(void *storage, const char* path)
   return action;
 }
 
-static surf_action_t storage_action_unlink(void *storage, surf_file_t stream)
-{
-  surf_action_t action = storage_action_execute(storage,0, UNLINK);
-
-  // Add memory to storage
-  ((storage_t)storage)->used_size -= stream->content->stat.size;
-
-  // Remove the file from storage
-  xbt_dict_t content_dict = ((storage_t)storage)->content;
-  xbt_dict_remove(content_dict,stream->name);
-
-  free(stream->name);
-  stream->content = NULL;
-  xbt_free(stream);
-
-  return action;
-}
-
-static surf_action_t storage_action_open(void *storage, const char* mount, const char* path, const char* mode)
+static surf_action_t storage_action_open(void *storage, const char* mount,
+                                         const char* path)
 {
   XBT_DEBUG("\tOpen file '%s'",path);
   xbt_dict_t content_dict = ((storage_t)storage)->content;
-  surf_stat_t content = xbt_dict_get_or_null(content_dict,path);
-
+  sg_storage_size_t size, *psize;
+  psize = (sg_storage_size_t*) xbt_dict_get_or_null(content_dict, path);
+  if (psize)
+    size = *psize;
+  else {
   // if file does not exist create an empty file
-  if(!content){
-    content = xbt_new0(s_surf_stat_t,1);
-    content->stat.date = xbt_strdup("");
-    content->stat.group = xbt_strdup("");
-    content->stat.size = 0;
-    content->stat.time = xbt_strdup("");
-    content->stat.user = xbt_strdup("");
-    content->stat.user_rights = xbt_strdup("");
-    xbt_dict_set(content_dict,path,content,NULL);
+    psize = xbt_new(sg_storage_size_t,1);
+    size = 0;
+    *psize = size;
+    xbt_dict_set(content_dict,path,psize,NULL);
     XBT_DEBUG("File '%s' was not found, file created.",path);
   }
   surf_file_t file = xbt_new0(s_surf_file_t,1);
   file->name = xbt_strdup(path);
-  file->content = content;
-  file->storage = mount;
+  file->size = size;
+  file->mount = xbt_strdup(mount);
 
   surf_action_t action = storage_action_execute(storage,0, OPEN);
   action->file = (void *)file;
   return action;
 }
 
-static surf_action_t storage_action_close(void *storage, surf_file_t fp)
+static surf_action_t storage_action_close(void *storage, surf_file_t fd)
 {
-  char *filename = fp->name;
-  XBT_DEBUG("\tClose file '%s' size '%f'",filename,fp->content->stat.size);
-  free(fp->name);
-  fp->content = NULL;
-  xbt_free(fp);
+  char *filename = fd->name;
+  XBT_DEBUG("\tClose file '%s' size '%" PRIu64 "'", filename, fd->size);
+  // unref write actions from storage
+  surf_action_storage_t write_action;
+  unsigned int i;
+  xbt_dynar_foreach(((storage_t)storage)->write_actions,i,write_action) {
+    if ((write_action->generic_lmm_action.generic_action.file) == fd) {
+      xbt_dynar_cursor_rm(((storage_t)storage)->write_actions, &i);
+      storage_action_unref((surf_action_t) write_action);
+    }
+  }
+  free(fd->name);
+  free(fd->mount);
+  xbt_free(fd);
   surf_action_t action = storage_action_execute(storage,0, CLOSE);
   return action;
 }
 
-static surf_action_t storage_action_read(void *storage, void* ptr, double size, size_t nmemb, surf_file_t stream)
+static surf_action_t storage_action_read(void *storage, surf_file_t fd, sg_storage_size_t size)
 {
-  surf_stat_t content = stream->content;
-  if(size > content->stat.size)
-    size = content->stat.size;
+  if(size > fd->size)
+    size = fd->size;
   surf_action_t action = storage_action_execute(storage,size,READ);
   return action;
 }
 
-static surf_action_t storage_action_write(void *storage, const void* ptr, size_t size, size_t nmemb, surf_file_t stream)
+static surf_action_t storage_action_write(void *storage, surf_file_t fd, sg_storage_size_t size)
 {
-  char *filename = stream->name;
-  surf_stat_t content = stream->content;
-  XBT_DEBUG("\tWrite file '%s' size '%zu/%f'",filename,size,content->stat.size);
+  char *filename = fd->name;
+  XBT_DEBUG("\tWrite file '%s' size '%" PRIu64 "/%" PRIu64 "'",
+            filename, size, fd->size);
 
   surf_action_t action = storage_action_execute(storage,size,WRITE);
-  action->file = stream;
+  action->file = fd;
 
   // If the storage is full
   if(((storage_t)storage)->used_size==((storage_t)storage)->size) {
@@ -172,12 +155,12 @@ static surf_action_t storage_action_write(void *storage, const void* ptr, size_t
   return action;
 }
 
-static surf_action_t storage_action_execute (void *storage, double size, e_surf_action_storage_type_t type)
+static surf_action_t storage_action_execute (void *storage, sg_storage_size_t size, e_surf_action_storage_type_t type)
 {
   surf_action_storage_t action = NULL;
   storage_t STORAGE = storage;
 
-  XBT_IN("(%s,%f)", surf_resource_name(STORAGE), size);
+  XBT_IN("(%s,%" PRIu64, surf_resource_name(STORAGE), size);
   action =
       surf_action_new(sizeof(s_surf_action_storage_t), size, surf_storage_model,
           STORAGE->state_current != SURF_RESOURCE_ON);
@@ -198,7 +181,6 @@ static surf_action_t storage_action_execute (void *storage, double size, e_surf_
   case OPEN:
   case CLOSE:
   case STAT:
-  case UNLINK:
   case LS:
     break;
   case READ:
@@ -209,6 +191,7 @@ static surf_action_t storage_action_execute (void *storage, double size, e_surf_
     lmm_expand(storage_maxmin_system, STORAGE->constraint_write,
                GENERIC_LMM_ACTION(action).variable, 1.0);
     xbt_dynar_push(((storage_t)storage)->write_actions,&action);
+    surf_action_ref((surf_action_t) action);
     break;
   }
   action->type = type;
@@ -216,15 +199,60 @@ static surf_action_t storage_action_execute (void *storage, double size, e_surf_
   return (surf_action_t) action;
 }
 
-static void* storage_create_resource(const char* id, const char* model,const char* type_id,const char* content_name)
+static xbt_dict_t storage_get_properties(const void *storage)
 {
+  return surf_resource_properties(surf_storage_resource_priv(storage));
+}
+
+static xbt_dict_t storage_get_content(void *storage)
+{
+  /* For the moment this action has no cost, but in the future we could take in account access latency of the disk */
+  /*surf_action_t action = storage_action_execute(storage,0, LS);*/
+
+  void *storage_resource = surf_storage_resource_priv(storage);
+  xbt_dict_t content_dict = xbt_dict_new_homogeneous(NULL);
+  xbt_dict_cursor_t cursor = NULL;
+  char *file;
+  sg_storage_size_t *psize;
+
+  xbt_dict_foreach(((storage_t)storage_resource)->content, cursor, file, psize){
+    xbt_dict_set(content_dict,file,psize,NULL);
+  }
+  return content_dict;
+}
+
+static sg_storage_size_t storage_get_size(void *storage){
+  void *storage_resource = surf_storage_resource_priv(storage);
+  return ((storage_t)storage_resource)->size;
+}
+
+static void storage_file_rename(void *storage, const char *src, const char *dest)
+{
+  void *storage_resource = surf_storage_resource_priv(storage);
+
+  sg_storage_size_t *psize, *new_psize;
+  psize = (sg_storage_size_t*) xbt_dict_get_or_null(((storage_t)storage_resource)->content,src);
+  new_psize = xbt_new(sg_storage_size_t, 1);
+  *new_psize = *psize;
+  if (psize){// src file exists
+    xbt_dict_remove(((storage_t)storage_resource)->content, src);
+    xbt_dict_set(((storage_t)storage_resource)->content, dest, new_psize,NULL);
+    XBT_DEBUG("Change file name from %s to %s, size '%" PRIu64 "'",src, dest, *psize);
+  }
+  else
+    XBT_DEBUG("File %s doesn't exist",src);
+}
+
+
+static void* storage_create_resource(const char* id, const char* model,
+    const char* type_id, const char* content_name, const char* content_type, xbt_dict_t properties){
   storage_t storage = NULL;
 
-  xbt_assert(!surf_storage_resource_by_name(id),
+  xbt_assert(!surf_storage_resource_priv(surf_storage_resource_by_name(id)),
               "Storage '%s' declared several times in the platform file",
               id);
   storage = (storage_t) surf_resource_new(sizeof(s_storage_t),
-          surf_storage_model, id,NULL);
+          surf_storage_model, id, properties, NULL);
 
   storage->state_current = SURF_RESOURCE_ON;
   storage->used_size = 0;
@@ -232,15 +260,21 @@ static void* storage_create_resource(const char* id, const char* model,const cha
   storage->write_actions = xbt_dynar_new(sizeof(char *),NULL);
 
   storage_type_t storage_type = xbt_lib_get_or_null(storage_type_lib, type_id,ROUTING_STORAGE_TYPE_LEVEL);
-  double Bread  = atof(xbt_dict_get(storage_type->properties,"Bread"));
-  double Bwrite = atof(xbt_dict_get(storage_type->properties,"Bwrite"));
-  double Bconnection   = atof(xbt_dict_get(storage_type->properties,"Bconnection"));
-  XBT_DEBUG("Create resource with Bconnection '%f' Bread '%f' Bwrite '%f' and Size '%ld'",Bconnection,Bread,Bwrite,storage_type->size);
+  double Bread =
+      surf_parse_get_bandwidth(xbt_dict_get(storage_type->properties,"Bread"));
+  double Bwrite =
+      surf_parse_get_bandwidth(xbt_dict_get(storage_type->properties,"Bwrite"));
+  double Bconnection =
+      surf_parse_get_bandwidth(xbt_dict_get(storage_type->properties,
+                                            "Bconnection"));
+  XBT_DEBUG("Create resource with Bconnection '%f' Bread '%f' Bwrite '%f' and Size '%lu'",Bconnection,Bread,Bwrite,(unsigned long)storage_type->size);
   storage->constraint       = lmm_constraint_new(storage_maxmin_system, storage, Bconnection);
   storage->constraint_read  = lmm_constraint_new(storage_maxmin_system, storage, Bread);
   storage->constraint_write = lmm_constraint_new(storage_maxmin_system, storage, Bwrite);
   storage->content = parse_storage_content((char*)content_name,&(storage->used_size));
+  storage->content_type = xbt_strdup(content_type);
   storage->size = storage_type->size;
+  storage->type_id = xbt_strdup(type_id);
 
   xbt_lib_set(storage_lib, id, SURF_STORAGE_LEVEL, storage);
 
@@ -251,7 +285,8 @@ static void* storage_create_resource(const char* id, const char* model,const cha
       storage_type->properties,
       Bread);
 
-  if(!storage_list) storage_list=xbt_dynar_new(sizeof(char *),NULL);
+  if (!storage_list)
+    storage_list = xbt_dynar_new(sizeof(char *),NULL);
   xbt_dynar_push(storage_list,&storage);
 
   return storage;
@@ -265,8 +300,7 @@ static void storage_finalize(void)
   surf_model_exit(surf_storage_model);
   surf_storage_model = NULL;
 
-  if(storage_list)
-    xbt_dynar_free(&storage_list);
+  xbt_dynar_free(&storage_list);
 
   xbt_swag_free
       (storage_running_action_set_that_does_not_need_being_checked);
@@ -279,19 +313,26 @@ static void storage_update_actions_state(double now, double delta)
   surf_action_storage_t next_action = NULL;
   xbt_swag_t running_actions = surf_storage_model->states.running_action_set;
 
-  // Update the disk usage
-  // Update the file size
-  // Foreach action of type write
+
   xbt_swag_foreach_safe(action, next_action, running_actions) {
     if(action->type == WRITE)
     {
+      // Update the disk usage
+      // Update the file size
+      // Update the storage content (with file size)
       double rate = lmm_variable_getvalue(GENERIC_LMM_ACTION(action).variable);
-      ((storage_t)(action->storage))->used_size += delta * rate; // disk usage
-      ((surf_action_t)action)->file->content->stat.size += delta * rate; // file size
-    }
-  }
+      /* Hack to avoid rounding differences between x86 and x86_64
+       * (note that the next sizes are of type sg_storage_size_t). */
+      long incr = delta * rate + MAXMIN_PRECISION;
+      ((storage_t)(action->storage))->used_size += incr; // disk usage
+      ((surf_action_t)action)->file->size += incr; // file size
 
-  xbt_swag_foreach_safe(action, next_action, running_actions) {
+      sg_storage_size_t *psize = xbt_new(sg_storage_size_t,1);
+      *psize = ((surf_action_t)action)->file->size;
+
+      xbt_dict_t content_dict = ((storage_t)(action->storage))->content;
+      xbt_dict_set(content_dict,((surf_action_t)action)->file->name,psize,NULL);
+    }
 
     double_update(&(GENERIC_ACTION(action).remains),
                   lmm_variable_getvalue(GENERIC_LMM_ACTION(action).variable) * delta);
@@ -433,20 +474,28 @@ static void parse_storage_init(sg_platf_storage_cbarg_t storage)
   // if storage content is not specified use the content of storage_type if exist
   if(!strcmp(storage->content,"") && strcmp(((storage_type_t) stype)->content,"")){
     storage->content = ((storage_type_t) stype)->content;
-    XBT_DEBUG("For disk '%s' content is empty, use the content of storage type '%s'",storage->id,((storage_type_t) stype)->type_id);
+    storage->content_type = ((storage_type_t) stype)->content_type;
+    XBT_DEBUG("For disk '%s' content is empty, inherit the content (of type %s) from storage type '%s' ",
+        storage->id,((storage_type_t) stype)->content_type,
+        ((storage_type_t) stype)->type_id);
   }
 
-  XBT_DEBUG("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' \n\t\tmodel '%s' \n\t\tcontent '%s'\n\t\tproperties '%p'\n",
+  XBT_DEBUG("SURF storage create resource\n\t\tid '%s'\n\t\ttype '%s' "
+      "\n\t\tmodel '%s' \n\t\tcontent '%s'\n\t\tcontent_type '%s' "
+      "\n\t\tproperties '%p'\n",
       storage->id,
       ((storage_type_t) stype)->model,
       ((storage_type_t) stype)->type_id,
       storage->content,
+      storage->content_type,
       ((storage_type_t) stype)->properties);
 
   storage_create_resource(storage->id,
      ((storage_type_t) stype)->model,
      ((storage_type_t) stype)->type_id,
-     storage->content);
+     storage->content,
+     storage->content_type,
+     storage->properties);
 }
 
 static void parse_mstorage_init(sg_platf_mstorage_cbarg_t mstorage)
@@ -503,15 +552,14 @@ static void surf_storage_model_init_internal(void)
   surf_storage_model->extension.storage.close = storage_action_close;
   surf_storage_model->extension.storage.read = storage_action_read;
   surf_storage_model->extension.storage.write = storage_action_write;
-  surf_storage_model->extension.storage.stat = storage_action_stat;
-  surf_storage_model->extension.storage.unlink = storage_action_unlink;
   surf_storage_model->extension.storage.ls = storage_action_ls;
-  surf_storage_model->extension.storage.create_resource = storage_create_resource;
-
+  surf_storage_model->extension.storage.get_properties = storage_get_properties;
+  surf_storage_model->extension.storage.get_content = storage_get_content;
+  surf_storage_model->extension.storage.get_size = storage_get_size;
+  surf_storage_model->extension.storage.rename = storage_file_rename;
   if (!storage_maxmin_system) {
     storage_maxmin_system = lmm_system_new(storage_selective_update);
   }
-
 }
 
 void surf_storage_model_init_default(void)
@@ -544,24 +592,13 @@ static void storage_parse_storage(sg_platf_storage_cbarg_t storage)
       (void *) xbt_strdup(storage->type_id));
 }
 
-static void free_storage_content(void *p)
-{
-  surf_stat_t content = p;
-  free(content->stat.date);
-  free(content->stat.group);
-  free(content->stat.time);
-  free(content->stat.user);
-  free(content->stat.user_rights);
-  free(content);
-}
-
-static xbt_dict_t parse_storage_content(char *filename, unsigned long *used_size)
+static xbt_dict_t parse_storage_content(char *filename, sg_storage_size_t *used_size)
 {
   *used_size = 0;
   if ((!filename) || (strcmp(filename, "") == 0))
     return NULL;
 
-  xbt_dict_t parse_content = xbt_dict_new_homogeneous(free_storage_content);
+  xbt_dict_t parse_content = xbt_dict_new_homogeneous(xbt_free);
   FILE *file = NULL;
 
   file = surf_fopen(filename, "r");
@@ -571,36 +608,22 @@ static xbt_dict_t parse_storage_content(char *filename, unsigned long *used_size
   char *line = NULL;
   size_t len = 0;
   ssize_t read;
-  char user_rights[12];
-  char user[100];
-  char group[100];
-  char date[12];
-  char time[12];
   char path[1024];
-  int nb;
-  unsigned long size;
+  sg_storage_size_t size;
 
-  surf_stat_t content;
-
-  while ((read = getline(&line, &len, file)) != -1) {
-    content = xbt_new0(s_surf_stat_t,1);
-    if(sscanf(line,"%s %d %s %s %ld %s %s %s",user_rights,&nb,user,group,&size,date,time,path)==8) {
-      content->stat.date = xbt_strdup(date);
-      content->stat.group = xbt_strdup(group);
-      content->stat.size = size;
-      content->stat.time = xbt_strdup(time);
-      content->stat.user = xbt_strdup(user);
-      content->stat.user_rights = xbt_strdup(user_rights);
-      *used_size += content->stat.size;
-      xbt_dict_set(parse_content,path,content,NULL);
-    } else {
-      xbt_die("Be sure of passing a good format for content file.\n");
-      // You can generate this kind of file with command line:
-      // find /path/you/want -type f -exec ls -l {} \; 2>/dev/null > ./content.txt
+  while ((read = xbt_getline(&line, &len, file)) != -1) {
+    if (read){
+    if (sscanf(line,"%s %" SCNu64, path, &size) == 2) {
+        *used_size += size;
+        sg_storage_size_t *psize = xbt_new(sg_storage_size_t, 1);
+        *psize = size;
+        xbt_dict_set(parse_content,path,psize,NULL);
+      } else {
+        xbt_die("Be sure of passing a good format for content file.\n");
+      }
     }
   }
-  if (line)
-      free(line);
+  free(line);
   fclose(file);
   return parse_content;
 }
@@ -614,13 +637,16 @@ static void storage_parse_storage_type(sg_platf_storage_type_cbarg_t storage_typ
   stype->model = xbt_strdup(storage_type->model);
   stype->properties = storage_type->properties;
   stype->content = xbt_strdup(storage_type->content);
+  stype->content_type = xbt_strdup(storage_type->content_type);
   stype->type_id = xbt_strdup(storage_type->id);
-  stype->size = storage_type->size * 1000000000; /* storage_type->size is in Gbytes and stype->sizeis in bytes */
+  stype->size = storage_type->size;
 
-  XBT_DEBUG("ROUTING Create a storage type id '%s' with model '%s' content '%s'",
+  XBT_DEBUG("ROUTING Create a storage type id '%s' with model '%s', "
+      "content '%s', and content_type '%s'",
       stype->type_id,
       stype->model,
-      storage_type->content);
+      storage_type->content,
+      storage_type->content_type);
 
   xbt_lib_set(storage_type_lib,
       stype->type_id,
@@ -655,14 +681,15 @@ static void storage_parse_mount(sg_platf_mount_cbarg_t mount)
 {
   // Verification of an existing storage
 #ifndef NDEBUG
-  void* storage = xbt_lib_get_or_null(storage_lib, mount->id,ROUTING_STORAGE_LEVEL);
+  void* storage = xbt_lib_get_or_null(storage_lib, mount->storageId,ROUTING_STORAGE_LEVEL);
 #endif
-  xbt_assert(storage,"Disk id \"%s\" does not exists", mount->id);
+  xbt_assert(storage,"Disk id \"%s\" does not exists", mount->storageId);
 
-  XBT_DEBUG("ROUTING Mount '%s' on '%s'",mount->id, mount->name);
+  XBT_DEBUG("ROUTING Mount '%s' on '%s'",mount->storageId, mount->name);
 
   s_mount_t mnt;
-  mnt.id = surf_storage_resource_by_name(mount->id);
+  mnt.storage =
+    surf_storage_resource_priv(surf_storage_resource_by_name(mount->storageId));
   mnt.name = xbt_strdup(mount->name);
 
   if(!mount_list){
@@ -678,6 +705,7 @@ static XBT_INLINE void routing_storage_type_free(void *r)
   free(stype->model);
   free(stype->type_id);
   free(stype->content);
+  free(stype->content_type);
   xbt_dict_free(&(stype->properties));
   free(stype);
 }
@@ -688,6 +716,8 @@ static XBT_INLINE void surf_storage_resource_free(void *r)
   storage_t storage = r;
   xbt_dict_free(&storage->content);
   xbt_dynar_free(&storage->write_actions);
+  free(storage->type_id);
+  free(storage->content_type);
   // generic resource
   surf_resource_free(r);
 }
@@ -710,3 +740,4 @@ void storage_register_callbacks() {
   sg_platf_storage_type_add_cb(storage_parse_storage_type);
   sg_platf_mount_add_cb(storage_parse_mount);
 }
+
