@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -10,6 +10,7 @@
 #include "simix/smx_host_private.h"
 #include "surf/surf_resource.h"
 #include "xbt/xbt_os_thread.h"
+#include "simgrid/sg_config.h"
 
 #include <ctype.h>
 
@@ -113,6 +114,8 @@ xbt_dynar_t model_list = NULL;
 tmgr_history_t history = NULL;
 lmm_system_t maxmin_system = NULL;
 xbt_dynar_t surf_path = NULL;
+xbt_dynar_t host_that_restart = NULL;
+xbt_dict_t watched_hosts_lib;
 
 /* Don't forget to update the option description in smx_config when you change this */
 s_surf_model_description_t surf_network_model_description[] = {
@@ -203,7 +206,6 @@ s_surf_model_description_t surf_new_model_description[] = {
 static xbt_parmap_t surf_parmap = NULL; /* parallel map on models */
 #endif
 
-static int surf_nthreads = 1;    /* number of threads of the parmap (1 means no parallelism) */
 static double *surf_mins = NULL; /* return value of share_resources for each model */
 static int surf_min_index;       /* current index in surf_mins */
 static double min;               /* duration determined by surf_solve */
@@ -231,7 +233,9 @@ int find_model_description(s_surf_model_description_t * table,
     if (!strcmp(name, table[i].name)) {
       return i;
     }
-  name_list = strdup(table[0].name);
+  if (!table[0].name)
+    xbt_die("No model is valid! This is a bug.");
+  name_list = xbt_strdup(table[0].name);
   for (i = 1; table[i].name; i++) {
     name_list =
         xbt_realloc(name_list,
@@ -350,14 +354,14 @@ double generic_share_resources_lazy(double now, surf_model_t model)
       max_dur_flag = 1;
     }
 
-    XBT_DEBUG("Action(%p) Start %lf Finish %lf Max_duration %lf", action,
+    XBT_DEBUG("Action(%p) Start %f Finish %f Max_duration %f", action,
         action->generic_action.start, now + value,
         action->generic_action.max_duration);
 
     if (min != -1) {
       surf_action_lmm_heap_remove(model->model_private->action_heap,action);
       surf_action_lmm_heap_insert(model->model_private->action_heap,action, min, max_dur_flag ? MAX_DURATION : NORMAL);
-      XBT_DEBUG("Insert at heap action(%p) min %lf now %lf", action, min,
+      XBT_DEBUG("Insert at heap action(%p) min %f now %f", action, min,
                 now);
     } else DIE_IMPOSSIBLE;
   }
@@ -368,7 +372,7 @@ double generic_share_resources_lazy(double now, surf_model_t model)
   else
     min = -1;
 
-  XBT_DEBUG("The minimum with the HEAP %lf", min);
+  XBT_DEBUG("The minimum with the HEAP %f", min);
 
   return min;
 }
@@ -391,8 +395,6 @@ void sg_version(int *ver_major,int *ver_minor,int *ver_patch) {
   *ver_patch = SIMGRID_VERSION_PATCH;
 }
 
-xbt_dynar_t sg_cmdline = NULL;
-
 void surf_init(int *argc, char **argv)
 {
   XBT_DEBUG("Create all Libs");
@@ -401,7 +403,7 @@ void surf_init(int *argc, char **argv)
   as_router_lib = xbt_lib_new();
   storage_lib = xbt_lib_new();
   storage_type_lib = xbt_lib_new();
-  watched_hosts_lib = xbt_dict_new();
+  watched_hosts_lib = xbt_dict_new_homogeneous(NULL);
 
   XBT_DEBUG("Add routing levels");
   ROUTING_HOST_LEVEL = xbt_lib_add_level(host_lib,routing_asr_host_free);
@@ -413,18 +415,19 @@ void surf_init(int *argc, char **argv)
   SURF_WKS_LEVEL = xbt_lib_add_level(host_lib,surf_resource_free);
   SURF_LINK_LEVEL = xbt_lib_add_level(link_lib,surf_resource_free);
 
-  sg_cmdline = xbt_dynar_new(sizeof(char*),NULL);
-  int i;
-  for (i=0;i<*argc;i++) {
-    xbt_dynar_push(sg_cmdline,&(argv[i]));
-  }
   xbt_init(argc, argv);
   if (!model_list)
     model_list = xbt_dynar_new(sizeof(surf_model_private_t), NULL);
   if (!history)
     history = tmgr_history_new();
 
-  surf_config_init(argc, argv);
+#ifdef HAVE_TRACING
+  TRACE_add_start_function(TRACE_surf_alloc);
+  TRACE_add_end_function(TRACE_surf_release);
+#endif
+
+  sg_config_init(argc, argv);
+
   surf_action_init();
   if (MC_is_active())
     MC_memory_init();
@@ -465,7 +468,12 @@ void surf_exit(void)
   unsigned int iter;
   surf_model_t model = NULL;
 
-  surf_config_finalize();
+#ifdef HAVE_TRACING
+  TRACE_end();                  /* Just in case it was not called by the upper
+                                 * layer (or there is no upper layer) */
+#endif
+
+  sg_config_finalize();
 
   xbt_dynar_foreach(model_list, iter, model)
       model->model_private->finalize();
@@ -487,9 +495,8 @@ void surf_exit(void)
   xbt_free(surf_mins);
   surf_mins = NULL;
 #endif
-
+  xbt_dynar_free(&host_that_restart);
   xbt_dynar_free(&surf_path);
-  xbt_dynar_free(&sg_cmdline);
 
   xbt_lib_free(&host_lib);
   xbt_lib_free(&link_lib);
@@ -524,9 +531,11 @@ void surf_presolve(void)
             tmgr_history_get_next_event_leq(history, next_event_date,
                                             &value,
                                             (void **) &resource))) {
-      resource->model->model_private->update_resource_state(resource,
-                                                            event, value,
-                                                            NOW);
+      if (value >= 0){
+        resource->model->model_private->update_resource_state(resource,
+                                                              event, value,
+                                                              NOW);
+      }
     }
   }
   xbt_dynar_foreach(model_list, iter, model)
@@ -544,6 +553,9 @@ double surf_solve(double max_date)
   tmgr_trace_event_t event = NULL;
   unsigned int iter;
 
+  if(!host_that_restart)
+    host_that_restart = xbt_dynar_new(sizeof(char*), NULL);
+
   if (max_date != -1.0 && max_date != NOW) {
     min = max_date - NOW;
   }
@@ -555,20 +567,9 @@ double surf_solve(double max_date)
   }
   surf_min_index = 0;
 
-
-  if (surf_get_nthreads() > 1) {
-    /* parallel version */
-#ifdef CONTEXT_THREADS
-    xbt_parmap_apply(surf_parmap, (void_f_pvoid_t) surf_share_resources, model_list);
-#else
-    xbt_die("Asked to run in parallel, but no thread at hand...");
-#endif
-  }
-  else {
-    /* sequential version */
-    xbt_dynar_foreach(model_list, iter, model) {
-      surf_share_resources(model);
-    }
+  /* sequential version */
+  xbt_dynar_foreach(model_list, iter, model) {
+    surf_share_resources(model);
   }
 
   unsigned i;
@@ -595,28 +596,30 @@ double surf_solve(double max_date)
         min = MAX(next_event_date - NOW, min);
       }
 
-      XBT_DEBUG("Run for NS3 at most %f", min);
+      XBT_DEBUG("Run for network at most %f", min);
       // run until min or next flow
       model_next_action_end = surf_network_model->model_private->share_resources(min);
 
-      XBT_DEBUG("Min for NS3 : %f", model_next_action_end);
+      XBT_DEBUG("Min for network : %f", model_next_action_end);
       if(model_next_action_end>=0.0)
         min = model_next_action_end;
     }
 
-    if (next_event_date == -1.0) {
+    if (next_event_date < 0.0) {
       XBT_DEBUG("no next TRACE event. Stop searching for it");
       break;
     }
 
-    if ((min != -1.0) && (next_event_date > NOW + min)) break;
+    if ((min == -1.0) || (next_event_date > NOW + min)) break;
 
-    XBT_DEBUG("Updating models");
+    XBT_DEBUG("Updating models (min = %g, NOW = %g, next_event_date = %g)",min, NOW, next_event_date);
     while ((event =
             tmgr_history_get_next_event_leq(history, next_event_date,
                                             &value,
                                             (void **) &resource))) {
-      if (resource->model->model_private->resource_used(resource)) {
+      if (resource->model->model_private->resource_used(resource) ||
+          xbt_dict_get_or_null(watched_hosts_lib,resource->name)
+          ) {
         min = next_event_date - NOW;
         XBT_DEBUG
             ("This event will modify model state. Next event set to %f",
@@ -624,11 +627,12 @@ double surf_solve(double max_date)
       }
       /* update state of model_obj according to new value. Does not touch lmm.
          It will be modified if needed when updating actions */
-      XBT_DEBUG("Calling update_resource_state for resource %s with min %lf",
-             resource->model->name, min);
+      XBT_DEBUG("Calling update_resource_state for resource %s with min %f",
+             resource->name, min);
+
       resource->model->model_private->update_resource_state(resource,
                                                             event, value,
-                                                            NOW + min);
+                                                            next_event_date);
     }
   } while (1);
 
@@ -644,17 +648,9 @@ double surf_solve(double max_date)
 
   NOW = NOW + min;
 
-  if (surf_get_nthreads() > 1) {
-    /* parallel version */
-#ifdef CONTEXT_THREADS
-    xbt_parmap_apply(surf_parmap, (void_f_pvoid_t) surf_update_actions_state, model_list);
-#endif
-  }
-  else {
-    /* sequential version */
-    xbt_dynar_foreach(model_list, iter, model) {
-      surf_update_actions_state(model);
-    }
+  /* sequential version */
+  xbt_dynar_foreach(model_list, iter, model) {
+    surf_update_actions_state(model);
   }
 
 #ifdef HAVE_TRACING
@@ -687,59 +683,3 @@ static void surf_update_actions_state(surf_model_t model)
   model->model_private->update_actions_state(NOW, min);
 }
 
-/**
- * \brief Returns the number of parallel threads used to update the models.
- * \return the number of threads (1 means no parallelism)
- */
-int surf_get_nthreads(void) {
-  return surf_nthreads;
-}
-
-/**
- * \brief Sets the number of parallel threads used to update the models.
- *
- * A value of 1 means no parallelism.
- *
- * \param nb_threads the number of threads to use
- */
-void surf_set_nthreads(int nthreads) {
-
-  if (nthreads<=0) {
-     nthreads = xbt_os_get_numcores();
-     XBT_INFO("Auto-setting surf/nthreads to %d",nthreads);
-  }
-
-#ifdef CONTEXT_THREADS
-  xbt_parmap_destroy(surf_parmap);
-  surf_parmap = NULL;
-#endif
-
-  if (nthreads > 1) {
-#ifdef CONTEXT_THREADS
-    surf_parmap = xbt_parmap_new(nthreads, XBT_PARMAP_DEFAULT);
-#else
-    THROWF(arg_error, 0, "Cannot activate parallel threads in Surf: your architecture does not support threads");
-#endif
-  }
-
-  surf_nthreads = nthreads;
-}
-
-void surf_watched_hosts(void)
-{
-  char *key;
-  void *host;
-  xbt_dict_cursor_t cursor;
-
-  XBT_DEBUG("Check for host SURF_RESOURCE_ON on watched_hosts_lib");
-  xbt_dict_foreach(watched_hosts_lib,cursor,key,host)
-  {
-    if(SIMIX_host_get_state(host) == SURF_RESOURCE_ON){
-      XBT_INFO("Restart processes on host: %s",SIMIX_host_get_name(host));
-      SIMIX_host_autorestart(host);
-      xbt_dict_remove(watched_hosts_lib,key);
-    }
-    else
-      XBT_DEBUG("See SURF_RESOURCE_OFF on host: %s",key);
-  }
-}
