@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -17,19 +17,6 @@
 
 #define NO_MAX_DURATION -1.0
 
-/* user-visible parameters */
-extern double sg_tcp_gamma;
-extern double sg_sender_gap;
-extern double sg_latency_factor;
-extern double sg_bandwidth_factor;
-extern double sg_weight_S_parameter;
-extern int sg_maxmin_selective_update;
-extern int sg_network_crosstraffic;
-#ifdef HAVE_GTNETS
-extern double sg_gtnets_jitter;
-extern int sg_gtnets_jitter_seed;
-#endif
-
 extern xbt_dict_t watched_hosts_lib;
 
 extern const char *surf_action_state_names[6];
@@ -93,6 +80,7 @@ void surf_action_set_category(surf_action_t action,
                                     const char *category);
 #endif
 double surf_action_get_remains(surf_action_t action);
+void update_resource_energy(surf_model_t model, surf_action_lmm_t action);
 void generic_update_action_remaining_lazy( surf_action_lmm_t action, double now);
 void generic_update_actions_state_lazy(double now, double delta, surf_model_t model);
 void generic_update_actions_state_full(double now, double delta, surf_model_t model);
@@ -100,33 +88,22 @@ void generic_update_actions_state_full(double now, double delta, surf_model_t mo
 FILE *surf_fopen(const char *name, const char *mode);
 
 extern tmgr_history_t history;
-extern xbt_dynar_t surf_path;
 
-void surf_config_init(int *argc, char **argv);
-void surf_config_finalize(void);
-void surf_config(const char *name, va_list pa);
+//void surf_config_init(int *argc, char **argv);
+//void surf_config_finalize(void);
+//void surf_config(const char *name, va_list pa);
 
 void net_action_recycle(surf_action_t action);
-double net_action_get_remains(surf_action_t action);
 #ifdef HAVE_LATENCY_BOUND_TRACKING
 int net_get_link_latency_limited(surf_action_t action);
 #endif
 
-/*
- * Returns the initial path. On Windows the initial path is
- * the current directory for the current process in the other
- * case the function returns "./" that represents the current
- * directory on Unix/Linux platforms.
- */
-const char *__surf_get_initial_path(void);
-
 /* The __surf_is_absolute_file_path() returns 1 if
  * file_path is a absolute file path, in the other
  * case the function returns 0.
  */
 int __surf_is_absolute_file_path(const char *file_path);
 
-typedef struct s_as *AS_t;
 typedef struct s_routing_edge {
   AS_t rc_component;
   e_surf_network_element_type_t rc_type;
@@ -175,6 +152,7 @@ typedef struct s_as {
   void (*get_route_and_latency) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, sg_platf_route_cbarg_t into, double *latency);
 
   xbt_dynar_t(*get_onelink_routes) (AS_t as);
+  void (*get_graph) (xbt_graph_t graph, xbt_dict_t nodes, xbt_dict_t edges, AS_t rc);
   sg_platf_route_cbarg_t(*get_bypass_route) (AS_t as, sg_routing_edge_t src, sg_routing_edge_t dst, double *lat);
   void (*finalize) (AS_t as);
 
@@ -214,6 +192,7 @@ XBT_PUBLIC(void) generic_free_route(sg_platf_route_cbarg_t route); // FIXME rena
 XBT_PUBLIC(void) routing_get_route_and_latency(sg_routing_edge_t src, sg_routing_edge_t dst,
                               xbt_dynar_t * route, double *latency);
 
+XBT_PUBLIC(void) generic_get_graph(xbt_graph_t graph, xbt_dict_t nodes, xbt_dict_t edges, AS_t rc);
 /**
  * Resource protected methods
  */
@@ -222,4 +201,10 @@ XBT_PUBLIC(void) surfxml_bufferstack_pop(int new);
 
 XBT_PUBLIC_DATA(int) surfxml_bufferstack_size;
 
+/********** Tracing **********/
+/* from surf_instr.c */
+void TRACE_surf_host_set_power(double date, const char *resource, double power);
+void TRACE_surf_link_set_bandwidth(double date, const char *resource, double bandwidth);
+
+
 #endif                          /* _SURF_SURF_PRIVATE_H */
