@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -13,12 +13,17 @@ static xbt_dynar_t none_get_onelink_routes(AS_t rc) {
 }
 
 static void none_get_route_and_latency(AS_t rc, sg_routing_edge_t src, sg_routing_edge_t dst,
-                                       route_t res,double *lat)
+    sg_platf_route_cbarg_t res,double *lat)
 {
   *lat = 0.0;
 }
 
-static route_t none_get_bypass_route(AS_t rc,
+static void none_get_graph(xbt_graph_t graph, xbt_dict_t nodes, xbt_dict_t edges, AS_t rc)
+{
+	XBT_INFO("No routing no graph");
+}
+
+static sg_platf_route_cbarg_t none_get_bypass_route(AS_t rc,
     sg_routing_edge_t src,
     sg_routing_edge_t dst, double *lat) {
   return NULL;
@@ -53,6 +58,7 @@ AS_t model_none_create_sized(size_t childsize) {
   new_component->get_onelink_routes = none_get_onelink_routes;
   new_component->get_bypass_route = none_get_bypass_route;
   new_component->finalize = model_none_finalize;
+  new_component->get_graph = none_get_graph;
   new_component->routing_sons = xbt_dict_new_homogeneous(NULL);
   new_component->index_network_elm = xbt_dynar_new(sizeof(char*),NULL);
 
@@ -62,6 +68,7 @@ AS_t model_none_create_sized(size_t childsize) {
 void model_none_finalize(AS_t as) {
   xbt_dict_free(&as->routing_sons);
   xbt_dynar_free(&as->index_network_elm);
+  xbt_dynar_free(&as->link_up_down_list);
   xbt_free(as);
 }
 
