@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -8,7 +8,7 @@
 #define _SURF_SURF_ROUTING_PRIVATE_H
 
 #include <float.h>
-#include "gras_config.h"
+#include "internal_config.h"
 
 #include "surf_private.h"
 #include "xbt/dynar.h"
@@ -57,22 +57,19 @@ generic_processing_units_exist(AS_t rc, char *element);
 void generic_src_dst_check(AS_t rc, sg_routing_edge_t src,
     sg_routing_edge_t dst);
 
-
 /* ************************************************************************** */
 /* *************************** FLOYD ROUTING ******************************** */
 AS_t model_floyd_create(void);  /* create structures for floyd routing model */
 void model_floyd_end(AS_t as);      /* finalize the creation of floyd routing model */
 void model_floyd_parse_route(AS_t rc, sg_platf_route_cbarg_t route);
 
-/* ************************************************** */
-/* ************** RULE-BASED ROUTING **************** */
-AS_t model_rulebased_create(void);      /* create structures for rulebased routing model */
-
 /* ************************************************** */
 /* **************  Cluster ROUTING   **************** */
 typedef struct {
   s_as_t generic_routing;
   void *backbone;
+  void *loopback;
+  sg_routing_edge_t router;
 } s_as_cluster_t, *as_cluster_t;
 
 AS_t model_cluster_create(void);      /* create structures for cluster routing model */
@@ -98,6 +95,10 @@ AS_t model_full_create(void);   /* create structures for full routing model */
 void model_full_end(AS_t as);       /* finalize the creation of full routing model */
 void model_full_set_route(  /* Set the route and ASroute between src and dst */
     AS_t rc, sg_platf_route_cbarg_t route);
+/* ************************************************************************** */
+/* ************************* GRAPH EXPORTING FUNCTIONS ********************** */
+xbt_node_t new_xbt_graph_node (xbt_graph_t graph, const char *name, xbt_dict_t nodes);
+xbt_edge_t new_xbt_graph_edge (xbt_graph_t graph, xbt_node_t s, xbt_node_t d, xbt_dict_t edges);
 
 
 #endif                          /* _SURF_SURF_ROUTING_PRIVATE_H */
