@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -18,15 +18,23 @@ static XBT_INLINE double euclidean_dist_comp(int index, xbt_dynar_t src, xbt_dyn
 
 static void vivaldi_get_route_and_latency(
     AS_t rc, sg_routing_edge_t src_p, sg_routing_edge_t dst_p,
-    route_t route, double *lat)
+    sg_platf_route_cbarg_t route, double *lat)
 {
+  s_surf_parsing_link_up_down_t info;
+
   XBT_DEBUG("vivaldi_get_route_and_latency from '%s'[%d] '%s'[%d]",src_p->name,src_p->id,dst_p->name,dst_p->id);
   char *src = (char*)src_p->name;
   char *dst = (char*)dst_p->name;
 
   if(src_p->rc_type == SURF_NETWORK_ELEMENT_AS) {
-    route->src_gateway = xbt_lib_get_or_null(as_router_lib,ROUTER_PEER(src),ROUTING_ASR_LEVEL);
-    route->dst_gateway = xbt_lib_get_or_null(as_router_lib,ROUTER_PEER(dst),ROUTING_ASR_LEVEL);
+    char *rp_src = ROUTER_PEER(src);
+    char *rp_dst = ROUTER_PEER(dst);
+    route->gw_src = xbt_lib_get_or_null(as_router_lib, rp_src,
+                                        ROUTING_ASR_LEVEL);
+    route->gw_dst = xbt_lib_get_or_null(as_router_lib, rp_dst,
+                                        ROUTING_ASR_LEVEL);
+    xbt_free(rp_src);
+    xbt_free(rp_dst);
   }
 
   double euclidean_dist;
@@ -35,6 +43,15 @@ static void vivaldi_get_route_and_latency(
 
   if(src_p->rc_type == SURF_NETWORK_ELEMENT_HOST){
     tmp_src_name = HOST_PEER(src);
+
+    if(rc->link_up_down_list){
+      info = xbt_dynar_get_as(rc->link_up_down_list,src_p->id,s_surf_parsing_link_up_down_t);
+      if(info.link_up) { // link up
+        xbt_dynar_push_as(route->link_list,void*,info.link_up);
+        if (lat)
+          *lat += surf_network_model->extension.network.get_link_latency(info.link_up);
+      }
+    }
     src_ctn = xbt_lib_get_or_null(host_lib, tmp_src_name, COORD_HOST_LEVEL);
     if(!src_ctn ) src_ctn = xbt_lib_get_or_null(host_lib, src, COORD_HOST_LEVEL);
   }
@@ -43,11 +60,20 @@ static void vivaldi_get_route_and_latency(
     src_ctn = xbt_lib_get_or_null(as_router_lib, tmp_src_name, COORD_ASR_LEVEL);
   }
   else{
-    xbt_die(" ");
+    THROW_IMPOSSIBLE;
   }
 
   if(dst_p->rc_type == SURF_NETWORK_ELEMENT_HOST){
     tmp_dst_name = HOST_PEER(dst);
+
+    if(rc->link_up_down_list){
+      info = xbt_dynar_get_as(rc->link_up_down_list,dst_p->id,s_surf_parsing_link_up_down_t);
+      if(info.link_down) { // link down
+        xbt_dynar_push_as(route->link_list,void*,info.link_down);
+        if (lat)
+          *lat += surf_network_model->extension.network.get_link_latency(info.link_down);
+      }
+    }
     dst_ctn = xbt_lib_get_or_null(host_lib, tmp_dst_name, COORD_HOST_LEVEL);
     if(!dst_ctn ) dst_ctn = xbt_lib_get_or_null(host_lib, dst, COORD_HOST_LEVEL);
   }
@@ -56,7 +82,7 @@ static void vivaldi_get_route_and_latency(
     dst_ctn = xbt_lib_get_or_null(as_router_lib, tmp_dst_name, COORD_ASR_LEVEL);
   }
   else{
-    xbt_die(" ");
+    THROW_IMPOSSIBLE;
   }
 
   xbt_assert(src_ctn,"No coordinate found for element '%s'",tmp_src_name);
@@ -67,15 +93,25 @@ static void vivaldi_get_route_and_latency(
   euclidean_dist = sqrt (euclidean_dist_comp(0,src_ctn,dst_ctn)+euclidean_dist_comp(1,src_ctn,dst_ctn))
                       + fabs(xbt_dynar_get_as(src_ctn, 2, double))+fabs(xbt_dynar_get_as(dst_ctn, 2, double));
 
-  if (lat)
-    *lat += euclidean_dist / 1000; //From .ms to .s
+  if (lat){
+    XBT_DEBUG("Updating latency %f += %f",*lat,euclidean_dist);
+    *lat += euclidean_dist / 1000.0; //From .ms to .s
+  }
 }
 
+static int vivaldi_parse_PU(AS_t rc, sg_routing_edge_t elm) {
+  XBT_DEBUG("Load process unit \"%s\"", elm->name);
+  xbt_dynar_push_as(rc->index_network_elm,sg_routing_edge_t,elm);
+  return xbt_dynar_length(rc->index_network_elm)-1;
+}
 
 /* Creation routing model functions */
 AS_t model_vivaldi_create(void)
 {
-  AS_t new_component = model_rulebased_create();
+  AS_t new_component = model_generic_create_sized(sizeof *new_component);
+  
   new_component->get_route_and_latency = vivaldi_get_route_and_latency;
+  new_component->parse_PU = vivaldi_parse_PU;
+  new_component->get_graph = generic_get_graph;
   return new_component;
 }
