@@ -1,9 +1,11 @@
-/* Copyright (c) 2006, 2007, 2008, 2009, 2010, 2011. The SimGrid Team.
+/* Copyright (c) 2006-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
+#include <errno.h>
+#include <math.h>
 #include <stdarg.h> /* va_arg */
 
 #include "xbt/misc.h"
@@ -12,6 +14,7 @@
 #include "xbt/dict.h"
 #include "surf/surfxml_parse.h"
 #include "surf/surf_private.h"
+#include "simgrid/sg_config.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(surf_parse, surf,
                                 "Logging specific to the SURF parsing module");
@@ -30,9 +33,12 @@ extern AS_t current_routing;
 void surf_parse_error(const char *fmt, ...) {
   va_list va;
   va_start(va,fmt);
+  int lineno = surf_parse_lineno;
   char *msg = bvprintf(fmt,va);
   va_end(va);
-  xbt_die("Parse error at %s:%d: %s", surf_parsed_filename, surf_parse_lineno, msg);
+  cleanup();
+  surf_exit();
+  xbt_die("Parse error at %s:%d: %s", surf_parsed_filename, lineno, msg);
 }
 void surf_parse_warn(const char *fmt, ...) {
   va_list va;
@@ -48,6 +54,7 @@ double surf_parse_get_double(const char *string) {
   int ret = sscanf(string, "%lg", &res);
   if (ret != 1)
     surf_parse_error("%s is not a double", string);
+  //printf("Parsed double [%g] %s\n", res, string);
   return res;
 }
 
@@ -59,6 +66,132 @@ int surf_parse_get_int(const char *string) {
   return res;
 }
 
+struct unit_scale {
+  const char *unit;
+  double scale;
+};
+
+/* Note: field `unit' for the last element of parametre `units' should be
+ * NULL. */
+static double surf_parse_get_value_with_unit(const char *string,
+                                             const struct unit_scale *units)
+{
+  char* ptr;
+  double res;
+  int i;
+  errno = 0;
+  res = strtod(string, &ptr);
+  if (errno == ERANGE)
+    surf_parse_error("value out of range: %s", string);
+  if (ptr == string)
+    surf_parse_error("cannot parse number: %s", string);
+  for (i = 0; units[i].unit != NULL && strcmp(ptr, units[i].unit) != 0; i++) {
+  }
+  if (units[i].unit != NULL)
+    res *= units[i].scale;
+  else
+    surf_parse_error("unknown unit: %s", ptr);
+  return res;
+}
+
+double surf_parse_get_time(const char *string)
+{
+  const struct unit_scale units[] = {
+    { "w",  7 * 24 * 60 * 60 },
+    { "d",  24 * 60 * 60 },
+    { "h",  60 * 60 },
+    { "m",  60 },
+    { "s",  1.0 },
+    { "",   1.0 },              /* default unit is seconds */
+    { "ms", 1e-3 },
+    { "us", 1e-6 },
+    { "ns", 1e-9 },
+    { "ps", 1e-12 },
+    { NULL, 0 }
+  };
+  return surf_parse_get_value_with_unit(string, units);
+}
+
+double surf_parse_get_size(const char *string)
+{
+  const struct unit_scale units[] = {
+    { "TiB", pow(1024, 4) },
+    { "GiB", pow(1024, 3) },
+    { "MiB", pow(1024, 2) },
+    { "KiB", 1024 },
+    { "TB",  1e12 },
+    { "GB",  1e9 },
+    { "MB",  1e6 },
+    { "kB",  1e3 },
+    { "B",   1.0 },
+    { "",      1.0 },           /* default unit is bytes*/
+    { "Tib", 0.125 * pow(1024, 4) },
+    { "Gib", 0.125 * pow(1024, 3) },
+    { "Mib", 0.125 * pow(1024, 2) },
+    { "Kib", 0.125 * 1024 },
+    { "Tb",  0.125 * 1e12 },
+    { "Gb",  0.125 * 1e9 },
+    { "Mb",  0.125 * 1e6 },
+    { "kb",  0.125 * 1e3 },
+    { "b",   0.125 },
+    { NULL,    0 }
+  };
+  return surf_parse_get_value_with_unit(string, units);
+}
+
+double surf_parse_get_bandwidth(const char *string)
+{
+  const struct unit_scale units[] = {
+    { "TiBps", pow(1024, 4) },
+    { "GiBps", pow(1024, 3) },
+    { "MiBps", pow(1024, 2) },
+    { "KiBps", 1024 },
+    { "TBps",  1e12 },
+    { "GBps",  1e9 },
+    { "MBps",  1e6 },
+    { "kBps",  1e3 },
+    { "Bps",   1.0 },
+    { "",      1.0 },           /* default unit is bytes per second */
+    { "Tibps", 0.125 * pow(1024, 4) },
+    { "Gibps", 0.125 * pow(1024, 3) },
+    { "Mibps", 0.125 * pow(1024, 2) },
+    { "Kibps", 0.125 * 1024 },
+    { "Tbps",  0.125 * 1e12 },
+    { "Gbps",  0.125 * 1e9 },
+    { "Mbps",  0.125 * 1e6 },
+    { "kbps",  0.125 * 1e3 },
+    { "bps",   0.125 },
+    { NULL,    0 }
+  };
+  return surf_parse_get_value_with_unit(string, units);
+}
+
+double surf_parse_get_power(const char *string)
+{
+  const struct unit_scale units[] = {
+    { "yottaflops", 1e24 },
+    { "Yf",         1e24 },
+    { "zettaflops", 1e21 },
+    { "Zf",         1e21 },
+    { "exaflops",   1e18 },
+    { "Ef",         1e18 },
+    { "petaflops",  1e15 },
+    { "Pf",         1e15 },
+    { "teraflops",  1e12 },
+    { "Tf",         1e12 },
+    { "gigaflops",  1e9 },
+    { "Gf",         1e9 },
+    { "megaflops",  1e6 },
+    { "Mf",         1e6 },
+    { "kiloflops",  1e3 },
+    { "kf",         1e3 },
+    { "flops",      1.0 },
+    { "f",          1.0 },
+    { "",           1.0 },      /* default unit is flops */
+    { NULL,         0 }
+  };
+  return surf_parse_get_value_with_unit(string, units);
+}
 
 /*
  * All the callback lists that can be overridden anywhere.
@@ -102,26 +235,28 @@ void ETag_surfxml_storage(void)
   storage.id = A_surfxml_storage_id;
   storage.type_id = A_surfxml_storage_typeId;
   storage.content = A_surfxml_storage_content;
+  storage.content_type = A_surfxml_storage_content___type;
   storage.properties = current_property_set;
   sg_platf_new_storage(&storage);
   current_property_set = NULL;
 }
-void STag_surfxml_storage_type(void)
+void STag_surfxml_storage___type(void)
 {
   AS_TAG = 0;
-  XBT_DEBUG("STag_surfxml_storage_type");
+  XBT_DEBUG("STag_surfxml_storage___type");
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
-void ETag_surfxml_storage_type(void)
+void ETag_surfxml_storage___type(void)
 {
   s_sg_platf_storage_type_cbarg_t storage_type;
   memset(&storage_type,0,sizeof(storage_type));
 
-  storage_type.content = A_surfxml_storage_type_content;
-  storage_type.id = A_surfxml_storage_type_id;
-  storage_type.model = A_surfxml_storage_type_model;
+  storage_type.content = A_surfxml_storage___type_content;
+  storage_type.content_type = A_surfxml_storage___type_content___type;
+  storage_type.id = A_surfxml_storage___type_id;
+  storage_type.model = A_surfxml_storage___type_model;
   storage_type.properties = current_property_set;
-  storage_type.size = surf_parse_get_int(A_surfxml_storage_type_size);
+  storage_type.size = surf_parse_get_size(A_surfxml_storage___type_size);
   sg_platf_new_storage_type(&storage_type);
   current_property_set = NULL;
 }
@@ -148,7 +283,7 @@ void ETag_surfxml_mount(void)
   memset(&mount,0,sizeof(mount));
 
   mount.name = A_surfxml_mount_name;
-  mount.id = A_surfxml_mount_id;
+  mount.storageId = A_surfxml_mount_storageId;
   sg_platf_new_mount(&mount);
 }
 
@@ -289,16 +424,45 @@ void STag_surfxml_prop(void)
 
 void ETag_surfxml_host(void)    {
   s_sg_platf_host_cbarg_t host;
+  char* buf;
   memset(&host,0,sizeof(host));
 
+
   host.properties = current_property_set;
 
   host.id = A_surfxml_host_id;
-  host.power_peak = get_cpu_power(A_surfxml_host_power);
+
+  buf = A_surfxml_host_power;
+  XBT_DEBUG("Buffer: %s", buf);
+  host.power_peak = xbt_dynar_new(sizeof(double), NULL);
+  if (strchr(buf, ',') == NULL){
+	  double power_value = get_cpu_power(A_surfxml_host_power);
+	  xbt_dynar_push_as(host.power_peak,double, power_value);
+  }
+  else {
+	  xbt_dynar_t pstate_list = xbt_str_split(buf, ",");
+	  int i;
+	  for (i = 0; i < xbt_dynar_length(pstate_list); i++) {
+		  double power_value;
+		  char* power_value_str;
+
+		  xbt_dynar_get_cpy(pstate_list, i, &power_value_str);
+		  xbt_str_trim(power_value_str, NULL);
+		  power_value = get_cpu_power(power_value_str);
+		  xbt_dynar_push_as(host.power_peak, double, power_value);
+		  XBT_DEBUG("Power value: %f", power_value);
+	  }
+	  xbt_dynar_free(&pstate_list);
+  }
+
+  XBT_DEBUG("pstate: %s", A_surfxml_host_pstate);
+  //host.power_peak = get_cpu_power(A_surfxml_host_power);
   host.power_scale = surf_parse_get_double( A_surfxml_host_availability);
   host.core_amount = surf_parse_get_int(A_surfxml_host_core);
-  host.power_trace = tmgr_trace_new_from_file(A_surfxml_host_availability_file);
-  host.state_trace = tmgr_trace_new_from_file(A_surfxml_host_state_file);
+  host.power_trace = tmgr_trace_new_from_file(A_surfxml_host_availability___file);
+  host.state_trace = tmgr_trace_new_from_file(A_surfxml_host_state___file);
+  host.pstate = surf_parse_get_int(A_surfxml_host_pstate);
+
   xbt_assert((A_surfxml_host_state == A_surfxml_host_state_ON) ||
         (A_surfxml_host_state == A_surfxml_host_state_OFF), "Invalid state");
   if (A_surfxml_host_state == A_surfxml_host_state_ON)
@@ -311,14 +475,14 @@ void ETag_surfxml_host(void)    {
   current_property_set = NULL;
 }
 
-void STag_surfxml_host_link(void){
-  XBT_DEBUG("Create a Host_link for %s",A_surfxml_host_link_id);
+void STag_surfxml_host___link(void){
+  XBT_DEBUG("Create a Host_link for %s",A_surfxml_host___link_id);
   s_sg_platf_host_link_cbarg_t host_link;
   memset(&host_link,0,sizeof(host_link));
 
-  host_link.id = A_surfxml_host_link_id;
-  host_link.link_up = A_surfxml_host_link_up;
-  host_link.link_down = A_surfxml_host_link_down;
+  host_link.id = A_surfxml_host___link_id;
+  host_link.link_up = A_surfxml_host___link_up;
+  host_link.link_down = A_surfxml_host___link_down;
   sg_platf_new_host_link(&host_link);
 }
 
@@ -332,31 +496,39 @@ void STag_surfxml_router(void){
   sg_platf_new_router(&router);
 }
 
-void STag_surfxml_cluster(void){
+void ETag_surfxml_cluster(void){
   s_sg_platf_cluster_cbarg_t cluster;
   memset(&cluster,0,sizeof(cluster));
+  cluster.properties = current_property_set;
+
   cluster.id = A_surfxml_cluster_id;
   cluster.prefix = A_surfxml_cluster_prefix;
   cluster.suffix = A_surfxml_cluster_suffix;
   cluster.radical = A_surfxml_cluster_radical;
-  cluster.power= surf_parse_get_double(A_surfxml_cluster_power);
+  cluster.power = surf_parse_get_power(A_surfxml_cluster_power);
   cluster.core_amount = surf_parse_get_int(A_surfxml_cluster_core);
-  cluster.bw =   surf_parse_get_double(A_surfxml_cluster_bw);
-  cluster.lat =  surf_parse_get_double(A_surfxml_cluster_lat);
-  if(strcmp(A_surfxml_cluster_bb_bw,""))
-    cluster.bb_bw = surf_parse_get_double(A_surfxml_cluster_bb_bw);
-  if(strcmp(A_surfxml_cluster_bb_lat,""))
-    cluster.bb_lat = surf_parse_get_double(A_surfxml_cluster_bb_lat);
-  cluster.router_id = A_surfxml_cluster_router_id;
+  cluster.bw =   surf_parse_get_bandwidth(A_surfxml_cluster_bw);
+  cluster.lat =  surf_parse_get_time(A_surfxml_cluster_lat);
+  if(strcmp(A_surfxml_cluster_bb___bw,""))
+    cluster.bb_bw = surf_parse_get_bandwidth(A_surfxml_cluster_bb___bw);
+  if(strcmp(A_surfxml_cluster_bb___lat,""))
+    cluster.bb_lat = surf_parse_get_time(A_surfxml_cluster_bb___lat);
+  if(strcmp(A_surfxml_cluster_limiter___link,""))
+    cluster.limiter_link = surf_parse_get_double(A_surfxml_cluster_limiter___link);
+  if(strcmp(A_surfxml_cluster_loopback___bw,""))
+    cluster.loopback_bw = surf_parse_get_bandwidth(A_surfxml_cluster_loopback___bw);
+  if(strcmp(A_surfxml_cluster_loopback___lat,""))
+    cluster.loopback_lat = surf_parse_get_time(A_surfxml_cluster_loopback___lat);
+  cluster.router_id = A_surfxml_cluster_router___id;
 
-  switch (AX_surfxml_cluster_sharing_policy) {
-  case A_surfxml_cluster_sharing_policy_SHARED:
+  switch (AX_surfxml_cluster_sharing___policy) {
+  case A_surfxml_cluster_sharing___policy_SHARED:
     cluster.sharing_policy = SURF_LINK_SHARED;
     break;
-  case A_surfxml_cluster_sharing_policy_FULLDUPLEX:
+  case A_surfxml_cluster_sharing___policy_FULLDUPLEX:
     cluster.sharing_policy = SURF_LINK_FULLDUPLEX;
     break;
-  case A_surfxml_cluster_sharing_policy_FATPIPE:
+  case A_surfxml_cluster_sharing___policy_FATPIPE:
     cluster.sharing_policy = SURF_LINK_FATPIPE;
     break;
   default:
@@ -364,11 +536,11 @@ void STag_surfxml_cluster(void){
                      cluster.id);
     break;
   }
-  switch (AX_surfxml_cluster_bb_sharing_policy) {
-  case A_surfxml_cluster_bb_sharing_policy_FATPIPE:
+  switch (AX_surfxml_cluster_bb___sharing___policy) {
+  case A_surfxml_cluster_bb___sharing___policy_FATPIPE:
     cluster.bb_sharing_policy = SURF_LINK_FATPIPE;
     break;
-  case A_surfxml_cluster_bb_sharing_policy_SHARED:
+  case A_surfxml_cluster_bb___sharing___policy_SHARED:
     cluster.bb_sharing_policy = SURF_LINK_SHARED;
     break;
   default:
@@ -377,9 +549,15 @@ void STag_surfxml_cluster(void){
     break;
   }
 
-  cluster.availability_trace = A_surfxml_cluster_availability_file;
-  cluster.state_trace = A_surfxml_cluster_state_file;
+  cluster.availability_trace = A_surfxml_cluster_availability___file;
+  cluster.state_trace = A_surfxml_cluster_state___file;
   sg_platf_new_cluster(&cluster);
+  
+  current_property_set = NULL;
+}
+
+void STag_surfxml_cluster(void){ 
+  xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
 }
 
 void STag_surfxml_cabinet(void){
@@ -388,9 +566,9 @@ void STag_surfxml_cabinet(void){
   cabinet.id = A_surfxml_cabinet_id;
   cabinet.prefix = A_surfxml_cabinet_prefix;
   cabinet.suffix = A_surfxml_cabinet_suffix;
-  cabinet.power = surf_parse_get_double(A_surfxml_cabinet_power);
-  cabinet.bw = surf_parse_get_double(A_surfxml_cabinet_bw);
-  cabinet.lat = surf_parse_get_double(A_surfxml_cabinet_lat);
+  cabinet.power = surf_parse_get_power(A_surfxml_cabinet_power);
+  cabinet.bw = surf_parse_get_bandwidth(A_surfxml_cabinet_bw);
+  cabinet.lat = surf_parse_get_time(A_surfxml_cabinet_lat);
   cabinet.radical = A_surfxml_cabinet_radical;
 
   sg_platf_new_cabinet(&cabinet);
@@ -400,13 +578,13 @@ void STag_surfxml_peer(void){
   s_sg_platf_peer_cbarg_t peer;
   memset(&peer,0,sizeof(peer));
   peer.id = A_surfxml_peer_id;
-  peer.power = surf_parse_get_double(A_surfxml_peer_power);
-  peer.bw_in = surf_parse_get_double(A_surfxml_peer_bw_in);
-  peer.bw_out = surf_parse_get_double(A_surfxml_peer_bw_out);
-  peer.lat = surf_parse_get_double(A_surfxml_peer_lat);
+  peer.power = surf_parse_get_power(A_surfxml_peer_power);
+  peer.bw_in = surf_parse_get_bandwidth(A_surfxml_peer_bw___in);
+  peer.bw_out = surf_parse_get_bandwidth(A_surfxml_peer_bw___out);
+  peer.lat = surf_parse_get_time(A_surfxml_peer_lat);
   peer.coord = A_surfxml_peer_coordinates;
-  peer.availability_trace = tmgr_trace_new_from_file(A_surfxml_peer_availability_file);
-  peer.state_trace = tmgr_trace_new_from_file(A_surfxml_peer_state_file);
+  peer.availability_trace = tmgr_trace_new_from_file(A_surfxml_peer_availability___file);
+  peer.state_trace = tmgr_trace_new_from_file(A_surfxml_peer_state___file);
 
   sg_platf_new_peer(&peer);
 }
@@ -423,10 +601,12 @@ void ETag_surfxml_link(void){
   link.properties = current_property_set;
 
   link.id = A_surfxml_link_id;
-  link.bandwidth = surf_parse_get_double(A_surfxml_link_bandwidth);
-  link.bandwidth_trace = tmgr_trace_new_from_file(A_surfxml_link_bandwidth_file);
-  link.latency = surf_parse_get_double(A_surfxml_link_latency);
-  link.latency_trace = tmgr_trace_new_from_file(A_surfxml_link_latency_file);
+  link.bandwidth = surf_parse_get_bandwidth(A_surfxml_link_bandwidth);
+  //printf("Link bandwidth [%g]\n", link.bandwidth);
+  link.bandwidth_trace = tmgr_trace_new_from_file(A_surfxml_link_bandwidth___file);
+  link.latency = surf_parse_get_time(A_surfxml_link_latency);
+  //printf("Link latency [%g]\n", link.latency);
+  link.latency_trace = tmgr_trace_new_from_file(A_surfxml_link_latency___file);
 
   switch (A_surfxml_link_state) {
   case A_surfxml_link_state_ON:
@@ -439,16 +619,16 @@ void ETag_surfxml_link(void){
     surf_parse_error("invalid state for link %s", link.id);
     break;
   }
-  link.state_trace = tmgr_trace_new_from_file(A_surfxml_link_state_file);
+  link.state_trace = tmgr_trace_new_from_file(A_surfxml_link_state___file);
 
-  switch (A_surfxml_link_sharing_policy) {
-  case A_surfxml_link_sharing_policy_SHARED:
+  switch (A_surfxml_link_sharing___policy) {
+  case A_surfxml_link_sharing___policy_SHARED:
     link.policy = SURF_LINK_SHARED;
     break;
-  case A_surfxml_link_sharing_policy_FATPIPE:
+  case A_surfxml_link_sharing___policy_FATPIPE:
      link.policy = SURF_LINK_FATPIPE;
      break;
-  case A_surfxml_link_sharing_policy_FULLDUPLEX:
+  case A_surfxml_link_sharing___policy_FULLDUPLEX:
      link.policy = SURF_LINK_FULLDUPLEX;
      break;
   default:
@@ -461,19 +641,19 @@ void ETag_surfxml_link(void){
   current_property_set = NULL;
 }
 
-void STag_surfxml_link_ctn(void){
+void STag_surfxml_link___ctn(void){
 
   char *link_id;
-  switch (A_surfxml_link_ctn_direction) {
-  case AU_surfxml_link_ctn_direction:
-  case A_surfxml_link_ctn_direction_NONE:
-    link_id = xbt_strdup(A_surfxml_link_ctn_id);
+  switch (A_surfxml_link___ctn_direction) {
+  case AU_surfxml_link___ctn_direction:
+  case A_surfxml_link___ctn_direction_NONE:
+    link_id = xbt_strdup(A_surfxml_link___ctn_id);
     break;
-  case A_surfxml_link_ctn_direction_UP:
-    link_id = bprintf("%s_UP", A_surfxml_link_ctn_id);
+  case A_surfxml_link___ctn_direction_UP:
+    link_id = bprintf("%s_UP", A_surfxml_link___ctn_id);
     break;
-  case A_surfxml_link_ctn_direction_DOWN:
-    link_id = bprintf("%s_DOWN", A_surfxml_link_ctn_id);
+  case A_surfxml_link___ctn_direction_DOWN:
+    link_id = bprintf("%s_DOWN", A_surfxml_link___ctn_id);
     break;
   }
 
@@ -489,8 +669,8 @@ void ETag_surfxml_backbone(void){
   link.properties = NULL;
 
   link.id = A_surfxml_backbone_id;
-  link.bandwidth = surf_parse_get_double(A_surfxml_backbone_bandwidth);
-  link.latency = surf_parse_get_double(A_surfxml_backbone_latency);
+  link.bandwidth = surf_parse_get_bandwidth(A_surfxml_backbone_bandwidth);
+  link.latency = surf_parse_get_time(A_surfxml_backbone_latency);
   link.state = SURF_RESOURCE_ON;
   link.policy = SURF_LINK_SHARED;
 
@@ -506,27 +686,32 @@ void STag_surfxml_route(void){
 }
 
 void STag_surfxml_ASroute(void){
-  xbt_assert(strlen(A_surfxml_ASroute_src) > 0 || strlen(A_surfxml_ASroute_dst) > 0
-      || strlen(A_surfxml_ASroute_gw_src) > 0 || strlen(A_surfxml_ASroute_gw_dst) > 0,
-      "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
-      A_surfxml_ASroute_src, A_surfxml_ASroute_dst,
-      A_surfxml_ASroute_gw_src,A_surfxml_ASroute_gw_dst);
+  xbt_assert(strlen(A_surfxml_ASroute_src) > 0
+             || strlen(A_surfxml_ASroute_dst) > 0
+             || strlen(A_surfxml_ASroute_gw___src) > 0
+             || strlen(A_surfxml_ASroute_gw___dst) > 0,
+             "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
+             A_surfxml_ASroute_src, A_surfxml_ASroute_dst,
+             A_surfxml_ASroute_gw___src, A_surfxml_ASroute_gw___dst);
   parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
 
 void STag_surfxml_bypassRoute(void){
-  xbt_assert(strlen(A_surfxml_bypassRoute_src) > 0 || strlen(A_surfxml_bypassRoute_dst) > 0,
-      "Missing end-points while defining bupass route \"%s\"->\"%s\"",
-      A_surfxml_bypassRoute_src, A_surfxml_bypassRoute_dst);
+  xbt_assert(strlen(A_surfxml_bypassRoute_src) > 0
+             || strlen(A_surfxml_bypassRoute_dst) > 0,
+             "Missing end-points while defining bupass route \"%s\"->\"%s\"",
+             A_surfxml_bypassRoute_src, A_surfxml_bypassRoute_dst);
   parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
 
 void STag_surfxml_bypassASroute(void){
-  xbt_assert(strlen(A_surfxml_bypassASroute_src) > 0 || strlen(A_surfxml_bypassASroute_dst) > 0
-      || strlen(A_surfxml_bypassASroute_gw_src) > 0 || strlen(A_surfxml_bypassASroute_gw_dst) > 0,
-      "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
-      A_surfxml_bypassASroute_src, A_surfxml_bypassASroute_dst,
-      A_surfxml_bypassASroute_gw_src,A_surfxml_bypassASroute_gw_dst);
+  xbt_assert(strlen(A_surfxml_bypassASroute_src) > 0
+             || strlen(A_surfxml_bypassASroute_dst) > 0
+             || strlen(A_surfxml_bypassASroute_gw___src) > 0
+             || strlen(A_surfxml_bypassASroute_gw___dst) > 0,
+             "Missing end-points while defining route \"%s\"->\"%s\" (with %s and %s as gateways)",
+             A_surfxml_bypassASroute_src, A_surfxml_bypassASroute_dst,
+             A_surfxml_bypassASroute_gw___src,A_surfxml_bypassASroute_gw___dst);
   parsed_link_list = xbt_dynar_new(sizeof(char *), &xbt_free_ref);
 }
 
@@ -561,17 +746,8 @@ void ETag_surfxml_ASroute(void){
   ASroute.src = A_surfxml_ASroute_src;
   ASroute.dst = A_surfxml_ASroute_dst;
 
-  if (!strcmp(current_routing->model_desc->name,"RuleBased")) {
-    // DIRTY PERL HACK AHEAD: with the rulebased routing, the {src,dst}_gateway fields
-    // store the provided name instead of the entity directly (model_rulebased_parse_ASroute knows)
-    //
-    // This is because the user will provide something like "^AS_(.*)$" instead of the proper name of a given entity
-    ASroute.gw_src = (sg_routing_edge_t) A_surfxml_ASroute_gw_src;
-    ASroute.gw_dst = (sg_routing_edge_t) A_surfxml_ASroute_gw_dst;
-  } else {
-    ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw_src);
-    ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw_dst);
-  }
+  ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw___src);
+  ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_ASroute_gw___dst);
 
   ASroute.link_list = parsed_link_list;
 
@@ -613,17 +789,8 @@ void ETag_surfxml_bypassASroute(void){
   ASroute.link_list = parsed_link_list;
   ASroute.symmetrical = FALSE;
 
-  if (!strcmp(current_routing->model_desc->name,"RuleBased")) {
-    // DIRTY PERL HACK AHEAD: with the rulebased routing, the {src,dst}_gateway fields
-    // store the provided name instead of the entity directly (model_rulebased_parse_ASroute knows)
-    //
-    // This is because the user will provide something like "^AS_(.*)$" instead of the proper name of a given entity
-    ASroute.gw_src = (sg_routing_edge_t) A_surfxml_bypassASroute_gw_src;
-    ASroute.gw_dst = (sg_routing_edge_t) A_surfxml_bypassASroute_gw_dst;
-  } else {
-    ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw_src);
-    ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw_dst);
-  }
+  ASroute.gw_src = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw___src);
+  ASroute.gw_dst = sg_routing_edge_by_name_or_null(A_surfxml_bypassASroute_gw___dst);
 
   sg_platf_new_bypassASroute(&ASroute);
   parsed_link_list = NULL;
@@ -641,32 +808,32 @@ void ETag_surfxml_trace(void){
   sg_platf_new_trace(&trace);
 }
 
-void STag_surfxml_trace_connect(void){
+void STag_surfxml_trace___connect(void){
   s_sg_platf_trace_connect_cbarg_t trace_connect;
   memset(&trace_connect,0,sizeof(trace_connect));
 
-  trace_connect.element = A_surfxml_trace_connect_element;
-  trace_connect.trace = A_surfxml_trace_connect_trace;
+  trace_connect.element = A_surfxml_trace___connect_element;
+  trace_connect.trace = A_surfxml_trace___connect_trace;
 
-  switch (A_surfxml_trace_connect_kind) {
-  case AU_surfxml_trace_connect_kind:
-  case A_surfxml_trace_connect_kind_POWER:
+  switch (A_surfxml_trace___connect_kind) {
+  case AU_surfxml_trace___connect_kind:
+  case A_surfxml_trace___connect_kind_POWER:
     trace_connect.kind =  SURF_TRACE_CONNECT_KIND_POWER;
     break;
-  case A_surfxml_trace_connect_kind_BANDWIDTH:
+  case A_surfxml_trace___connect_kind_BANDWIDTH:
     trace_connect.kind =  SURF_TRACE_CONNECT_KIND_BANDWIDTH;
     break;
-  case A_surfxml_trace_connect_kind_HOST_AVAIL:
+  case A_surfxml_trace___connect_kind_HOST___AVAIL:
     trace_connect.kind =  SURF_TRACE_CONNECT_KIND_HOST_AVAIL;
     break;
-  case A_surfxml_trace_connect_kind_LATENCY:
+  case A_surfxml_trace___connect_kind_LATENCY:
     trace_connect.kind =  SURF_TRACE_CONNECT_KIND_LATENCY;
     break;
-  case A_surfxml_trace_connect_kind_LINK_AVAIL:
+  case A_surfxml_trace___connect_kind_LINK___AVAIL:
     trace_connect.kind =  SURF_TRACE_CONNECT_KIND_LINK_AVAIL;
     break;
   }
-  sg_platf_new_trace_connect(&trace_connect);
+  sg_platf_trace_connect(&trace_connect);
 }
 
 void STag_surfxml_AS(void){
@@ -694,13 +861,11 @@ void ETag_surfxml_AS(void){
   sg_platf_new_AS_end();
 }
 
-extern int _surf_init_status; /* FIXME: find a proper way to export this at some point */
-
 void STag_surfxml_config(void){
   AS_TAG = 0;
   xbt_assert(current_property_set == NULL, "Someone forgot to reset the property set to NULL in its closing tag (or XML malformed)");
   XBT_DEBUG("START configuration name = %s",A_surfxml_config_id);
-  if (_surf_init_status == 2) {
+  if (_sg_cfg_init_status == 2) {
     surf_parse_error("All <config> tags must be given before any platform elements (such as <AS>, <host>, <cluster>, <link>, etc).");
   }
 }
@@ -711,8 +876,8 @@ void ETag_surfxml_config(void){
   char *cfg;
   xbt_dict_foreach(current_property_set, cursor, key, elem) {
     cfg = bprintf("%s:%s",key,elem);
-    if(xbt_cfg_is_default_value(_surf_cfg_set, key))
-      xbt_cfg_set_parse(_surf_cfg_set, cfg);
+    if(xbt_cfg_is_default_value(_sg_cfg_set, key))
+      xbt_cfg_set_parse(_sg_cfg_set, cfg);
     else
       XBT_INFO("The custom configuration '%s' is already defined by user!",key);
     free(cfg);
@@ -742,15 +907,15 @@ void ETag_surfxml_process(void){
   process.properties = current_property_set;
   process.host = A_surfxml_process_host;
   process.function = A_surfxml_process_function;
-  process.start_time = surf_parse_get_double(A_surfxml_process_start_time);
-  process.kill_time = surf_parse_get_double(A_surfxml_process_kill_time);
+  process.start_time = surf_parse_get_double(A_surfxml_process_start___time);
+  process.kill_time = surf_parse_get_double(A_surfxml_process_kill___time);
 
-  switch (A_surfxml_process_on_failure) {
-  case AU_surfxml_process_on_failure:
-  case A_surfxml_process_on_failure_DIE:
+  switch (A_surfxml_process_on___failure) {
+  case AU_surfxml_process_on___failure:
+  case A_surfxml_process_on___failure_DIE:
     process.on_failure =  SURF_PROCESS_ON_FAILURE_DIE;
     break;
-  case A_surfxml_process_on_failure_RESTART:
+  case A_surfxml_process_on___failure_RESTART:
     process.on_failure =  SURF_PROCESS_ON_FAILURE_RESTART;
     break;
   }
@@ -786,15 +951,14 @@ void ETag_surfxml_gpu(void)
 void ETag_surfxml_prop(void){}
 void STag_surfxml_random(void){}
 void ETag_surfxml_random(void){}
-void ETag_surfxml_trace_connect(void){}
+void ETag_surfxml_trace___connect(void){}
 void STag_surfxml_trace(void){}
 void ETag_surfxml_router(void){}
-void ETag_surfxml_host_link(void){}
-void ETag_surfxml_cluster(void){}
+void ETag_surfxml_host___link(void){}
 void ETag_surfxml_cabinet(void){}
 void ETag_surfxml_peer(void){}
 void STag_surfxml_backbone(void){}
-void ETag_surfxml_link_ctn(void){}
+void ETag_surfxml_link___ctn(void){}
 void ETag_surfxml_argument(void){}
 
 /* Open and Close parse file */
@@ -835,6 +999,7 @@ void surf_parse_close(void)
   xbt_dynar_free(&surf_parsed_filename_stack);
 
   free(surf_parsed_filename);
+  surf_parsed_filename = NULL;
 
   if (surf_file_to_parse) {
     surf_parse__delete_buffer(surf_input_buffer);
@@ -877,7 +1042,7 @@ double get_cpu_power(const char *power)
       power_scale = random_generate(random);
     }
   } else {
-    power_scale = surf_parse_get_double(power);
+    power_scale = surf_parse_get_power(power);
   }
   return power_scale;
 }
@@ -892,7 +1057,7 @@ static void init_randomness(void)
   random_min = surf_parse_get_double(A_surfxml_random_min);
   random_max = surf_parse_get_double(A_surfxml_random_max);
   random_mean = surf_parse_get_double(A_surfxml_random_mean);
-  random_std_deviation = surf_parse_get_double(A_surfxml_random_std_deviation);
+  random_std_deviation = surf_parse_get_double(A_surfxml_random_std___deviation);
   switch (A_surfxml_random_generator) {
   case AU_surfxml_random_generator:
   case A_surfxml_random_generator_NONE:
@@ -915,7 +1080,7 @@ static void init_randomness(void)
 
 static void add_randomness(void)
 {
-  /* If needed aditional properties can be added by using the prop tag */
+  /* If needed, additional properties can be added by using the prop tag */
   random_data_t random =
       random_new(random_generator, 0, random_min, random_max, random_mean,
                  random_std_deviation);
