@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2007, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2005, 2007, 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -269,7 +269,7 @@ tmgr_trace_t tmgr_trace_new_from_string(const char *id, const char *input,
   }
 
   xbt_assert(periodicity >= 0,
-              "Invalid periodicity %lg (must be positive)", periodicity);
+              "Invalid periodicity %g (must be positive)", periodicity);
 
   trace = xbt_new0(s_tmgr_trace_t, 1);
   trace->type = e_trace_list;
@@ -292,10 +292,17 @@ tmgr_trace_t tmgr_trace_new_from_string(const char *id, const char *input,
     if (last_event) {
       if (last_event->delta > event.delta) {
         xbt_die("%s:%d: Invalid trace: Events must be sorted, "
-                "but time %lg > time %lg.\n%s",
+                "but time %g > time %g.\n%s",
                 id, linecount, last_event->delta, event.delta, input);
       }
       last_event->delta = event.delta - last_event->delta;
+    } else {
+      if(event.delta > 0.0){
+        s_tmgr_event_t first_event;
+        first_event.delta=event.delta;
+        first_event.value=-1.0;
+        xbt_dynar_push(trace->s_list.event_list, &first_event);
+      }
     }
     xbt_dynar_push(trace->s_list.event_list, &event);
     last_event =
@@ -437,7 +444,7 @@ tmgr_trace_event_t tmgr_history_get_next_event_leq(tmgr_history_t h,
         trace_event->idx++;
       } else if (event->delta > 0) {        /* Last element, checking for periodicity */
         xbt_heap_push(h->heap, trace_event, event_date + event->delta);
-        trace_event->idx = 0;
+        trace_event->idx = 1; /* not 0 as the first event is a placeholder to handle when events really start */
       } else {                      /* We don't need this trace_event anymore */
         trace_event->free_me = 1;
       }
