@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -10,10 +10,11 @@
 #include "surf_private.h"
 #include "storage_private.h"
 #include "surf/surf_resource.h"
+#include "simgrid/sg_config.h"
+#include <inttypes.h>
 
 typedef struct workstation_CLM03 {
   s_surf_resource_t generic_resource;   /* Must remain first to add this to a trace */
-  void *cpu;
   void *net_elm;
   xbt_dynar_t storage;
 } s_workstation_CLM03_t, *workstation_CLM03_t;
@@ -29,7 +30,6 @@ static void workstation_new(sg_platf_host_cbarg_t host)
 
   workstation->generic_resource.model = surf_workstation_model;
   workstation->generic_resource.name = xbt_strdup(host->id);
-  workstation->cpu = xbt_lib_get_or_null(host_lib, host->id, SURF_CPU_LEVEL);
   workstation->storage = xbt_lib_get_or_null(storage_lib,host->id,ROUTING_STORAGE_HOST_LEVEL);
   workstation->net_elm = xbt_lib_get_or_null(host_lib,host->id,ROUTING_HOST_LEVEL);
   XBT_DEBUG("Create workstation %s with %ld mounted disks",host->id,xbt_dynar_length(workstation->storage));
@@ -112,14 +112,14 @@ static void ws_update_resource_state(void *id,
 
 static surf_action_t ws_execute(void *workstation, double size)
 {
-  void *cpu = ((workstation_CLM03_t) workstation)->cpu;
-  return ((surf_resource_t) cpu)->model->extension.cpu.execute(cpu, size);
+  surf_resource_t cpu = ((surf_resource_t) surf_cpu_resource_priv(workstation));
+  return cpu->model->extension.cpu.execute(workstation, size);
 }
 
 static surf_action_t ws_action_sleep(void *workstation, double duration)
 {
   return surf_cpu_model->extension.cpu.
-      sleep(((workstation_CLM03_t) workstation)->cpu, duration);
+      sleep(workstation, duration);
 }
 
 static void ws_action_suspend(surf_action_t action)
@@ -209,8 +209,8 @@ static surf_action_t ws_communicate(void *workstation_src,
                                     void *workstation_dst, double size,
                                     double rate)
 {
-  workstation_CLM03_t src = (workstation_CLM03_t) workstation_src;
-  workstation_CLM03_t dst = (workstation_CLM03_t) workstation_dst;
+  workstation_CLM03_t src = surf_workstation_resource_priv(workstation_src);
+  workstation_CLM03_t dst = surf_workstation_resource_priv(workstation_dst);
   return surf_network_model->extension.network.
       communicate(src->net_elm,
                   dst->net_elm, size, rate);
@@ -219,28 +219,90 @@ static surf_action_t ws_communicate(void *workstation_src,
 static e_surf_resource_state_t ws_get_state(void *workstation)
 {
   return surf_cpu_model->extension.cpu.
-      get_state(((workstation_CLM03_t) workstation)->cpu);
+      get_state(workstation);
 }
 
 static double ws_get_speed(void *workstation, double load)
 {
   return surf_cpu_model->extension.cpu.
-      get_speed(((workstation_CLM03_t) workstation)->cpu, load);
+      get_speed(workstation, load);
 }
 
+static int ws_get_core(void *workstation)
+{
+  return surf_cpu_model->extension.cpu.
+      get_core(workstation);
+}
+
+
+
 static double ws_get_available_speed(void *workstation)
 {
   return surf_cpu_model->extension.cpu.
-      get_available_speed(((workstation_CLM03_t)
-                           workstation)->cpu);
+      get_available_speed(workstation);
 }
 
+static double ws_get_current_power_peak(void *workstation)
+{
+  return surf_cpu_model->extension.cpu.
+      get_current_power_peak(workstation);
+}
+
+static double ws_get_power_peak_at(void *workstation, int pstate_index)
+{
+  return surf_cpu_model->extension.cpu.
+      get_power_peak_at(workstation, pstate_index);
+}
+
+static int ws_get_nb_pstates(void *workstation)
+{
+  return surf_cpu_model->extension.cpu.
+      get_nb_pstates(workstation);
+}
+
+static void ws_set_power_peak_at(void *workstation, int pstate_index)
+{
+  surf_cpu_model->extension.cpu.
+      set_power_peak_at(workstation, pstate_index);
+}
+
+static double ws_get_consumed_energy(void *workstation)
+{
+  return surf_cpu_model->extension.cpu.
+      get_consumed_energy(workstation);
+}
+
+
 static surf_action_t ws_execute_parallel_task(int workstation_nb,
                                               void **workstation_list,
                                               double *computation_amount,
                                               double *communication_amount,
                                               double rate)
 {
+#define cost_or_zero(array,pos) ((array)?(array)[pos]:0.0)
+  if ((workstation_nb == 1)
+      && (cost_or_zero(communication_amount, 0) == 0.0))
+    return ws_execute(workstation_list[0], computation_amount[0]);
+  else if ((workstation_nb == 1)
+           && (cost_or_zero(computation_amount, 0) == 0.0))
+    return ws_communicate(workstation_list[0], workstation_list[0],communication_amount[0], rate);
+  else if ((workstation_nb == 2)
+             && (cost_or_zero(computation_amount, 0) == 0.0)
+             && (cost_or_zero(computation_amount, 1) == 0.0)) {
+    int i,nb = 0;
+    double value = 0.0;
+
+    for (i = 0; i < workstation_nb * workstation_nb; i++) {
+      if (cost_or_zero(communication_amount, i) > 0.0) {
+        nb++;
+        value = cost_or_zero(communication_amount, i);
+      }
+    }
+    if (nb == 1)
+      return ws_communicate(workstation_list[0], workstation_list[1],value, rate);
+  }
+#undef cost_or_zero
+
   THROW_UNIMPLEMENTED;          /* This model does not implement parallel tasks */
   return NULL;
 }
@@ -250,8 +312,8 @@ static surf_action_t ws_execute_parallel_task(int workstation_nb,
 static xbt_dynar_t ws_get_route(void *workstation_src, void *workstation_dst)
 {
   XBT_DEBUG("ws_get_route");
-  workstation_CLM03_t src = (workstation_CLM03_t) workstation_src;
-  workstation_CLM03_t dst = (workstation_CLM03_t) workstation_dst;
+  workstation_CLM03_t src = surf_workstation_resource_priv(workstation_src);
+  workstation_CLM03_t dst = surf_workstation_resource_priv(workstation_dst);
   return surf_network_model->extension.
       network.get_route(src->net_elm,
                   dst->net_elm);
@@ -280,78 +342,111 @@ static void ws_finalize(void)
 
 static xbt_dict_t ws_get_properties(const void *ws)
 {
-  return surf_resource_properties(((workstation_CLM03_t) ws)->cpu);
+  return surf_resource_properties(surf_cpu_resource_priv(ws));
 }
 
-static storage_t find_storage_on_mount_list(void *workstation,const char* storage)
+static storage_t find_storage_on_mount_list(void *workstation,const char* mount)
 {
   storage_t st = NULL;
   s_mount_t mnt;
   unsigned int cursor;
-  xbt_dynar_t storage_list = ((workstation_CLM03_t) workstation)->storage;
+  workstation_CLM03_t ws = (workstation_CLM03_t) surf_workstation_resource_priv(workstation);
+  xbt_dynar_t storage_list = ws->storage;
 
-  XBT_DEBUG("Search for storage name '%s' on '%s'",storage,((workstation_CLM03_t) workstation)->generic_resource.name);
+  XBT_DEBUG("Search for storage name '%s' on '%s'",mount,ws->generic_resource.name);
   xbt_dynar_foreach(storage_list,cursor,mnt)
   {
     XBT_DEBUG("See '%s'",mnt.name);
-    if(!strcmp(storage,mnt.name)){
-      st = mnt.id;
+    if(!strcmp(mount,mnt.name)){
+      st = mnt.storage;
       break;
     }
   }
-  if(!st) xbt_die("Can't find mount '%s' for '%s'",storage,((workstation_CLM03_t) workstation)->generic_resource.name);
+  if(!st) xbt_die("Can't find mount '%s' for '%s'",mount,ws->generic_resource.name);
   return st;
 }
 
-static surf_action_t ws_action_open(void *workstation, const char* mount, const char* path, const char* mode)
+static xbt_dict_t ws_get_storage_list(void *workstation)
+{
+  s_mount_t mnt;
+  unsigned int i;
+  xbt_dict_t storage_list = xbt_dict_new_homogeneous(NULL);
+  char *storage_name = NULL;
+
+  workstation_CLM03_t ws = (workstation_CLM03_t) surf_workstation_resource_priv(workstation);
+  xbt_dynar_t storages = ws->storage;
+
+  xbt_dynar_foreach(storages,i,mnt){
+    storage_name = ((storage_t)mnt.storage)->generic_resource.name;
+    xbt_dict_set(storage_list,mnt.name,storage_name,NULL);
+  }
+  return storage_list;
+}
+
+static surf_action_t ws_action_open(void *workstation, const char* mount,
+                                    const char* path)
 {
   storage_t st = find_storage_on_mount_list(workstation, mount);
   XBT_DEBUG("OPEN on disk '%s'",st->generic_resource.name);
   surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.open(st, mount, path, mode);
+  return model->extension.storage.open(st, mount, path);
 }
 
-static surf_action_t ws_action_close(void *workstation, surf_file_t fp)
+static surf_action_t ws_action_close(void *workstation, surf_file_t fd)
 {
-  storage_t st = find_storage_on_mount_list(workstation, fp->storage);
+  storage_t st = find_storage_on_mount_list(workstation, fd->mount);
   XBT_DEBUG("CLOSE on disk '%s'",st->generic_resource.name);
   surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.close(st, fp);
+  return model->extension.storage.close(st, fd);
 }
 
-static surf_action_t ws_action_read(void *workstation, void* ptr, size_t size, size_t nmemb, surf_file_t stream)
+static surf_action_t ws_action_read(void *workstation, surf_file_t fd, sg_storage_size_t size)
 {
-  storage_t st = find_storage_on_mount_list(workstation, stream->storage);
+  storage_t st = find_storage_on_mount_list(workstation, fd->mount);
   XBT_DEBUG("READ on disk '%s'",st->generic_resource.name);
   surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.read(st, ptr, (double)size, nmemb, stream);
+  return model->extension.storage.read(st, fd, size);
 }
 
-static surf_action_t ws_action_write(void *workstation, const void* ptr, size_t size, size_t nmemb, surf_file_t stream)
+static surf_action_t ws_action_write(void *workstation, surf_file_t fd, sg_storage_size_t size)
 {
-  storage_t st = find_storage_on_mount_list(workstation, stream->storage);
+  storage_t st = find_storage_on_mount_list(workstation, fd->mount);
   XBT_DEBUG("WRITE on disk '%s'",st->generic_resource.name);
   surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.write(st,  ptr, size, nmemb, stream);
+  return model->extension.storage.write(st, fd, size);
 }
 
-static surf_action_t ws_action_stat(void *workstation, surf_file_t stream)
+static int ws_file_unlink(void *workstation, surf_file_t fd)
 {
-  storage_t st = find_storage_on_mount_list(workstation, stream->storage);
-  XBT_DEBUG("STAT on disk '%s'",st->generic_resource.name);
-  surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.stat(st,  stream);
+  if (!fd){
+    XBT_WARN("No such file descriptor. Impossible to unlink");
+    return 0;
+  } else {
+//    XBT_INFO("%s %zu", fd->storage, fd->size);
+    storage_t st = find_storage_on_mount_list(workstation, fd->mount);
+    xbt_dict_t content_dict = (st)->content;
+    /* Check if the file is on this storage */
+    if (!xbt_dict_get_or_null(content_dict, fd->name)){
+      XBT_WARN("File %s is not on disk %s. Impossible to unlink", fd->name,
+          st->generic_resource.name);
+      return 0;
+    } else {
+      XBT_DEBUG("UNLINK on disk '%s'",st->generic_resource.name);
+      st->used_size -= fd->size;
+
+      // Remove the file from storage
+      xbt_dict_remove(content_dict,fd->name);
+
+      free(fd->name);
+      free(fd->mount);
+      xbt_free(fd);
+      return 1;
+    }
+  }
 }
 
-static surf_action_t ws_action_unlink(void *workstation, surf_file_t stream)
-{
-  storage_t st = find_storage_on_mount_list(workstation, stream->storage);
-  XBT_DEBUG("UNLINK on disk '%s'",st->generic_resource.name);
-  surf_model_t model = st->generic_resource.model;
-  return model->extension.storage.unlink(st,  stream);
-}
-
-static surf_action_t ws_action_ls(void *workstation, const char* mount, const char *path)
+static surf_action_t ws_action_ls(void *workstation, const char* mount,
+                                  const char *path)
 {
   XBT_DEBUG("LS on mount '%s' and file '%s'",mount, path);
   storage_t st = find_storage_on_mount_list(workstation, mount);
@@ -359,6 +454,38 @@ static surf_action_t ws_action_ls(void *workstation, const char* mount, const ch
   return model->extension.storage.ls(st, path);
 }
 
+static sg_storage_size_t ws_file_get_size(void *workstation, surf_file_t fd)
+{
+  return fd->size;
+}
+
+static xbt_dynar_t ws_file_get_info(void *workstation, surf_file_t fd)
+{
+  storage_t st = find_storage_on_mount_list(workstation, fd->mount);
+  sg_storage_size_t *psize = xbt_new(sg_storage_size_t, 1);
+  *psize = fd->size;
+  xbt_dynar_t info = xbt_dynar_new(sizeof(void*), NULL);
+  xbt_dynar_push_as(info, sg_storage_size_t *, psize);
+  xbt_dynar_push_as(info, void *, fd->mount);
+  xbt_dynar_push_as(info, void *, st->generic_resource.name);
+  xbt_dynar_push_as(info, void *, st->type_id);
+  xbt_dynar_push_as(info, void *, st->content_type);
+
+  return info;
+}
+
+static sg_storage_size_t ws_storage_get_free_size(void *workstation,const char* name)
+{
+  storage_t st = find_storage_on_mount_list(workstation, name);
+  return st->size - st->used_size;
+}
+
+static sg_storage_size_t ws_storage_get_used_size(void *workstation,const char* name)
+{
+  storage_t st = find_storage_on_mount_list(workstation, name);
+  return st->used_size;
+}
+
 static void surf_workstation_model_init_internal(void)
 {
   surf_workstation_model = surf_model_init();
@@ -393,9 +520,15 @@ static void surf_workstation_model_init_internal(void)
   surf_workstation_model->extension.workstation.execute = ws_execute;
   surf_workstation_model->extension.workstation.sleep = ws_action_sleep;
   surf_workstation_model->extension.workstation.get_state = ws_get_state;
+  surf_workstation_model->extension.workstation.get_core = ws_get_core;
   surf_workstation_model->extension.workstation.get_speed = ws_get_speed;
   surf_workstation_model->extension.workstation.get_available_speed =
       ws_get_available_speed;
+  surf_workstation_model->extension.workstation.get_current_power_peak = ws_get_current_power_peak;
+  surf_workstation_model->extension.workstation.get_power_peak_at = ws_get_power_peak_at;
+  surf_workstation_model->extension.workstation.get_nb_pstates = ws_get_nb_pstates;
+  surf_workstation_model->extension.workstation.set_power_peak_at = ws_set_power_peak_at;
+  surf_workstation_model->extension.workstation.get_consumed_energy = ws_get_consumed_energy;
 
   surf_workstation_model->extension.workstation.communicate =
       ws_communicate;
@@ -415,21 +548,24 @@ static void surf_workstation_model_init_internal(void)
   surf_workstation_model->extension.workstation.close = ws_action_close;
   surf_workstation_model->extension.workstation.read = ws_action_read;
   surf_workstation_model->extension.workstation.write = ws_action_write;
-  surf_workstation_model->extension.workstation.stat = ws_action_stat;
-  surf_workstation_model->extension.workstation.unlink = ws_action_unlink;
+  surf_workstation_model->extension.workstation.unlink = ws_file_unlink;
   surf_workstation_model->extension.workstation.ls = ws_action_ls;
+  surf_workstation_model->extension.workstation.get_size = ws_file_get_size;
+  surf_workstation_model->extension.workstation.get_info = ws_file_get_info;
+  surf_workstation_model->extension.workstation.get_free_size = ws_storage_get_free_size;
+  surf_workstation_model->extension.workstation.get_used_size = ws_storage_get_used_size;
+  surf_workstation_model->extension.workstation.get_storage_list = ws_get_storage_list;
 }
 
 void surf_workstation_model_init_current_default(void)
 {
   surf_workstation_model_init_internal();
-  xbt_cfg_setdefault_int(_surf_cfg_set, "network/crosstraffic", 1);
+  xbt_cfg_setdefault_boolean(_sg_cfg_set, "network/crosstraffic", "yes");
   surf_cpu_model_init_Cas01();
   surf_network_model_init_LegrandVelho();
 
   xbt_dynar_push(model_list, &surf_workstation_model);
   sg_platf_host_add_cb(workstation_new);
-//  sg_platf_postparse_add_cb(create_workstations);
 }
 
 void surf_workstation_model_init_compound()
@@ -440,5 +576,4 @@ void surf_workstation_model_init_compound()
   surf_workstation_model_init_internal();
   xbt_dynar_push(model_list, &surf_workstation_model);
   sg_platf_host_add_cb(workstation_new);
-//  sg_platf_postparse_add_cb(create_workstations);
 }
