@@ -1,7 +1,7 @@
 /* backtrace_linux - backtrace displaying on linux platform                 */
 /* This file is included by ex.c on need (have execinfo.h, popen & addrline)*/
 
-/* Copyright (c) 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2008-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -12,6 +12,9 @@
 #include "xbt/str.h"
 #include "xbt/module.h"         /* xbt_binary_name */
 #include "xbt_modinter.h"       /* backtrace initialization headers */
+#ifdef HAVE_MC
+#include <libunwind.h>
+#endif
 /* end of "useless" inclusions */
 
 extern char **environ;          /* the environment, as specified by the opengroup */
@@ -80,7 +83,8 @@ void xbt_backtrace_current(xbt_ex_t * e)
   e->used = backtrace((void **) e->bt, XBT_BACKTRACE_SIZE);
   if (e->used == 0) {
     fprintf(stderr, "The backtrace() function failed, which probably means that the memory is exhausted. Here is a crude dump of the exception that I was trying to build:");
-    fprintf(stderr, "%s:%s(%d) [%s:%d] %s", e->host,e->procname,e->pid, e->file,e->line,e->msg);
+    fprintf(stderr, "%s(%d) [%s:%d] %s",
+            e->procname, e->pid, e->file, e->line, e->msg);
     fprintf(stderr, "Bailing out now since there is nothing I can do without a decent amount of memory. Please go fix the memleaks\n");
     exit(1);
   }
@@ -116,7 +120,7 @@ void xbt_ex_setup_backtrace(xbt_ex_t * e) //FIXME: This code could be greatly im
 
   e->bt_strings = NULL;
 
-  if (!xbt_binary_name) /* no binary name, nothing to do */
+  if (xbt_binary_name == NULL) /* no binary name, nothing to do */
     return;
 
   if (e->used <= 1)
@@ -364,3 +368,37 @@ void xbt_ex_setup_backtrace(xbt_ex_t * e) //FIXME: This code could be greatly im
   free(backtrace_syms);
   free(cmd);
 }
+
+#ifdef HAVE_MC
+
+int xbt_libunwind_backtrace(void* bt[XBT_BACKTRACE_SIZE], int size){
+  
+  int i = 0;
+  for(i=0; i < size; i++)
+    bt[i] = NULL;
+
+  i=0;
+
+  unw_cursor_t c;
+  unw_context_t uc;
+
+  unw_getcontext (&uc);
+	unw_init_local (&c, &uc);
+ 
+  unw_word_t ip;
+
+  unw_step(&c);
+ 
+  while(unw_step(&c) >= 0 && i < size){
+
+    unw_get_reg(&c, UNW_REG_IP, &ip);
+    bt[i] = (void*)(long)ip;
+    i++;
+
+  }
+
+  return i;
+  
+}
+
+#endif
