@@ -2,7 +2,7 @@
 
 /* This is useful to build named structs, like option or property sets.     */
 
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -45,7 +45,15 @@ typedef struct {
 } s_xbt_cfgelm_t, *xbt_cfgelm_t;
 
 static const char *xbt_cfgelm_type_name[xbt_cfgelm_type_count] =
-    { "int", "double", "string", "peer", "any" };
+    { "int", "double", "string", "boolean", "peer", "any" };
+
+const struct xbt_boolean_couple xbt_cfgelm_boolean_values[] = {
+  { "yes",    "no"},
+  {  "on",   "off"},
+  {"true", "false"},
+  {   "1",     "0"},
+  {  NULL,    NULL}
+};
 
 /* Internal stuff used in cache to free a variable */
 static void xbt_cfgelm_free(void *data);
@@ -58,7 +66,7 @@ static xbt_cfgelm_t xbt_cfgelm_get(xbt_cfg_t cfg, const char *name,
 
 /** @brief Constructor
  *
- * Initialise an config set
+ * Initialise a config set
  */
 
 
@@ -69,8 +77,8 @@ xbt_cfg_t xbt_cfg_new(void)
 
 /** \brief Copy an existing configuration set
  *
- * \arg whereto the config set to be created
- * \arg tocopy the source data
+ * @param whereto the config set to be created
+ * @param tocopy the source data
  *
  * This only copy the registrations, not the actual content
  */
@@ -86,7 +94,7 @@ void xbt_cfg_cpy(xbt_cfg_t tocopy, xbt_cfg_t * whereto)
   xbt_assert(tocopy, "cannot copy NULL config");
 
   xbt_dict_foreach((xbt_dict_t) tocopy, cursor, name, variable) {
-    xbt_cfg_register(whereto, name, variable->desc, variable->type, NULL,
+    xbt_cfg_register(whereto, name, variable->desc, variable->type,
                      variable->min, variable->max, variable->cb_set,
                      variable->cb_rm);
   }
@@ -101,9 +109,9 @@ void xbt_cfg_free(xbt_cfg_t * cfg)
 
 /** @brief Dump a config set for debuging purpose
  *
- * \arg name The name to give to this config set
- * \arg indent what to write at the begining of each line (right number of spaces)
- * \arg cfg the config set
+ * @param name The name to give to this config set
+ * @param indent what to write at the begining of each line (right number of spaces)
+ * @param cfg the config set
  */
 void xbt_cfg_dump(const char *name, const char *indent, xbt_cfg_t cfg)
 {
@@ -154,6 +162,13 @@ void xbt_cfg_dump(const char *name, const char *indent, xbt_cfg_t cfg)
       }
       break;
 
+    case xbt_cfgelm_boolean:
+      for (i = 0; i < size; i++) {
+        ival = xbt_dynar_get_as(variable->content, i, int);
+        printf("%s    %d\n", indent, ival);
+      }
+      break;
+
     case xbt_cfgelm_peer:
       for (i = 0; i < size; i++) {
         hval = xbt_dynar_get_as(variable->content, i, xbt_peer_t);
@@ -195,16 +210,20 @@ void xbt_cfgelm_free(void *data)
 
 /** @brief Register an element within a config set
  *
- *  @arg cfg the config set
- *  @arg type the type of the config element
- *  @arg min the minimum
- *  @arg max the maximum
+ *  @param cfg the config set
+ *  @param name the name of the config element
+ *  @param desc a description for this item (used by xbt_cfg_help())
+ *  @param type the type of the config element
+ *  @param min the minimum number of values for this config element
+ *  @param max the maximum number of values for this config element
+ *  @param cb_set callback function called when a value is set
+ *  @param cb_rm callback function called when a value is removed
  */
 
 void
 xbt_cfg_register(xbt_cfg_t * cfg,
                  const char *name, const char *desc,
-                 e_xbt_cfgelm_type_t type, void *default_value, int min,
+                 e_xbt_cfgelm_type_t type, int min,
                  int max, xbt_cfg_cb_t cb_set, xbt_cfg_cb_t cb_rm)
 {
   xbt_cfgelm_t res;
@@ -237,26 +256,22 @@ xbt_cfg_register(xbt_cfg_t * cfg,
   switch (type) {
   case xbt_cfgelm_int:
     res->content = xbt_dynar_new(sizeof(int), NULL);
-    if (default_value)
-      xbt_dynar_push(res->content, default_value);
     break;
 
   case xbt_cfgelm_double:
     res->content = xbt_dynar_new(sizeof(double), NULL);
-    if (default_value)
-      xbt_dynar_push(res->content, default_value);
     break;
 
   case xbt_cfgelm_string:
     res->content = xbt_dynar_new(sizeof(char *), xbt_free_ref);
-    if (default_value)
-      xbt_dynar_push(res->content, default_value);
+    break;
+
+  case xbt_cfgelm_boolean:
+    res->content = xbt_dynar_new(sizeof(int), NULL);
     break;
 
   case xbt_cfgelm_peer:
     res->content = xbt_dynar_new(sizeof(xbt_peer_t), xbt_peer_free_voidp);
-    if (default_value)
-      xbt_dynar_push(res->content, default_value);
     break;
 
   default:
@@ -268,8 +283,8 @@ xbt_cfg_register(xbt_cfg_t * cfg,
 
 /** @brief Unregister an element from a config set.
  *
- *  @arg cfg the config set
- *  @arg name the name of the elem to be freed
+ *  @param cfg the config set
+ *  @param name the name of the elem to be freed
  *
  *  Note that it removes both the description and the actual content.
  *  Throws not_found when no such element exists.
@@ -284,8 +299,8 @@ void xbt_cfg_unregister(xbt_cfg_t cfg, const char *name)
 /**
  * @brief Parse a string and register the stuff described.
  *
- * @arg cfg the config set
- * @arg entry a string describing the element to register
+ * @param cfg the config set
+ * @param entry a string describing the element to register
  *
  * The string may consist in several variable descriptions separated by a space.
  * Each of them must use the following syntax: \<name\>:\<min nb\>_to_\<max nb\>_\<type\>
@@ -333,7 +348,7 @@ void xbt_cfg_register_str(xbt_cfg_t * cfg, const char *entry)
               "Invalid type in config element descriptor: %s%s", entry,
               "; Should be one of 'string', 'int', 'peer' or 'double'.");
 
-  xbt_cfg_register(cfg, entrycpy, NULL, type, NULL, min, max, NULL, NULL);
+  xbt_cfg_register(cfg, entrycpy, NULL, type, min, max, NULL, NULL);
 
   free(entrycpy);               /* strdup'ed by dict mechanism, but cannot be const */
 }
@@ -370,48 +385,48 @@ void xbt_cfg_help(xbt_cfg_t cfg)
       else
         printf("Arity: min:%d to max:%d; ", variable->min, variable->max);
     }
-    printf("Current value: ");
     size = xbt_dynar_length(variable->content);
+    printf("Current value%s: ", (size <= 1 ? "" : "s"));
 
-    switch (variable->type) {
-      int ival;
-      char *sval;
-      double dval;
-      xbt_peer_t hval;
+    if (size != 1)
+      printf(size == 0 ? "n/a\n" : "{ ");
+    for (i = 0; i < size; i++) {
+      const char *sep = (size == 1 ? "\n" : (i < size - 1 ? ", " : " }\n"));
 
-    case xbt_cfgelm_int:
-      for (i = 0; i < size; i++) {
-        ival = xbt_dynar_get_as(variable->content, i, int);
-        printf("%s%d\n", (i == 0 ? "" : "              "), ival);
+      switch (variable->type) {
+      case xbt_cfgelm_int:
+        printf("%d%s", xbt_dynar_get_as(variable->content, i, int), sep);
+        break;
+
+      case xbt_cfgelm_double:
+        printf("%f%s", xbt_dynar_get_as(variable->content, i, double), sep);
+        break;
+
+      case xbt_cfgelm_string:
+        printf("'%s'%s", xbt_dynar_get_as(variable->content, i, char *), sep);
+        break;
+
+      case xbt_cfgelm_boolean: {
+        int b = xbt_dynar_get_as(variable->content, i, int);
+        const char *bs = b ? xbt_cfgelm_boolean_values[0].true_val
+                           : xbt_cfgelm_boolean_values[0].false_val;
+        if (b == 0 || b == 1)
+          printf("'%s'%s", bs, sep);
+        else
+          printf("'%s/%d'%s", bs, b, sep);
+        break;
       }
-      break;
 
-    case xbt_cfgelm_double:
-      for (i = 0; i < size; i++) {
-        dval = xbt_dynar_get_as(variable->content, i, double);
-        printf("%s%f\n", (i == 0 ? "" : "              "), dval);
+      case xbt_cfgelm_peer: {
+        xbt_peer_t hval = xbt_dynar_get_as(variable->content, i, xbt_peer_t);
+        printf("%s:%d%s", hval->name, hval->port, sep);
+        break;
       }
-      break;
 
-    case xbt_cfgelm_string:
-      for (i = 0; i < size; i++) {
-        sval = xbt_dynar_get_as(variable->content, i, char *);
-        printf("%s'%s'\n", (i == 0 ? "" : "              "), sval);
+      default:
+        printf("Invalid type!!%s", sep);
       }
-      break;
-
-    case xbt_cfgelm_peer:
-      for (i = 0; i < size; i++) {
-        hval = xbt_dynar_get_as(variable->content, i, xbt_peer_t);
-        printf("%s%s:%d\n", (i == 0 ? "" : "              "), hval->name,
-               hval->port);
-      }
-      break;
-
-    default:
-      printf("Invalid type!!\n");
     }
-
   }
 
   xbt_dynar_free(&names);
@@ -473,10 +488,10 @@ static xbt_cfgelm_t xbt_cfgelm_get(xbt_cfg_t cfg,
 
 /** @brief Get the type of this variable in that configuration set
  *
- * \arg cfg the config set
- * \arg name the name of the element
- * \arg type the result
+ * @param cfg the config set
+ * @param name the name of the element
  *
+ * @return the type of the given element
  */
 
 e_xbt_cfgelm_type_t xbt_cfg_get_type(xbt_cfg_t cfg, const char *name)
@@ -490,7 +505,7 @@ e_xbt_cfgelm_type_t xbt_cfg_get_type(xbt_cfg_t cfg, const char *name)
            "Can't get the type of '%s' since this variable does not exist",
            name);
 
-  XBT_INFO("type in variable = %d", (int)variable->type);
+  XBT_DEBUG("type in variable = %d", (int)variable->type);
 
   return variable->type;
 }
@@ -498,9 +513,9 @@ e_xbt_cfgelm_type_t xbt_cfg_get_type(xbt_cfg_t cfg, const char *name)
 /*----[ Setting ]---------------------------------------------------------*/
 /**  @brief va_args version of xbt_cfg_set
  *
- * \arg cfg config set to fill
- * \arg n   variable name
- * \arg pa  variable value
+ * @param cfg config set to fill
+ * @param name  variable name
+ * @param pa  variable value
  *
  * Add some values to the config set.
  */
@@ -548,6 +563,11 @@ void xbt_cfg_set_vargs(xbt_cfg_t cfg, const char *name, va_list pa)
     xbt_cfg_set_double(cfg, name, d);
     break;
 
+  case xbt_cfgelm_boolean:
+    str = va_arg(pa, char *);
+    xbt_cfg_set_boolean(cfg, name, str);
+    break;
+
   default:
     xbt_die("Config element variable %s not valid (type=%d)", name, (int)type);
   }
@@ -555,9 +575,9 @@ void xbt_cfg_set_vargs(xbt_cfg_t cfg, const char *name, va_list pa)
 
 /** @brief Add a NULL-terminated list of pairs {(char*)key, value} to the set
  *
- * \arg cfg config set to fill
- * \arg name variable name
- * \arg varargs variable value
+ * @param cfg config set to fill
+ * @param name variable name
+ * @param ... variable value
  *
  */
 void xbt_cfg_set(xbt_cfg_t cfg, const char *name, ...)
@@ -571,8 +591,8 @@ void xbt_cfg_set(xbt_cfg_t cfg, const char *name, ...)
 
 /** @brief Add values parsed from a string into a config set
  *
- * \arg cfg config set to fill
- * \arg options a string containing the content to add to the config set. This
+ * @param cfg config set to fill
+ * @param options a string containing the content to add to the config set. This
  * is a '\\t',' ' or '\\n' or ',' separated list of variables. Each individual variable is
  * like "[name]:[value]" where [name] is the name of an already registred
  * variable, and [value] conforms to the data type under which this variable was
@@ -581,15 +601,8 @@ void xbt_cfg_set(xbt_cfg_t cfg, const char *name, ...)
  * @todo This is a crude manual parser, it should be a proper lexer.
  */
 
-void xbt_cfg_set_parse(xbt_cfg_t cfg, const char *options)
-{
-  xbt_ex_t e;
+void xbt_cfg_set_parse(xbt_cfg_t cfg, const char *options) {
 
-  int i;
-  double d;
-  char *str;
-
-  volatile xbt_cfgelm_t variable = NULL;
   char *optionlist_cpy;
   char *option, *name, *val;
 
@@ -645,95 +658,107 @@ void xbt_cfg_set_parse(xbt_cfg_t cfg, const char *options)
 
     val = strchr(name, ':');
     if (!val) {
-      free(optionlist_cpy);
+      /* don't free(optionlist_cpy) here, 'name' points inside it */
       xbt_die("Option '%s' badly formated. Should be of the form 'name:value'",
               name);
     }
     *(val++) = '\0';
 
-    if (strncmp(name, "contexts/", strlen("contexts/")))
+    if (strncmp(name, "contexts/", strlen("contexts/")) && strncmp(name, "path", strlen("path")))
       XBT_INFO("Configuration change: Set '%s' to '%s'", name, val);
 
     TRY {
-      variable = xbt_dict_get((xbt_dict_t) cfg, name);
-    }
-    CATCH(e) {
-      if (e.category == not_found_error) {
-        xbt_ex_free(e);
-        TRY {
-          THROWF(not_found_error, 0,
-                 "No registered variable corresponding to '%s'.", name);
-        }
-        TRY_CLEANUP {
-          /* name points into optionlist_cpy, it cannot be freed before */
-          free(optionlist_cpy);
-        }
-        CATCH_ANONYMOUS {
-          RETHROW;
-        }
-      }
-      free(optionlist_cpy);
-      RETHROW;
-    }
-
-    TRY {
-      switch (variable->type) {
-      case xbt_cfgelm_string:
-        xbt_cfg_set_string(cfg, name, val);     /* throws */
-        break;
-
-      case xbt_cfgelm_int:
-        i = strtol(val, &val, 0);
-        if (val == NULL) {
-          free(optionlist_cpy);
-          xbt_die("Value of option %s not valid. Should be an integer", name);
-        }
-
-        xbt_cfg_set_int(cfg, name, i);  /* throws */
-        break;
-
-      case xbt_cfgelm_double:
-        d = strtod(val, &val);
-        if (val == NULL) {
-          free(optionlist_cpy);
-          xbt_die("Value of option %s not valid. Should be a double", name);
-        }
-
-        xbt_cfg_set_double(cfg, name, d);       /* throws */
-        break;
-
-      case xbt_cfgelm_peer:
-        str = val;
-        val = strchr(val, ':');
-        if (!val) {
-          free(optionlist_cpy);
-          xbt_die("Value of option %s not valid. Should be an peer (machine:port)",
-                  name);
-        }
-
-        *(val++) = '\0';
-        i = strtol(val, &val, 0);
-        if (val == NULL) {
-          free(optionlist_cpy);
-          xbt_die("Value of option %s not valid. Should be an peer (machine:port)",
-                  name);
-        }
-
-        xbt_cfg_set_peer(cfg, name, str, i);    /* throws */
-        break;
-
-      default:
-        THROWF(unknown_error, 0, "Type of config element %s is not valid.",
-               name);
-      }
-    }
-    CATCH_ANONYMOUS {
+      xbt_cfg_set_as_string(cfg,name,val);
+    } CATCH_ANONYMOUS {
       free(optionlist_cpy);
       RETHROW;
     }
   }
   free(optionlist_cpy);
+}
 
+/** @brief Set the value of a variable, using the string representation of that value
+ *
+ * @param cfg config set to modify
+ * @param key name of the variable to modify
+ * @param value string representation of the value to set
+ *
+ * @return the first char after the parsed value in val
+ */
+
+void *xbt_cfg_set_as_string(xbt_cfg_t cfg, const char *key, const char *value) {
+  xbt_ex_t e;
+
+  char *ret;
+  volatile xbt_cfgelm_t variable = NULL;
+  int i;
+  double d;
+  char *str, *val;
+
+
+  TRY {
+    variable = xbt_dict_get((xbt_dict_t) cfg, key);
+  }
+  CATCH(e) {
+    if (e.category == not_found_error) {
+      xbt_ex_free(e);
+      THROWF(not_found_error, 0,
+          "No registered variable corresponding to '%s'.", key);
+    }
+    RETHROW;
+  }
+
+  switch (variable->type) {
+  case xbt_cfgelm_string:
+    xbt_cfg_set_string(cfg, key, value);     /* throws */
+    break;
+
+  case xbt_cfgelm_int:
+    i = strtol(value, &ret, 0);
+    if (ret == value) {
+      xbt_die("Value of option %s not valid. Should be an integer", key);
+    }
+
+    xbt_cfg_set_int(cfg, key, i);  /* throws */
+    break;
+
+  case xbt_cfgelm_double:
+    d = strtod(value, &ret);
+    if (ret == value) {
+      xbt_die("Value of option %s not valid. Should be a double", key);
+    }
+
+    xbt_cfg_set_double(cfg, key, d);       /* throws */
+    break;
+
+  case xbt_cfgelm_boolean:
+    xbt_cfg_set_boolean(cfg, key, value);  /* throws */
+    break;
+
+  case xbt_cfgelm_peer:
+    val = xbt_strdup(value);
+    str = val;
+    val = strchr(val, ':');
+    if (!val) {
+      xbt_die("Value of option %s not valid. Should be an peer (machine:port)", key);
+    }
+
+    *(val++) = '\0';
+    i = strtol(val, &ret, 0);
+    if (ret == val) {
+      xbt_die("Value of option %s not valid. Should be an peer (machine:port)", key);
+    }
+
+    xbt_cfg_set_peer(cfg, key, str, i);    /* throws */
+    free(val);
+    break;
+
+  default:
+    THROWF(unknown_error, 0, "Type of config element %s is not valid.", key);
+    break;
+  }
+
+  return ret;
 }
 
 /** @brief Set an integer value to \a name within \a cfg if it wasn't changed yet
@@ -770,7 +795,7 @@ void xbt_cfg_setdefault_double(xbt_cfg_t cfg, const char *name, double val)
   }
   else
     XBT_DEBUG
-        ("Do not override configuration variable '%s' with value '%lf' because it was already set.",
+        ("Do not override configuration variable '%s' with value '%f' because it was already set.",
          name, val);
 }
 
@@ -794,6 +819,26 @@ void xbt_cfg_setdefault_string(xbt_cfg_t cfg, const char *name,
          name, val);
 }
 
+
+/** @brief Set an boolean value to \a name within \a cfg if it wasn't changed yet
+ *
+ * This is useful to change the default value of a variable while allowing
+ * users to override it with command line arguments
+ */
+void xbt_cfg_setdefault_boolean(xbt_cfg_t cfg, const char *name, const char *val)
+{
+  xbt_cfgelm_t variable = xbt_cfgelm_get(cfg, name, xbt_cfgelm_boolean);
+
+  if (variable->isdefault){
+    xbt_cfg_set_boolean(cfg, name, val);
+    variable->isdefault = 1;
+  }
+   else
+    XBT_DEBUG
+        ("Do not override configuration variable '%s' with value '%s' because it was already set.",
+         name, val);
+}
+
 /** @brief Set a peer value to \a name within \a cfg if it wasn't changed yet
  *
  * This is useful to change the default value of a variable while allowing
@@ -816,9 +861,9 @@ void xbt_cfg_setdefault_peer(xbt_cfg_t cfg, const char *name,
 
 /** @brief Set or add an integer value to \a name within \a cfg
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the value of the variable
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value of the variable
  */
 void xbt_cfg_set_int(xbt_cfg_t cfg, const char *name, int val)
 {
@@ -850,9 +895,9 @@ void xbt_cfg_set_int(xbt_cfg_t cfg, const char *name, int val)
 
 /** @brief Set or add a double value to \a name within \a cfg
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the doule to set
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the doule to set
  */
 
 void xbt_cfg_set_double(xbt_cfg_t cfg, const char *name, double val)
@@ -884,9 +929,9 @@ void xbt_cfg_set_double(xbt_cfg_t cfg, const char *name, double val)
 
 /** @brief Set or add a string value to \a name within \a cfg
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the value to be added
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value to be added
  *
  */
 
@@ -927,12 +972,61 @@ void xbt_cfg_set_string(xbt_cfg_t cfg, const char *name, const char *val)
   variable->isdefault = 0;
 }
 
+/** @brief Set or add a boolean value to \a name within \a cfg
+ *
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value of the variable
+ */
+void xbt_cfg_set_boolean(xbt_cfg_t cfg, const char *name, const char *val)
+{
+  xbt_cfgelm_t variable;
+  int i, bval;
+
+  XBT_VERB("Configuration setting: %s=%s", name, val);
+  variable = xbt_cfgelm_get(cfg, name, xbt_cfgelm_boolean);
+
+  for (i = 0; xbt_cfgelm_boolean_values[i].true_val != NULL; i++) {
+	if (strcmp(val, xbt_cfgelm_boolean_values[i].true_val) == 0){
+	  bval = 1;
+	  break;
+	}
+	if (strcmp(val, xbt_cfgelm_boolean_values[i].false_val) == 0){
+	  bval = 0;
+	  break;
+	}
+  }
+  if (xbt_cfgelm_boolean_values[i].true_val == NULL) {
+    xbt_die("Value of option '%s' not valid. Should be a boolean (yes,no,on,off,true,false,0,1)", val);
+  }
+
+  if (variable->max == 1) {
+    if (variable->cb_rm && !xbt_dynar_is_empty(variable->content))
+      variable->cb_rm(name, 0);
+
+    xbt_dynar_set(variable->content, 0, &bval);
+  } else {
+    if (variable->max
+        && xbt_dynar_length(variable->content) ==
+        (unsigned long) variable->max)
+      THROWF(mismatch_error, 0,
+             "Cannot add value %s to the config element %s since it's already full (size=%d)",
+             val, name, variable->max);
+
+    xbt_dynar_push(variable->content, &bval);
+  }
+
+  if (variable->cb_set)
+    variable->cb_set(name, xbt_dynar_length(variable->content) - 1);
+  variable->isdefault = 0;
+}
+
 /** @brief Set or add an peer value to \a name within \a cfg
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg peer the peer
- * \arg port the port number
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param peer the peer
+ * @param port the port number
  *
  * \e peer values are composed of a string (peername) and an integer (port)
  */
@@ -972,9 +1066,9 @@ xbt_cfg_set_peer(xbt_cfg_t cfg, const char *name, const char *peer,
 
 /** @brief Remove the provided \e val integer value from a variable
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the value to be removed
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value to be removed
  */
 void xbt_cfg_rm_int(xbt_cfg_t cfg, const char *name, int val)
 {
@@ -1006,9 +1100,9 @@ void xbt_cfg_rm_int(xbt_cfg_t cfg, const char *name, int val)
 
 /** @brief Remove the provided \e val double value from a variable
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the value to be removed
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value to be removed
  */
 
 void xbt_cfg_rm_double(xbt_cfg_t cfg, const char *name, double val)
@@ -1040,9 +1134,9 @@ void xbt_cfg_rm_double(xbt_cfg_t cfg, const char *name, double val)
 
 /** @brief Remove the provided \e val string value from a variable
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the value of the string which will be removed
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value of the string which will be removed
  */
 void xbt_cfg_rm_string(xbt_cfg_t cfg, const char *name, const char *val)
 {
@@ -1071,12 +1165,46 @@ void xbt_cfg_rm_string(xbt_cfg_t cfg, const char *name, const char *val)
          val, name);
 }
 
+/** @brief Remove the provided \e val boolean value from a variable
+ *
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param val the value to be removed
+ */
+void xbt_cfg_rm_boolean(xbt_cfg_t cfg, const char *name, int val)
+{
+
+  xbt_cfgelm_t variable;
+  unsigned int cpt;
+  int seen;
+
+  variable = xbt_cfgelm_get(cfg, name, xbt_cfgelm_boolean);
+
+  if (xbt_dynar_length(variable->content) == variable->min)
+    THROWF(mismatch_error, 0,
+           "Cannot remove value %d from the config element %s since it's already at its minimal size (=%d)",
+           val, name, variable->min);
+
+  xbt_dynar_foreach(variable->content, cpt, seen) {
+    if (seen == val) {
+      if (variable->cb_rm)
+        variable->cb_rm(name, cpt);
+      xbt_dynar_cursor_rm(variable->content, &cpt);
+      return;
+    }
+  }
+
+  THROWF(not_found_error, 0,
+         "Can't remove the value %d of config element %s: value not found.",
+         val, name);
+}
+
 /** @brief Remove the provided \e val peer value from a variable
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg peer the peername
- * \arg port the port number
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param peer the peername
+ * @param port the port number
  */
 
 void
@@ -1129,8 +1257,8 @@ void xbt_cfg_rm_at(xbt_cfg_t cfg, const char *name, int pos)
 
 /** @brief Remove all the values from a variable
  *
- * \arg cfg the config set
- * \arg name the name of the variable
+ * @param cfg the config set
+ * @param name the name of the variable
  */
 
 void xbt_cfg_empty(xbt_cfg_t cfg, const char *name)
@@ -1175,9 +1303,8 @@ int xbt_cfg_is_default_value(xbt_cfg_t cfg, const char *name)
 
 /** @brief Retrieve an integer value of a variable (get a warning if not uniq)
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the wanted value
+ * @param cfg the config set
+ * @param name the name of the variable
  *
  * Returns the first value from the config set under the given name.
  * If there is more than one value, it will issue a warning. Consider using
@@ -1200,9 +1327,8 @@ int xbt_cfg_get_int(xbt_cfg_t cfg, const char *name)
 
 /** @brief Retrieve a double value of a variable (get a warning if not uniq)
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg val the wanted value
+ * @param cfg the config set
+ * @param name the name of the variable
  *
  * Returns the first value from the config set under the given name.
  * If there is more than one value, it will issue a warning. Consider using
@@ -1226,9 +1352,8 @@ double xbt_cfg_get_double(xbt_cfg_t cfg, const char *name)
 
 /** @brief Retrieve a string value of a variable (get a warning if not uniq)
  *
- * \arg th the config set
- * \arg name the name of the variable
- * \arg val the wanted value
+ * @param cfg the config set
+ * @param name the name of the variable
  *
  * Returns the first value from the config set under the given name.
  * If there is more than one value, it will issue a warning. Consider using
@@ -1252,12 +1377,36 @@ char *xbt_cfg_get_string(xbt_cfg_t cfg, const char *name)
   return xbt_dynar_get_as(variable->content, 0, char *);
 }
 
+/** @brief Retrieve a boolean value of a variable (get a warning if not uniq)
+ *
+ * @param cfg the config set
+ * @param name the name of the variable
+ *
+ * Returns the first value from the config set under the given name.
+ * If there is more than one value, it will issue a warning. Consider using
+ * xbt_cfg_get_dynar() instead.
+ *
+ * \warning the returned value is the actual content of the config set
+ */
+int xbt_cfg_get_boolean(xbt_cfg_t cfg, const char *name)
+{
+  xbt_cfgelm_t variable = xbt_cfgelm_get(cfg, name, xbt_cfgelm_boolean);
+
+  if (xbt_dynar_length(variable->content) > 1) {
+    XBT_WARN
+        ("You asked for the first value of the config element '%s', but there is %lu values",
+         name, xbt_dynar_length(variable->content));
+  }
+
+  return xbt_dynar_get_as(variable->content, 0, int);
+}
+
 /** @brief Retrieve an peer value of a variable (get a warning if not uniq)
  *
- * \arg cfg the config set
- * \arg name the name of the variable
- * \arg peer the peer
- * \arg port the port number
+ * @param cfg the config set
+ * @param name the name of the variable
+ * @param peer the peer
+ * @param port the port number
  *
  * Returns the first value from the config set under the given name.
  * If there is more than one value, it will issue a warning. Consider using
@@ -1287,9 +1436,8 @@ void xbt_cfg_get_peer(xbt_cfg_t cfg, const char *name, char **peer,
 
 /** @brief Retrieve the dynar of all the values stored in a variable
  *
- * \arg cfg where to search in
- * \arg name what to search for
- * \arg dynar result
+ * @param cfg where to search in
+ * @param name what to search for
  *
  * Get the data stored in the config set.
  *
@@ -1341,6 +1489,14 @@ char *xbt_cfg_get_string_at(xbt_cfg_t cfg, const char *name, int pos)
   return xbt_dynar_get_as(variable->content, pos, char *);
 }
 
+/** @brief Retrieve one of the boolean value of a variable */
+int xbt_cfg_get_boolean_at(xbt_cfg_t cfg, const char *name, int pos)
+{
+
+  xbt_cfgelm_t variable = xbt_cfgelm_get(cfg, name, xbt_cfgelm_boolean);
+  return xbt_dynar_get_as(variable->content, pos, int);
+}
+
 /** @brief Retrieve one of the peer value of a variable */
 void
 xbt_cfg_get_peer_at(xbt_cfg_t cfg, const char *name, int pos,
