@@ -1,6 +1,6 @@
 /* dict - a generic dictionary, variation over hash table                   */
 
-/* Copyright (c) 2004-2011. The SimGrid Team.
+/* Copyright (c) 2004-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -171,8 +171,9 @@ XBT_INLINE void xbt_dict_set_ext(xbt_dict_t dict,
   xbt_dictelm_t current, previous = NULL;
   xbt_assert(dict);
 
-  XBT_DEBUG("ADD %.*s hash = %u, size = %d, & = %u", key_len, key, hash_code,
-         dict->table_size, hash_code & dict->table_size);
+  XBT_CDEBUG(xbt_dict,
+             "ADD %.*s hash = %u, size = %d, & = %u", key_len, key, hash_code,
+             dict->table_size, hash_code & dict->table_size);
   current = dict->table[hash_code & dict->table_size];
   while (current != NULL &&
          (hash_code != current->hash_code || key_len != current->key_len
@@ -194,9 +195,9 @@ XBT_INLINE void xbt_dict_set_ext(xbt_dict_t dict,
       previous->next = current;
     }
   } else {
-    XBT_DEBUG("Replace %.*s by %.*s under key %.*s",
-           key_len, (char *) current->content,
-           key_len, (char *) data, key_len, (char *) key);
+    XBT_CDEBUG(xbt_dict, "Replace %.*s by %.*s under key %.*s",
+               key_len, (char *) current->content,
+               key_len, (char *) data, key_len, (char *) key);
     /* there is already an element with the same key: overwrite it */
     xbt_dictelm_set_data(dict, current, data, free_ctn);
   }
@@ -316,27 +317,45 @@ char *xbt_dict_get_key(xbt_dict_t dict, const void *data)
  */
 XBT_INLINE void *xbt_dict_get(xbt_dict_t dict, const char *key)
 {
-
-  unsigned int hash_code = xbt_str_hash(key);
-  xbt_dictelm_t current;
-
-  xbt_assert(dict);
-
-  current = dict->table[hash_code & dict->table_size];
-  while (current != NULL
-         && (hash_code != current->hash_code || strcmp(key, current->key)))
-    current = current->next;
+  return xbt_dict_get_elm(dict, key)->content;
+}
+/**
+ * \brief Retrieve element from the dict (null-terminated key)
+ *
+ * \param dict the dealer of data
+ * \param key the key to find data
+ * \return the s_xbt_dictelm_t that we are looking for
+ *
+ * Search the given \a key. Throws not_found_error when not found.
+ * Check xbt_dict_get_or_null() for a version returning NULL without exception when
+ * not found.
+ */
+XBT_INLINE xbt_dictelm_t xbt_dict_get_elm(xbt_dict_t dict, const char *key)
+{
+  xbt_dictelm_t current = xbt_dict_get_elm_or_null(dict, key);
 
   if (current == NULL)
     THROWF(not_found_error, 0, "key %s not found", key);
 
-  return current->content;
+  return current;
 }
 
 /**
  * \brief like xbt_dict_get(), but returning NULL when not found
  */
 XBT_INLINE void *xbt_dict_get_or_null(xbt_dict_t dict, const char *key)
+{
+  xbt_dictelm_t current = xbt_dict_get_elm_or_null(dict, key);
+  
+  if (current == NULL)
+    return NULL;
+
+  return current->content;
+}
+/**
+ * \brief like xbt_dict_get_elm(), but returning NULL when not found
+ */
+XBT_INLINE xbt_dictelm_t xbt_dict_get_elm_or_null(xbt_dict_t dict, const char *key)
 {
   unsigned int hash_code = xbt_str_hash(key);
   xbt_dictelm_t current;
@@ -347,11 +366,7 @@ XBT_INLINE void *xbt_dict_get_or_null(xbt_dict_t dict, const char *key)
   while (current != NULL &&
          (hash_code != current->hash_code || strcmp(key, current->key)))
     current = current->next;
-
-  if (current == NULL)
-    return NULL;
-
-  return current->content;
+  return current;
 }
 
 
@@ -658,11 +673,6 @@ XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(xbt_dict);
 
 XBT_TEST_SUITE("dict", "Dict data container");
 
-static void print_str(void *str)
-{
-  printf("%s", (char *) PRINTF_STR(str));
-}
-
 static void debuged_add_ext(xbt_dict_t head, const char *key,
                             const char *data_to_fill, void_f_pvoid_t free_f)
 {
@@ -704,19 +714,24 @@ static void fill(xbt_dict_t * head, int homogeneous)
 
 static void search_ext(xbt_dict_t head, const char *key, const char *data)
 {
-  void *found;
+  char *found;
 
   xbt_test_add("Search %s", key);
   found = xbt_dict_get(head, key);
-  xbt_test_log("Found %s", (char *) found);
-  if (data)
+  xbt_test_log("Found %s", found);
+  if (data) {
     xbt_test_assert(found,
-                     "data do not match expectations: found NULL while searching for %s",
-                     data);
-  if (found)
-    xbt_test_assert(!strcmp((char *) data, found),
-                     "data do not match expectations: found %s while searching for %s",
-                     (char *) found, data);
+                    "data do not match expectations: found NULL while searching for %s",
+                    data);
+    if (found)
+      xbt_test_assert(!strcmp(data, found),
+                      "data do not match expectations: found %s while searching for %s",
+                      found, data);
+  } else {
+    xbt_test_assert(!found,
+                    "data do not match expectations: found %s while searching for NULL",
+                    found);
+  }
 }
 
 static void search(xbt_dict_t head, const char *key)
