@@ -1,7 +1,7 @@
 /* dict_elm - elements of generic dictionnaries                             */
 /* This file is not to be loaded from anywhere but dict.c                   */
 
-/* Copyright (c) 2004-2011. The SimGrid Team.
+/* Copyright (c) 2004-2011, 2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -17,19 +17,8 @@
 #include "xbt/dict.h"
 #include "xbt/mallocator.h"
 
-typedef struct s_xbt_dictelm *xbt_dictelm_t;
-
 #define MAX_FILL_PERCENT 80
 
-typedef struct s_xbt_dictelm {
-  char *key;
-  int key_len;
-  unsigned int hash_code;
-
-  void *content;
-
-  xbt_dictelm_t next;
-} s_xbt_dictelm_t;
 
 typedef struct s_xbt_het_dictelm {
   s_xbt_dictelm_t element;
