@@ -1,6 +1,6 @@
 /* ex - Exception Handling                                                  */
 
-/*  Copyright (c) 2005-2010 The SimGrid team                                */
+/*  Copyright (c) 2005-2013. The SimGrid Team.                              */
 /*  Copyright (c) 2002-2004 Ralf S. Engelschall <rse@engelschall.com>       */
 /*  Copyright (c) 2002-2004 The OSSP Project <http://www.ossp.org/>         */
 /*  Copyright (c) 2002-2004 Cable & Wireless <http://www.cw.com/>           */
@@ -46,11 +46,9 @@
 #include "portable.h"           /* execinfo when available */
 #include "xbt/ex.h"
 #include "xbt/str.h"
-#include "xbt/module.h"         /* xbt_binary_name */
+#include "xbt/synchro_core.h"
 #include "xbt_modinter.h"       /* backtrace initialization headers */
-#include "xbt/synchro.h"        /* xbt_thread_self */
 
-#include "gras/Virtu/virtu_interface.h" /* gras_os_myname */
 #include "xbt/ex_interface.h"
 
 #undef HAVE_BACKTRACE
@@ -77,7 +75,7 @@ xbt_running_ctx_t *__xbt_ex_ctx_default(void)
      real life and in simulation when using threads to implement the simulation
      processes (ie, with pthreads and on windows).
 
-     It also gets overriden in xbt/context.c when using ucontextes (as well as
+     It also gets overriden in xbt/context.c when using ucontexts (as well as
      in Java for now, but after the java overhaul, it will get cleaned out)
    */
   static xbt_running_ctx_t ctx = XBT_RUNNING_CTX_INITIALIZER;
@@ -106,7 +104,6 @@ void xbt_backtrace_display(xbt_ex_t * e)
 
   /* don't fool xbt_ex_free with uninitialized msg field */
   e->msg = NULL;
-  e->remote = 0;
   xbt_ex_free(*e);
 #else
 
@@ -135,20 +132,17 @@ void xbt_ex_display(xbt_ex_t * e)
 {
   char *thrower = NULL;
 
-  if (e->remote)
-    thrower = bprintf(" on host %s(%d)", e->host, e->pid);
-
   fprintf(stderr,
           "** SimGrid: UNCAUGHT EXCEPTION received on %s(%d): category: %s; value: %d\n"
           "** %s\n"
           "** Thrown by %s()%s\n",
-          gras_os_myname(), xbt_getpid(),
+          xbt_binary_name, xbt_getpid(),
           xbt_ex_catname(e->category), e->value, e->msg,
           e->procname, thrower ? thrower : " in this process");
   XBT_CRITICAL("%s", e->msg);
   xbt_free(thrower);
 
-  if (!e->remote && !e->bt_strings)
+  if (!e->bt_strings)
     xbt_ex_setup_backtrace(e);
 
 #ifdef HAVE_BACKTRACE
@@ -190,12 +184,6 @@ void xbt_ex_free(xbt_ex_t e)
   int i;
 
   free(e.msg);
-  if (e.remote) {
-    free(e.procname);
-    free(e.file);
-    free(e.func);
-    free(e.host);
-  }
 
   if (e.bt_strings) {
     for (i = 0; i < e.used; i++)
@@ -319,7 +307,8 @@ XBT_TEST_UNIT("variables", test_variables, "variable value preservation")
   xbt_ex_t ex;
   int r1;
   int _XBT_GNUC_UNUSED r2;
-  volatile int v1, v2;
+  int v1;
+  volatile int v2;
 
   r1 = r2 = v1 = v2 = 1234;
   TRY {
@@ -421,7 +410,7 @@ typedef struct {
 
 static void good_example(void)
 {
-  global_context_t *global_context = malloc(sizeof(global_context_t));
+  global_context_t *global_context = xbt_malloc(sizeof(global_context_t));
 
   /* GOOD_EXAMPLE */
   {                             /*01 */
