@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -331,6 +331,42 @@ int xbt_fifo_is_in(xbt_fifo_t f, void *content)
   return 0;
 }
 
+/**
+ * @brief Search the given element in the fifo using a comparison function
+ *
+ * This function allows to search an item with a user provided function instead
+ * of the pointer comparison used elsewhere in this module. Assume for example that you have a fifo of
+ * strings. You cannot use xbt_fifo_remove() to remove, say, "TOTO" from it because internally, xbt_fifo_remove()
+ * will do something like "if (item->content == "toto"), then remove it". And the pointer to the item content
+ * and the pointer to "toto" will never match. As a solution, the current function provides a way to search elements
+ * that are semanticaly equivalent instead of only syntaxically. So, removing "Toto" from a fifo can be
+ * achieved this way:
+ *
+ *  @verbatim
+int my_comparison_function(void *searched, void *seen) {
+  return !strcmp(searched, seen);
+}
+
+  xbt_fifo_remove_item(fifo,
+                       xbt_fifo_search_item(fifo, my_comparison_function, "Toto"));
+@endverbatim
+ *
+ * \param f a fifo list
+ * \param cmp_fun the comparison function. Prototype: void *a,void *b -> int. Semantic: returns true iff a=b
+ * @param closure the element to search. It will be provided as first argument to each call of cmp_fun
+ * \return the first item matching the comparison function, or NULL if no such item exists
+ */
+xbt_fifo_item_t xbt_fifo_search_item(xbt_fifo_t f, int_f_pvoid_pvoid_t cmp_fun, void *closure) {
+  xbt_fifo_item_t item = xbt_fifo_get_first_item(f);
+  while (item) {
+    if (cmp_fun(closure, item->content))
+      return item;
+    item = item->next;
+  }
+  return NULL;
+
+}
+
 /**
  * \param f a list
  * \return a table with the objects stored in \a f.
@@ -393,7 +429,7 @@ XBT_INLINE xbt_fifo_item_t xbt_fifo_new_item(void)
  */
 XBT_INLINE xbt_fifo_item_t xbt_fifo_newitem(void)
 {
-  XBT_WARN("This function is deprecated. Use xbt_fifo_new_item.");
+  XBT_CWARN(xbt_fifo, "This function is deprecated. Use xbt_fifo_new_item.");
   return xbt_fifo_new_item();
 }
 
@@ -433,7 +469,7 @@ XBT_INLINE void xbt_fifo_free_item(xbt_fifo_item_t b)
  */
 XBT_INLINE void xbt_fifo_freeitem(xbt_fifo_item_t b)
 {
-  XBT_WARN("This function is deprecated. Use xbt_fifo_free_item.");
+  XBT_CWARN(xbt_fifo, "This function is deprecated. Use xbt_fifo_free_item.");
   xbt_fifo_free_item(b);
   return;
 }
@@ -473,7 +509,7 @@ XBT_INLINE xbt_fifo_item_t xbt_fifo_get_last_item(xbt_fifo_t l)
  */
 XBT_INLINE xbt_fifo_item_t xbt_fifo_getFirstItem(xbt_fifo_t l)
 {
-  XBT_WARN("This function is deprecated. Use xbt_fifo_get_first_item.");
+  XBT_CWARN(xbt_fifo, "This function is deprecated. Use xbt_fifo_get_first_item.");
   return xbt_fifo_get_first_item(l);
 }
 
@@ -494,7 +530,7 @@ XBT_INLINE xbt_fifo_item_t xbt_fifo_get_next_item(xbt_fifo_item_t i)
  */
 xbt_fifo_item_t xbt_fifo_getNextItem(xbt_fifo_item_t i)
 {
-  XBT_WARN("This function is deprecated. Use xbt_fifo_get_next_item.");
+  XBT_CWARN(xbt_fifo, "This function is deprecated. Use xbt_fifo_get_next_item.");
   return xbt_fifo_get_next_item(i);
 }
 
