@@ -3,7 +3,8 @@
 
    Contributed by Fred Fish at Cygnus Support.   fnf@cygnus.com */
 
-/* Copyright (c) 2010-2012. The SimGrid Team. All rights reserved.          */
+/* Copyright (c) 2010-2012. The SimGrid Team.
+ * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
@@ -53,7 +54,7 @@ static size_t pagesize;
 
     It never returns NULL. Instead, it dies verbosely on errors. */
 
-void *mmorecore(struct mdesc *mdp, int size)
+void *mmorecore(struct mdesc *mdp, ssize_t size)
 {
   ssize_t test = 0;
   void *result; // please keep it uninitialized to track issues
