@@ -1,8 +1,8 @@
 /* Change the size of a block allocated by `mmalloc'.
    Copyright 1990, 1991 Free Software Foundation
-		  Written May 1989 by Mike Haertel. */
+   Written May 1989 by Mike Haertel. */
 
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -38,7 +38,7 @@ void *mrealloc(xbt_mheap_t mdp, void *ptr, size_t size)
 
   if ((char *) ptr < (char *) mdp->heapbase || BLOCK(ptr) > mdp->heapsize) {
     printf
-        ("FIXME. Ouch, this pointer is not mine, refusing to proceed (another solution would be to malloc it instead of reallocing it, see source code)\n");
+      ("FIXME. Ouch, this pointer is not mine, refusing to proceed (another solution would be to malloc it instead of reallocing it, see source code)\n");
     result = mmalloc(mdp, size);
     abort();
     return result;
@@ -79,18 +79,21 @@ void *mrealloc(xbt_mheap_t mdp, void *ptr, size_t size)
     /* Full blocks -> Full blocks; see if we can hold it in place. */
     blocks = BLOCKIFY(size);
     if (blocks < mdp->heapinfo[block].busy_block.size) {
-    	int it;
+      int it;
       /* The new size is smaller; return excess memory to the free list. */
       //printf("(%s) return excess memory...",xbt_thread_self_name());
-    	for (it= block+blocks; it< mdp->heapinfo[block].busy_block.size ; it++)
-    	  mdp->heapinfo[it].type = 0; // FIXME that should be useless, type should already be 0 here
+      for (it= block+blocks; it< mdp->heapinfo[block].busy_block.size ; it++){
+        mdp->heapinfo[it].type = 0; // FIXME that should be useless, type should already be 0 here
+        mdp->heapinfo[it].busy_block.ignore = 0;
+      }
 
       mdp->heapinfo[block + blocks].busy_block.size
-          = mdp->heapinfo[block].busy_block.size - blocks;
+        = mdp->heapinfo[block].busy_block.size - blocks;
       mfree(mdp, ADDRESS(block + blocks));
 
       mdp->heapinfo[block].busy_block.size = blocks;
       mdp->heapinfo[block].busy_block.busy_size = requested_size;
+      mdp->heapinfo[block].busy_block.ignore = 0;
 
       result = ptr;
     } else if (blocks == mdp->heapinfo[block].busy_block.size) {
@@ -98,11 +101,13 @@ void *mrealloc(xbt_mheap_t mdp, void *ptr, size_t size)
       /* No block size change necessary; only update the requested size  */
       result = ptr;
       mdp->heapinfo[block].busy_block.busy_size = requested_size;
+      mdp->heapinfo[block].busy_block.ignore = 0;
 
     } else {
       /* Won't fit, so allocate a new region that will.
-         Free the old region first in case there is sufficient
-         adjacent free space to grow without moving. */
+         Free the old region first in case there is sufficient adjacent free space to grow without moving.
+         This trick mandates using a specific version of mmalloc that does not memset the memory to 0 after
+           action for obvious reasons. */
       blocks = mdp->heapinfo[block].busy_block.size;
       /* Prevent free from actually returning memory to the system.  */
       oldlimit = mdp->heaplimit;
@@ -110,9 +115,12 @@ void *mrealloc(xbt_mheap_t mdp, void *ptr, size_t size)
       mfree(mdp, ptr);
       mdp->heaplimit = oldlimit;
 
-      result = mmalloc(mdp, requested_size);
+      result = mmalloc_no_memset(mdp, requested_size);
+      //fprintf(stderr,"remalloc(%zu)~>%p\n",requested_size,result);
+
       if (ptr != result)
         memmove(result, ptr, blocks * BLOCKSIZE);
+      /* FIXME: we should memset the end of the recently area */
     }
     break;
 
@@ -125,6 +133,7 @@ void *mrealloc(xbt_mheap_t mdp, void *ptr, size_t size)
       result = ptr;
       int frag_nb = RESIDUAL(result, BLOCKSIZE) >> type;
       mdp->heapinfo[block].busy_frag.frag_size[frag_nb] = requested_size;
+      mdp->heapinfo[block].busy_frag.ignore[frag_nb] = 0;
 
     } else { /* fragment -> Either other fragment, or block */
       /* The new size is different; allocate a new space,
