@@ -1,18 +1,19 @@
 /* layout_simple - a dumb log layout                                        */
 
-/* Copyright (c) 2007-2011. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
 #include "portable.h"           /* execinfo when available */
+#include "xbt/synchro_core.h"   /* xbt_thread_self_name */
 #include "xbt/ex_interface.h"
 #include "xbt/sysdep.h"
 #include "xbt/strbuff.h"
 #include "xbt/log_private.h"
-#include "gras/virtu.h"         /* gras_os_myname (KILLME) */
-#include "xbt/synchro.h"        /* xbt_thread_self_name */
+#include "simgrid/simix.h"      /* SIMIX_host_self_get_name */
+#include "surf/surf.h"
 #include <stdio.h>
 
 extern const char *xbt_log_priority_names[8];
@@ -126,7 +127,7 @@ static int xbt_log_layout_format_doit(xbt_log_layout_t l,
         show_string(xbt_log_priority_names[ev->priority]);
         break;
       case 'h':                 /* host name; SimGrid extension */
-        show_string(gras_os_myname());
+        show_string(SIMIX_host_self_get_name());
         break;
       case 't':                 /* thread name; LOG4J compliant */
         show_string(xbt_thread_self_name());
@@ -162,7 +163,6 @@ static int xbt_log_layout_format_doit(xbt_log_layout_t l,
           e.used = backtrace((void **) e.bt, XBT_BACKTRACE_SIZE);
           e.bt_strings = NULL;
           e.msg = NULL;
-          e.remote = 0;
           xbt_ex_setup_backtrace(&e);
           if (*q == 'B') {
             show_string(e.bt_strings[1] + 8);
@@ -184,10 +184,10 @@ static int xbt_log_layout_format_doit(xbt_log_layout_t l,
 #endif
         break;
       case 'd':                 /* date; LOG4J compliant */
-        show_double(gras_os_time());
+        show_double(surf_get_clock());
         break;
       case 'r':                 /* application age; LOG4J compliant */
-        show_double(gras_os_time() - format_begin_of_time);
+        show_double(surf_get_clock() - format_begin_of_time);
         break;
       case 'm': {               /* user-provided message; LOG4J compliant */
         int len, sz;
@@ -223,7 +223,7 @@ xbt_log_layout_t xbt_log_layout_format_new(char *arg)
   res->data = xbt_strdup((char *) arg);
 
   if (format_begin_of_time < 0)
-    format_begin_of_time = gras_os_time();
+    format_begin_of_time = surf_get_clock();
 
   return res;
 }
