@@ -1,6 +1,6 @@
 /* module handling                                                          */
 
-/* Copyright (c) 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2006-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -8,8 +8,7 @@
 
 #include "xbt/misc.h"
 #include "simgrid_config.h"     /*HAVE_MMAP _XBT_WIN32 */
-#include "gras_config.h"        /* MMALLOC_WANT_OVERRIDE_LEGACY */
-#include "time.h"               /* to seed the random generator */
+#include "internal_config.h"        /* MMALLOC_WANT_OVERRIDE_LEGACY */
 
 #include "xbt/sysdep.h"
 #include "xbt/log.h"
@@ -20,18 +19,21 @@
 
 #include "xbt_modinter.h"       /* prototype of other module's init/exit in XBT */
 
+#include "simgrid/sg_config.h"
+
+#include <stdio.h>
+
 XBT_LOG_NEW_CATEGORY(xbt, "All XBT categories (simgrid toolbox)");
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(module, xbt, "module handling");
 
 XBT_LOG_NEW_CATEGORY(smpi, "All SMPI categories"); /* lives here even if that's a bit odd to solve linking issues: this is used in xbt_log_file_appender to detect whether SMPI is used (and thus whether we should unbench the writing to disk) */
 
 
-char *xbt_binary_name = NULL;   /* Mandatory to retrieve neat backtraces */
-int xbt_initialized = 0;
+char *xbt_binary_name = NULL;   /* Name of the system process containing us (mandatory to retrieve neat backtraces) */
+xbt_dynar_t xbt_cmdline = NULL; /* all we got in argv */
 
-int _surf_do_model_check = 0;
-int _surf_mc_checkpoint=0;
-char* _surf_mc_property_file=NULL;
+int xbt_initialized = 0;
+int _sg_do_clean_atexit = 1;
 
 /* Declare xbt_preinit and xbt_postexit as constructor/destructor of the library.
  * This is crude and rather compiler-specific, unfortunately.
@@ -79,36 +81,39 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason,
 
 #endif
 
-static void xbt_preinit(void)
-{
+static void xbt_preinit(void) {
+  unsigned int seed = 2147483647;
+
 #ifdef MMALLOC_WANT_OVERRIDE_LEGACY
   mmalloc_preinit();
+#endif
+#ifdef _TWO_DIGIT_EXPONENT
+  /* Even printf behaves differently on Windows... */
+  _set_output_format(_TWO_DIGIT_EXPONENT);
 #endif
   xbt_log_preinit();
-
   xbt_backtrace_preinit();
   xbt_os_thread_mod_preinit();
   xbt_fifo_preinit();
   xbt_dict_preinit();
-  xbt_datadesc_preinit();
-  xbt_trp_preinit();
+   
+  srand(seed);
+#ifndef _WIN32
+  srand48(seed);
+#endif
 
   atexit(xbt_postexit);
 }
 
 static void xbt_postexit(void)
 {
-  xbt_trp_postexit();
-  xbt_datadesc_postexit();
-
+  if(!_sg_do_clean_atexit) return;
   xbt_backtrace_postexit();
-
   xbt_fifo_postexit();
   xbt_dict_postexit();
-
-  xbt_log_postexit();
   xbt_os_thread_mod_postexit();
-
+  xbt_dynar_free(&xbt_cmdline);
+  xbt_log_postexit();
   free(xbt_binary_name);
 #ifdef MMALLOC_WANT_OVERRIDE_LEGACY
   mmalloc_postexit();
@@ -118,19 +123,23 @@ static void xbt_postexit(void)
 /** @brief Initialize the xbt mechanisms. */
 void xbt_init(int *argc, char **argv)
 {
-  // FIXME it would be nice to assert that this function is called only once. But each gras process do call it...
-  xbt_initialized++;
-
-  if (xbt_initialized > 1)
+  if (xbt_initialized++) {
+    XBT_DEBUG("XBT was initialized %d times.", xbt_initialized);
     return;
+  }
 
   xbt_binary_name = xbt_strdup(argv[0]);
-  srand((unsigned int) time(NULL));
-
+  xbt_cmdline = xbt_dynar_new(sizeof(char*),NULL);
+  int i;
+  for (i=0;i<*argc;i++) {
+    xbt_dynar_push(xbt_cmdline,&(argv[i]));
+  }
+  
   xbt_log_init(argc, argv);
 }
 
-/** @brief Finalize the xbt mechanisms. */
+/** @brief Finalize the xbt mechanisms.
+ *  @warning this function is deprecated. Just don't call it, there is nothing more to do to finalize xbt*/
 void xbt_exit()
 {
   XBT_WARN("This function is deprecated, you shouldn't use it");
