@@ -2,13 +2,13 @@
 /* Used in RL to get win/lin portability, and in SG when CONTEXT_THREAD     */
 /* in SG, when using CONTEXT_UCONTEXT, xbt_os_thread_stub is used instead   */
 
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "gras_config.h"
+#include "internal_config.h"
 #include "xbt/sysdep.h"
 #include "xbt/ex.h"
 #include "xbt/ex_interface.h"   /* We play crude games with exceptions */
@@ -23,6 +23,7 @@ XBT_LOG_NEW_DEFAULT_SUBCATEGORY(xbt_sync_os, xbt,
 /* ********************************* PTHREAD IMPLEMENTATION ************************************ */
 #ifdef HAVE_PTHREAD_H
 
+#include <limits.h>
 #include <semaphore.h>
 
 #ifdef HAVE_MUTEX_TIMEDLOCK
@@ -53,6 +54,11 @@ static xbt_os_thread_t main_thread = NULL;
 static pthread_key_t xbt_self_thread_key;
 static int thread_mod_inited = 0;
 
+/* attribute structure to handle pthread stack size changing */
+//FIXME: find where to put this
+static pthread_attr_t attr;
+static int thread_attr_inited = 0;
+
 /* frees the xbt_os_thread_t corresponding to the current thread */
 static void xbt_os_thread_free_thread_data(xbt_os_thread_t thread)
 {
@@ -159,6 +165,7 @@ static void *wrapper_start_routine(void *s)
   return res;
 }
 
+
 xbt_os_thread_t xbt_os_thread_create(const char *name,
                                      pvoid_f_pvoid_t start_routine,
                                      void *param,
@@ -175,14 +182,50 @@ xbt_os_thread_t xbt_os_thread_create(const char *name,
   XBT_RUNNING_CTX_INITIALIZE(res_thread->running_ctx);
   res_thread->extra_data = extra_data;
   
-  if ((errcode = pthread_create(&(res_thread->t), NULL,
+  if ((errcode = pthread_create(&(res_thread->t), thread_attr_inited!=0? &attr: NULL,
                                 wrapper_start_routine, res_thread)))
     THROWF(system_error, errcode,
            "pthread_create failed: %s", strerror(errcode));
 
+
+
   return res_thread;
 }
 
+
+void xbt_os_thread_setstacksize(int stack_size)
+{
+  size_t sz;
+  int res;
+
+  if (stack_size < 0)
+    xbt_die("stack size %d is negative, maybe it exceeds MAX_INT?", stack_size);
+
+  sz = stack_size;
+  pthread_attr_init(&attr);
+  res = pthread_attr_setstacksize(&attr, sz);
+
+#ifdef PTHREAD_STACK_MIN
+  if (res == EINVAL) {
+    /* Invalid size, try again with a multiple of PTHREAD_STACK_MIN. */
+    size_t rem = sz % PTHREAD_STACK_MIN;
+    if (rem != 0 || sz == 0) {
+      size_t sz2 = sz - rem + PTHREAD_STACK_MIN;
+      XBT_DEBUG("pthread_attr_setstacksize failed for %#zx, try again with %#zx",
+                sz, sz2);
+      sz = sz2;
+      res = pthread_attr_setstacksize(&attr, sz);
+    }
+  }
+#endif
+
+  if (res == EINVAL)
+    XBT_WARN("invalid stack size (maybe too big): %#zx", sz);
+  else if (res != 0)
+    XBT_WARN("unknown error %d in pthread stacksize setting: %#zx", res, sz);
+  thread_attr_inited = 1;
+}
+
 const char *xbt_os_thread_name(xbt_os_thread_t t)
 {
   return t->name;
@@ -305,7 +348,7 @@ void xbt_os_mutex_timedacquire(xbt_os_mutex_t mutex, double delay)
       THROWF(timeout_error, 0, "mutex %p not ready", mutex);
     default:
       THROWF(system_error, errcode,
-             "xbt_mutex_timedacquire(%p) failed: %s", mutex,
+             "xbt_os_mutex_timedacquire(%p) failed: %s", mutex,
              strerror(errcode));
     }
 
@@ -643,7 +686,7 @@ typedef struct xbt_os_thread_ {
 
 /* the default size of the stack of the threads (in bytes)*/
 #define XBT_DEFAULT_THREAD_STACK_SIZE  4096
-
+static int stack_size=0;
 /* key to the TLS containing the xbt_os_thread_t structure */
 static unsigned long xbt_self_thread_key;
 
@@ -694,7 +737,7 @@ xbt_os_thread_t xbt_os_thread_create(const char *name,
   t->start_routine = start_routine;
   t->param = param;
   t->extra_data = extra_data;
-  t->handle = CreateThread(NULL, XBT_DEFAULT_THREAD_STACK_SIZE,
+  t->handle = CreateThread(NULL, stack_size==0 ? XBT_DEFAULT_THREAD_STACK_SIZE : stack_size,
                            (LPTHREAD_START_ROUTINE) wrapper_start_routine,
                            t, STACK_SIZE_PARAM_IS_A_RESERVATION, &(t->id));
 
@@ -706,6 +749,11 @@ xbt_os_thread_t xbt_os_thread_create(const char *name,
   return t;
 }
 
+void xbt_os_thread_setstacksize(int size)
+{
+  stack_size = size;
+}
+
 const char *xbt_os_thread_name(xbt_os_thread_t t)
 {
   return t->name;
@@ -1204,7 +1252,8 @@ void xbt_os_thread_set_extra_data(void *data)
 
 void *xbt_os_thread_get_extra_data(void)
 {
-  return xbt_os_thread_self()->extra_data;
+  xbt_os_thread_t self = xbt_os_thread_self();
+  return self? self->extra_data : NULL;
 }
 
 xbt_os_rmutex_t xbt_os_rmutex_init(void)
