@@ -1,16 +1,17 @@
-/* Copyright (c) 2010. The SimGrid Team.
+/* Copyright (c) 2010, 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
-#include "simgrid_config.h" //For getline, keep that include first
 
-#include "gras_config.h"
+#include "internal_config.h"
 #include <errno.h>
 #include "xbt/sysdep.h"
 #include "xbt/log.h"
 #include "xbt/str.h"
 #include "xbt/replay.h"
+#include <ctype.h>
+#include <wchar.h>
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(replay,xbt,"Replay trace reader");
 
@@ -30,7 +31,17 @@ xbt_dict_t action_queues;
 static char *action_line = NULL;
 static size_t action_len = 0;
 
-static const char **action_get_action(char *name);
+static char **action_get_action(char *name);
+
+static char *str_tolower (const char *str)
+{
+  char *ret = xbt_strdup (str);
+  int i, n = strlen (ret);
+  for (i = 0; i < n; i++)
+    ret[i] = tolower (str[i]);
+  return ret;
+}
+
 
 xbt_replay_reader_t xbt_replay_reader_new(const char *filename)
 {
@@ -53,7 +64,7 @@ const char **xbt_replay_reader_get(xbt_replay_reader_t reader)
 {
   ssize_t read;
   xbt_dynar_t d;
-  read = getline(&reader->line, &reader->line_len, reader->fp);
+  read = xbt_getline(&reader->line, &reader->line_len, reader->fp);
   //XBT_INFO("got from trace: %s",reader->line);
   reader->linenum++;
   if (read==-1)
@@ -97,7 +108,9 @@ void xbt_replay_reader_free(xbt_replay_reader_t *reader)
  */
 void xbt_replay_action_register(const char *action_name, action_fun function)
 {
-  xbt_dict_set(action_funs, action_name, function, NULL);
+  char* lowername = str_tolower (action_name);
+  xbt_dict_set(action_funs, lowername, function, NULL);
+  xbt_free(lowername);
 }
 
 /** \ingroup XBT_replay
@@ -107,7 +120,9 @@ void xbt_replay_action_register(const char *action_name, action_fun function)
  */
 void xbt_replay_action_unregister(const char *action_name)
 {
-  xbt_dict_remove(action_funs, action_name);
+  char* lowername = str_tolower (action_name);
+  xbt_dict_remove(action_funs, lowername);
+  xbt_free(lowername);
 }
 
 void _xbt_replay_action_init(void)
@@ -132,16 +147,21 @@ void _xbt_replay_action_exit(void)
  */
 int xbt_replay_action_runner(int argc, char *argv[])
 {
-  const char **evt;
+  int i;
   if (action_fp) {              // A unique trace file
-
+    char **evt;
     while ((evt = action_get_action(argv[0]))) {
+      char* lowername = str_tolower (evt[1]);
       action_fun function =
-        (action_fun)xbt_dict_get(action_funs, evt[1]);
-      function(evt);
+        (action_fun)xbt_dict_get(action_funs, lowername);
+      xbt_free(lowername);
+      function((const char **)evt);
+      for (i=0;evt[i]!= NULL;i++)
+        free(evt[i]);
       free(evt);
     }
   } else {                      // Should have got my trace file in argument
+    const char **evt;
     xbt_assert(argc >= 2,
                 "No '%s' agent function provided, no simulation-wide trace file provided, "
                 "and no process-wide trace file provided in deployment file. Aborting.",
@@ -150,14 +170,15 @@ int xbt_replay_action_runner(int argc, char *argv[])
     xbt_replay_reader_t reader = xbt_replay_reader_new(argv[1]);
     while ((evt=xbt_replay_reader_get(reader))) {
       if (!strcmp(argv[0],evt[0])) {
-        action_fun function =
-          (action_fun)xbt_dict_get(action_funs, evt[1]);
+        char* lowername = str_tolower (evt[1]);
+        action_fun function = (action_fun)xbt_dict_get(action_funs, lowername);
+        xbt_free(lowername);
         function(evt);
-        free(evt);
       } else {
         XBT_WARN("%s: Ignore trace element not for me",
               xbt_replay_reader_position(reader));
       }
+      free(evt);
     }
     xbt_replay_reader_free(&reader);
   }
@@ -165,7 +186,7 @@ int xbt_replay_action_runner(int argc, char *argv[])
 }
 
 
-static const char **action_get_action(char *name)
+static char **action_get_action(char *name)
 {
   xbt_dynar_t evt = NULL;
   char *evtname = NULL;
@@ -178,7 +199,7 @@ static const char **action_get_action(char *name)
     }
     // Read lines until I reach something for me (which breaks in loop body)
     // or end of file reached
-    while (getline(&action_line, &action_len, action_fp) != -1) {
+    while (xbt_getline(&action_line, &action_len, action_fp) != -1) {
       // cleanup and split the string I just read
       char *comment = strchr(action_line, '#');
       if (comment != NULL)
@@ -192,7 +213,7 @@ static const char **action_get_action(char *name)
 
       // if it's for me, I'm done
       evtname = xbt_dynar_get_as(evt, 0, char *);
-      if (!strcmp(name, evtname)) {
+      if (!strcasecmp(name, evtname)) {
         return xbt_dynar_to_array(evt);
       } else {
         // Else, I have to store it for the relevant colleague
