@@ -1,6 +1,6 @@
 /* xbt_str.c - various helping functions to deal with strings               */
 
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -305,14 +305,14 @@ xbt_dynar_t xbt_str_split_str(const char *s, const char *sep)
     //if substring was not found add the entire string
     if (NULL == q) {
       v = strlen(p);
-      to_push = malloc(v + 1);
+      to_push = xbt_malloc(v + 1);
       memcpy(to_push, p, v);
       to_push[v] = '\0';
       xbt_dynar_push(res, &to_push);
       done = 1;
     } else {
       //get the appearance
-      to_push = malloc(q - p + 1);
+      to_push = xbt_malloc(q - p + 1);
       memcpy(to_push, p, q - p);
       //add string terminator
       to_push[q - p] = '\0';
@@ -327,11 +327,14 @@ xbt_dynar_t xbt_str_split_str(const char *s, const char *sep)
  *
  * The string passed as argument must be writable (not const)
  * The elements of the dynar are just parts of the string passed as argument.
+ * So if you don't store that argument elsewhere, you should free it in addition
+ * to freeing the dynar. This can be done by simply freeing the first argument
+ * of the dynar:
+ *  free(xbt_dynar_get_ptr(dynar,0));
  *
- * To free the structure constructed by this function, free the first element and free the dynar:
- *
- * free(xbt_dynar_get_ptr(dynar,0));
- * xbt_dynar_free(&dynar);
+ * Actually this function puts a bunch of \0 in the memory area you passed as
+ * argument to separate the elements, and pushes the address of each chunk
+ * in the resulting dynar. Yes, that's uneven. Yes, that's gory. But that's efficient.
  */
 xbt_dynar_t xbt_str_split_quoted_in_place(char *s) {
   xbt_dynar_t res = xbt_dynar_new(sizeof(char *), NULL);
@@ -345,7 +348,7 @@ xbt_dynar_t xbt_str_split_quoted_in_place(char *s) {
 
   beg = s;
 
-  /* do not trim leading spaces: caller responsability to clean his cruft */
+  /* do not trim leading spaces: caller responsibility to clean his cruft */
   end = beg;
 
   while (!done) {
@@ -517,60 +520,53 @@ char *xbt_str_join_array(const char *const *strs, const char *sep)
   return res;
 }
 
-#if defined(SIMGRID_NEED_GETLINE) || defined(DOXYGEN)
 /** @brief Get a single line from the stream (reimplementation of the GNU getline)
  *
- * This is a redefinition of the GNU getline function, used on platforms where it does not exists.
+ * This is a reimplementation of the GNU getline function, so that our code don't depends on the GNU libc.
  *
- * getline() reads an entire line from stream, storing the address of the buffer
- * containing the text into *buf.  The buffer is null-terminated and includes
- * the newline character, if one was found.
+ * xbt_getline() reads an entire line from stream, storing the address of the
+ * buffer containing the text into *buf.  The buffer is null-terminated and
+ * includes the newline character, if one was found.
  *
- * If *buf is NULL, then getline() will allocate a buffer for storing the line,
- * which should be freed by the user program.  Alternatively, before calling getline(),
- * *buf can contain a pointer to a malloc()-allocated buffer *n bytes in size.  If the buffer
- * is not large enough to hold the line, getline() resizes it with realloc(), updating *buf and *n
- * as necessary.  In either case, on a successful call, *buf and *n will be updated to
- * reflect the buffer address and allocated size respectively.
+ * If *buf is NULL, then xbt_getline() will allocate a buffer for storing the
+ * line, which should be freed by the user program.
+ *
+ * Alternatively, before calling xbt_getline(), *buf can contain a pointer to a
+ * malloc()-allocated buffer *n bytes in size.  If the buffer is not large
+ * enough to hold the line, xbt_getline() resizes it with realloc(), updating
+ * *buf and *n as necessary.
+ *
+ * In either case, on a successful call, *buf and *n will be updated to reflect
+ * the buffer address and allocated size respectively.
  */
-long getline(char **buf, size_t * n, FILE * stream)
+ssize_t xbt_getline(char **buf, size_t *n, FILE *stream)
 {
-
-  size_t i;
+  ssize_t i;
   int ch;
 
+  ch = getc(stream);
+  if (ferror(stream) || feof(stream))
+    return -1;
+
   if (!*buf) {
-    *buf = xbt_malloc(512);
     *n = 512;
+    *buf = xbt_malloc(*n);
   }
 
-  if (feof(stream))
-    return (ssize_t) - 1;
-
-  for (i = 0; (ch = fgetc(stream)) != EOF; i++) {
-
-    if (i >= (*n) + 1)
+  i = 0;
+  do {
+    if (i == *n)
       *buf = xbt_realloc(*buf, *n += 512);
-
-    (*buf)[i] = ch;
-
-    if ((*buf)[i] == '\n') {
-      i++;
-      (*buf)[i] = '\0';
-      break;
-    }
-  }
+    (*buf)[i++] = ch;
+  } while (ch != '\n' && (ch = getc(stream)) != EOF);
 
   if (i == *n)
     *buf = xbt_realloc(*buf, *n += 1);
-
   (*buf)[i] = '\0';
 
-  return (ssize_t) i;
+  return i;
 }
 
-#endif                          /* HAVE_GETLINE */
-
 /*
  * Diff related functions
  *
