@@ -1,6 +1,6 @@
 /* xbt_modinter - How to init/exit the XBT modules                          */
 
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2010, 2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -26,11 +26,6 @@ void xbt_dict_postexit(void);
 void xbt_os_thread_mod_preinit(void);
 void xbt_os_thread_mod_postexit(void);
 
-void xbt_trp_preinit(void);
-void xbt_trp_postexit(void);
-void xbt_datadesc_preinit(void);
-void xbt_datadesc_postexit(void);
-
 void *mmalloc_preinit(void);
 void mmalloc_postexit(void);
 
