@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2009-2010, 2012-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -18,7 +18,7 @@ int test_trace(int argc, char *argv[]);
 /** test the trace integration cpu model */
 int test_trace(int argc, char *argv[])
 {
-  m_task_t task;
+  msg_task_t task;
   double task_comp_size = 2800;
   double task_prio = 1.0;
 
@@ -32,8 +32,8 @@ int test_trace(int argc, char *argv[])
   task_prio = atof(argv[2]);
 
   XBT_INFO("Testing the trace integration cpu model: CpuTI");
-  XBT_INFO("Task size: %lf", task_comp_size);
-  XBT_INFO("Task prio: %lf", task_prio);
+  XBT_INFO("Task size: %f", task_comp_size);
+  XBT_INFO("Task prio: %f", task_prio);
 
   /* Create and execute a single task. */
   task = MSG_task_create("proc 0", task_comp_size, 0, NULL);
@@ -50,10 +50,10 @@ int test_trace(int argc, char *argv[])
 /** Main function */
 int main(int argc, char *argv[])
 {
-  MSG_error_t res = MSG_OK;
+  msg_error_t res = MSG_OK;
 
   /* Verify if the platform xml file was passed by command line. */
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
   if (argc < 2) {
     printf("Usage: %s test_trace_integration_model.xml\n", argv[0]);
     exit(1);
@@ -63,12 +63,10 @@ int main(int argc, char *argv[])
   MSG_function_register("test_trace", test_trace);
   /* Use the same file for platform and deployment. */
   MSG_create_environment(argv[1]);
-  MSG_launch_application(argv[1]);
+  MSG_launch_application(argv[2]);
   /* Run the example. */
   res = MSG_main();
 
-  MSG_clean();
-
   if (res == MSG_OK)
     return 0;
   else
