@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2008-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -24,18 +24,18 @@ int main(int argc, char **argv)
 {
   int res;
   xbt_dynar_t all_hosts;
-  m_host_t first_host;
-  MSG_global_init(&argc, argv);
+  msg_host_t first_host;
+  MSG_init(&argc, argv);
   MSG_create_environment(argv[1]);
   MSG_function_register("host", host);
   all_hosts = MSG_hosts_as_dynar();
-  first_host = xbt_dynar_pop_as(all_hosts,m_host_t);
+  first_host = xbt_dynar_pop_as(all_hosts,msg_host_t);
   MSG_process_create( "host", host, NULL, first_host);
   xbt_dynar_free(&all_hosts);
 
   res = MSG_main();
   XBT_INFO("Simulation time %g", MSG_get_clock());
-  MSG_clean();
+
   if (res == MSG_OK)
     return 0;
   else
