@@ -5,7 +5,7 @@
 int main(int argc, char **argv) {
 
   int rank, i, j;
-  double a[SIZE][SIZE];
+  double a[SIZE][SIZE] = {{0}};
 
   MPI_Datatype columntype;
 
@@ -35,7 +35,7 @@ int main(int argc, char **argv) {
     }
 
 
-
+  MPI_Type_free(&columntype);
   MPI_Finalize();
   return 0;
 }
