@@ -1,13 +1,12 @@
 /* log_large_test -- log a very large string to test the dynamic variants   */
 
-/* Copyright (c) 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2007-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
  * under the terms of the license (GNU LGPL) which comes with this package. */
 
-#include "gras.h"
-#include "xbt/synchro.h"
+#include "xbt.h"
 
 XBT_LOG_NEW_DEFAULT_CATEGORY(test, "Logs of this example");
 
