@@ -1,6 +1,6 @@
 /* A crash few tests for the maxmin library                                 */
 
-/* Copyright (c) 2004, 2005, 2006, 2007, 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -11,12 +11,14 @@
 #endif
 
 
-#include <stdlib.h>
-#include <stdio.h>
 #include "surf/maxmin.h"
+#include "xbt/module.h"
 #include "xbt/xbt_os_time.h"
 #include "xbt/sysdep.h"         /* time manipulation for benchmarking */
 
+#include <stdlib.h>
+#include <stdio.h>
+
 double date;
 
 double float_random(double max);
@@ -84,6 +86,7 @@ int main(int argc, char **argv)
   int nb_cnst = 2000;
   int nb_var = 2000;
   int nb_elem = 80;
+  xbt_init(&argc, argv);
   date = xbt_os_time() * 1000000;
   test(nb_cnst, nb_var, nb_elem);
   printf("One shot execution time for a total of %d constraints, "
