@@ -1,6 +1,6 @@
 /* A few tests for the trace library                                       */
 
-/* Copyright (c) 2004, 2005, 2006, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2006, 2009-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -9,13 +9,15 @@
 #pragma hdrstop
 #endif
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include "surf/trace_mgr.h"
 #include "surf/surf.h"
 
 #include "xbt/log.h"
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+
 XBT_LOG_NEW_DEFAULT_CATEGORY(surf_test,
                              "Messages specific for surf example");
 
@@ -23,8 +25,8 @@ void test(void);
 void test(void)
 {
   tmgr_history_t history = tmgr_history_new();
-  tmgr_trace_t trace_A = tmgr_trace_new("trace_A.txt");
-  tmgr_trace_t trace_B = tmgr_trace_new("trace_B.txt");
+  tmgr_trace_t trace_A = tmgr_trace_new_from_file("trace_A.txt");
+  tmgr_trace_t trace_B = tmgr_trace_new_from_file("trace_B.txt");
   double next_event_date = -1.0;
   double value = -1.0;
   char *resource = NULL;
