@@ -1,6 +1,6 @@
 /* log_usage - A test of normal usage of the log facilities                 */
 
-/* Copyright (c) 2004, 2005, 2006, 2007, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2004-2007, 2009-2013. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -10,9 +10,7 @@
 #pragma hdrstop
 #endif
 
-#include "gras.h"
-
-
+#include "xbt.h"
 
 XBT_LOG_NEW_DEFAULT_SUBCATEGORY(test, top,
                                 "Logging specific to this test");
@@ -40,7 +38,9 @@ int main(int argc, char **argv)
   dolog("");
   dolog(" ");
   dolog(" test.thres:info root.thres:info  ");
+#ifndef NDEBUG
   dolog(" test.thres:debug ");
+#endif
   dolog(" test.thres:verbose root.thres:error ");
   dolog(" test.thres:critical ");
 
