@@ -3,7 +3,8 @@
 #include <xbt/dynar.h>
 #include <xbt/parmap.h>
 #include <xbt/sysdep.h>
-#include <gras_config.h>        /* HAVE_FUTEX_H */
+#include <internal_config.h>        /* HAVE_FUTEX_H */
+#include "simgrid/simix.h"
 #include "xbt/xbt_os_time.h"
 
 #define MODES_DEFAULT 0x7
@@ -166,6 +167,8 @@ int main(int argc, char *argv[])
   int nthreads;
   unsigned modes = MODES_DEFAULT;
 
+  SIMIX_global_init(&argc, argv);
+
   if (argc != 2 && argc != 3) {
     fprintf(stderr,
             "Usage: %s nthreads [modes]\n"
