@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2009, 2010. The SimGrid Team.
+/* Copyright (c) 2008-2012. The SimGrid Team.
  * All rights reserved.                                                     */
 
 /* This program is free software; you can redistribute it and/or modify it
@@ -18,7 +18,7 @@ int main(int argc, char **argv)
 {
   XBT_LOG_CONNECT(graphicator);
 #ifdef HAVE_TRACING
-  MSG_global_init(&argc, argv);
+  MSG_init(&argc, argv);
 
   if (argc < 3){
     XBT_INFO("Usage: %s <platform_file.xml> <graphviz_file.dot>", argv[0]);
@@ -31,9 +31,8 @@ int main(int argc, char **argv)
 
   int status = TRACE_platform_graph_export_graphviz (graphvizFile);
   if (status == 0){
-    XBT_INFO ("%s expects --cfg=tracing:1 --cfg=tracing/platform:1", argv[0]);
+    XBT_INFO ("%s expects --cfg=tracing:yes --cfg=tracing/platform:yes", argv[0]);
   }
-  MSG_clean();
 #else
   XBT_INFO ("works only if simgrid was compiled with tracing enabled.");
 #endif
