@@ -5,11 +5,12 @@ eval 'exec perl -S $0 ${1+"$@"}'
 # This script updates the simgrid XML file passed as argument (modification in place)
 # It is built to do the conversion incrementally.
 
-# Copyright (c) 2006-2016. The SimGrid Team.
+# Copyright (c) 2006-2012. The SimGrid Team.
 # All rights reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the license (GNU LGPL) which comes with this package.
+# This file is part of the SimGrid project. This is free software:
+# You can redistribute and/or modify it under the terms of the
+# GNU LGPL (v2.1) licence.
 
 =encoding UTF-8
 
@@ -28,7 +29,7 @@ is modified in place, without any kind of backup. You may want to save a copy
 before running the script.
 
 In SimGrid XML files, the standard version is indicated in the version
-attribute of the platform tag. Current version is 4. Here is a list of major
+attribute of the platform tag. Current version is 3. Here is a list of major
 changes in each version.
 
 =over 4
@@ -41,7 +42,7 @@ changes in each version.
 
 =item 
 
-The version attribute of platform were added to allow file versioning.
+The version attribute of platform were added to allow file versionning.
 
 =item
 
@@ -65,13 +66,13 @@ Several tags were renamed:
 
 =back
 
-=item B<Version 3:> Introduced in SimGrid 3.5
+=item B<Version 3:> Introduced in SimGrid 3.5 (this is the current version)
 
 =over 4
 
 =item
 
-The AS tag were introduced. Every platform should now contain an englobing AS
+The AS tag were introduced. Every plaform should now contain an englobing AS
 tag.
 
 =item 
@@ -87,42 +88,6 @@ Several tags were renamed (for sake of XML sanity):
 
 =back
 
-=item B<Version 4:> Introduced in SimGrid 3.13 (this is the current version)
-
-=over 4
-
-=item
-
-Rename the attributes describing the amount of flop that a host / peer / cluster / cabinet can deliver per second.
-
-  <host power=...> -> <host speed=...>
-
-=item
-
-In <trace_connect>, attribute kind="POWER" is now kind="SPEED".
-
-=item
-
-The DOCTYPE points to the right URL: http://simgrid.gforge.inria.fr/simgrid/simgrid.dtd
-
-=item
-
-Units are now mandatory in attributes. USE THE SCRIPT sg_xml_unit_converter.py TO CONVERT THIS
-
-     - speed. Old default: 'f' or 'flops'. Also defined: 
-        'Yf',         'Zf',         'Ef',       'Pf',        'Tf',        'Gf',        'Mf',        'kf' 
-        'yottaflops', 'zettaflops', 'exaflops', 'petaflops', 'teraflops', 'gigaflops', 'megaflops', 'kiloflops'
-        
-     - bandwidth. Old default: 'Bps' bytes per second (or 'bps' but 1 Bps = 8 bps)
-       Also defined in bytes: 'TiBps', 'GiBps', 'MiBps', 'KiBps', 'TBps', 'GBps', 'MBps', 'kBps', 'Bps'
-       And the same in bits:  'Tibps', 'Gibps', 'Mibps', 'Kibps', 'Tbps', 'Gbps', 'Mbps', 'kbps', 'bps' 
-       
-     - latency. Old default: 's' second. Also defined:
-       'w' week, 'd' day, 'h' hour, 'm' minute, 'ms' millisecond, 'us' microsecond, 'ns' nanosecond, 'ps' picosecond   
-
-
-=back
-
 =back
 
 =head1 AUTHORS
@@ -131,7 +96,7 @@ Units are now mandatory in attributes. USE THE SCRIPT sg_xml_unit_converter.py T
   
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006-2016. The SimGrid Team. All rights reserved.
+Copyright (c) 2006-2012. The SimGrid Team. All rights reserved.
   
 This program is free software; you may redistribute it and/or modify it
 under the terms of GNU LGPL (v2.1) license.
@@ -142,15 +107,18 @@ under the terms of GNU LGPL (v2.1) license.
 use strict;
 
 my $fromversion=-1;
-my $toversion=4;
+my $toversion=3;
 
-my $filename = $ARGV[0] or die "Usage: simgrid_update_xml.pl file_to_convert.xml\nPlease provide an XML to convert as a parameter.\n";
-open INPUT, "$filename" or die "Cannot open input file $filename: $!\n";
+my($output_string);
 
-my $output_string = "<?xml version='1.0'?>\n".
-    "<!DOCTYPE platform SYSTEM \"http://simgrid.gforge.inria.fr/simgrid/simgrid.dtd\">\n".
+$ARGV[0] or die "simgrid_update_xml.pl <platform.xml>\n";
+open INPUT, "$ARGV[0]" or die "Cannot open input file $ARGV[0]: $!\n";
+
+$output_string = "<?xml version='1.0'?>\n".
+    "<!DOCTYPE platform SYSTEM \"http://simgrid.gforge.inria.fr/simgrid.dtd\">\n".
     "<platform version=\"$toversion\">\n";
 
+
 my($AS_opened)=0;
 
 my $line;
@@ -162,18 +130,18 @@ while (defined($line = <INPUT>)) {
     
     if ($line =~ s/<platform(_description)? *>//) {
 	$fromversion = 0;
-	print "$filename was using version 0\n";
+	print "version 0\n";
 	next if !$line =~ /\S/;
-    } elsif ($line =~ s/<platform.*version=["']*([0-9.])["']*>//) {
+    } elsif ($line =~ s/<platform.*version=["]*([0-9.])["]*>//) {
 	$fromversion = $1;
+	print "version $fromversion\n";
 	if ($fromversion == $toversion) {
-	    die "Input platform file $filename is already conformant to version $fromversion. This should be a no-op.\n";
+	    die "Input platform file version is already $fromversion. This should be a no-op.\n";
 	}
 	if ($fromversion > $toversion) {
-	    die "Input platform file $filename is more recent than this script (file version: $fromversion; script version: $toversion)\n";
+	    die "Input platform file version is more recent than this script (file version: $fromversion; script version: $toversion)\n";
 	}
 	next if !$line =~ /\S/;
-	print "$filename was using version $fromversion\n";
     }
     
     if ($fromversion == 0) {
@@ -218,23 +186,18 @@ while (defined($line = <INPUT>)) {
 	    $output_string .=  " <AS  id=\"AS0\"  routing=\"Full\">\n";
 	    $AS_opened=1;
 	}
+    }
 	
 	if($line=~/<route /){$line =~ s/\<route/\<route symmetrical=\"NO\"/g;}
-    }
-    if ($fromversion < 4) {
-	$line =~ s/\bpower\b/speed/g;	
-	$line =~ s/\bkind="POWER"/kind="SPEED"/g;
-    }
-	
     $output_string .= "$line\n";
 }
 
 close INPUT;
 
 if ($fromversion == -1) {
-    die "Cannot retrieve the platform version of $filename\n";
+    die "Cannot retrieve the platform version\n";
 }
 
-open OUTPUT, "> $filename";
+open OUTPUT, "> $ARGV[0]";
 print OUTPUT $output_string;
 close OUTPUT;
